<?php

namespace Pg\Modules\Countries\Controllers;

/**
 * Countries module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */

/**
 * Countries api controller
 *
 * @package 	PG_RealEstate
 * @subpackage 	Countries
 *
 * @category	controllers
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Api_Countries extends \Controller
{

    /**
     * Class constructor
     *
     * @return Api_Countries
     */
    public function __construct()
    {
        parent::__construct();
        $this->load->model('Countries_model');
    }

    /**
     * Get locations
     *
     * @param string  $name    location name (post)
     * @param string  $country country code (post)
     * @param integer $region  region identifier (post)
     *
     * @return void
     */
    public function get_locations()
    {
        $name = $this->input->post('name', true);
        $country_input = $this->input->post('country', true);
        $region_input = $this->input->post('region', true);
        $city_input = $this->input->post('city', true);
        $data = array();
        if ($name) {
            $locations = $this->Countries_model->get_locations($name, array('priority' => 'ASC'), $this->pg_language->current_lang_id, $this->pg_language->languages, $country_input, $region_input, $city_input);
            $data['count'] = count($locations['countries']) + count($locations['regions']) + count($locations['cities']) + count($locations['districts']);

            $locations_ids = array();

            if (!empty($locations['countries'])) {
                foreach ($locations['countries'] as $i => $country) {
                    $locations_ids['country_' . $country['code']] = array(
                        'country' => $country['code'],
                    );
                }
            }

            if (!empty($locations['regions'])) {
                foreach ($locations['regions'] as $i => $region) {
                    $locations_ids['region_' . $region['id']] = array(
                        'region' => $region['id'],
                    );
                    if (!$country_input) {
                        $locations_ids['region_' . $region['id']]['country'] = $region['country_code'];
                    }
                }
            }

            if (!empty($locations['cities'])) {
                foreach ($locations['cities'] as $i => $city) {
                    $locations_ids['city_' . $city['id']] = array(
                        'city' => $city['id'],
                    );
                    if (!$country_input) {
                        $locations_ids['city_' . $city['id']]['country'] = $city['country_code'];
                    }
                    if (!$region_input) {
                        $locations_ids['city_' . $city['id']]['region'] = $city['id_region'];
                    }
                }
            }

            if (!empty($locations['districts'])) {
                foreach ($locations['districts'] as $i => $district) {
                    $locations_ids['district_' . $district['id']] = array(
                        'district' => $district['id'],
                    );
                    if (!$country_input) {
                        $locations_ids['district_' . $district['id']]['country'] = $district['country_code'];
                    }
                    if (!$region_input) {
                        $locations_ids['district_' . $district['id']]['region'] = $district['id_region'];
                    }
                    if (!$city_input) {
                        $locations_ids['district_' . $district['id']]['city'] = $district['id_city'];
                    }
                }
            }

            $this->load->helper('countries');
            $locations_data = address_output_format($locations_ids, 'district');

            if (!empty($locations['countries'])) {
                foreach ($locations['countries'] as $i => $country) {
                    $locations['countries'][$i]['fullname'] = $locations_data['country_' . $country['code']];
                }
            }

            if (!empty($locations['regions'])) {
                foreach ($locations['regions'] as $i => $region) {
                    $locations['regions'][$i]['fullname'] = $locations_data['region_' . $region['id']];
                }
            }

            if (!empty($locations['cities'])) {
                foreach ($locations['cities'] as $i => $city) {
                    $locations['cities'][$i]['fullname'] = $locations_data['city_' . $city['id']];
                }
            }

            if (!empty($locations['districts'])) {
                foreach ($locations['districts'] as $i => $district) {
                    $locations['districts'][$i]['fullname'] = $locations_data['district_' . $district['id']];
                }
            }

            $data['items'] = $locations ? $locations : array();

            $this->set_api_content('data', array('name' => $name, 'data' => $data));
        } else {
            $this->set_api_content('error', l('error_name_empty', 'countries'));
        }
    }

    /**
     * Get countries
     *
     * @return void
     */
    public function get_countries()
    {
        $data = array();
        $countries = $this->Countries_model->get_countries(array('priority' => 'ASC'), array(), array(), $this->pg_language->current_lang_id);
        $data['count'] = count($countries);
        if ($data['count']) {
            foreach ($countries as $country) {
                $data['items'][] = array(
                    'id' => $country['id'],
                    'code' => $country['code'],
                    'name' => $country['name'],
                );
            }
            $this->set_api_content('data', array('countries' => $data));
        } else {
            $this->set_api_content('messages', l('no_countries', 'countries'));
        }
    }

    /**
     * Returns regions by country
     *
     * @param string $country_code
     */
    public function get_regions()
    {
        $data = array();
        $data['count'] = 0;
        $data['items'] = array();

        $country_code = $this->input->post('country_code', true);
        if (!$country_code) {
            log_message('error', 'realestate API: Empty country code');
            $this->set_api_content('error', l('api_error_empty_country_code', 'countries'));

            return false;
        }
        $data['country'] = $this->Countries_model->get_country($country_code, $this->pg_language->current_lang_id);
        if (!$data['country']) {
            log_message('error', 'realestate API: Wrong country code ("' . $country_code . '")');
            $this->set_api_content('error', l('api_error_wrong_country_code', 'countries') . '("' . $country_code . '")');

            return false;
        }

        $regions = $this->Countries_model->get_regions($country_code, array('priority' => 'ASC'), array(), array(), $this->pg_language->current_lang_id);

        $data['count'] = count($regions);

        foreach ($regions as $region) {
            $data['items'][] = array(
                'id' => $region['id'],
                'code' => $region['code'],
                'name' => $region['name'],
            );
        }

        $this->set_api_content('data', array('country_code' => $country_code, 'regions' => $data));
    }

    /**
     * Returns cities by region
     *
     * @param int    $region_id
     * @param string $search
     * @param int    $page
     */
    public function get_cities()
    {
        $cities = array();
        $params = array();
        $items_on_page = 400;

        $region_id = $this->input->post('region_id', true);
        if (!$region_id) {
            log_message('error', 'realestate API: Empty region id');
            $this->set_api_content('error', l('api_error_empty_region_id', 'countries'));

            return false;
        }

        $cities['region'] = $this->Countries_model->get_region($region_id, $this->pg_language->current_lang_id);
        if (!$cities['region']) {
            log_message('error', 'realestate API: Wrong region id ("' . $region_id . '")');
            $this->set_api_content('error', l('api_error_wrong_region_id', 'countries') . '("' . $region_id . '")');

            return false;
        }

        $cities['country'] = $this->Countries_model->get_country($cities['region']['country_code'], $this->pg_language->current_lang_id);
        if (!$cities['country']) {
            log_message('error', 'realestate API: Wrong country code ("' . $cities['region']['country_code'] . '")');
            $this->set_api_content('error', l('api_error_wrong_country_code', 'countries') . '"(' . $cities['region']['country_code'] . ')"');
        }

        $search_string = trim(strip_tags($this->input->post('search', true)));

        $params['where']['id_region'] = $region_id;
        if (!empty($search_string)) {
            if ($this->pg_language->current_lang_id) {
                $var = 'lang_' . $this->pg_language->current_lang_id;
            } else {
                $var = 'name';
            }
            $params['where'][$var . ' LIKE'] = '%' . $search_string . '%';
        }
        $cities['count'] = $this->Countries_model->get_cities_count($params);
        $page = $this->input->post('page', true);
        $this->load->helper('sort_order');
        $page = get_exists_page_number($page, $cities['count'], $items_on_page);

        $cities['use_district'] = $this->pg_module->get_module_config('countries', 'use_district');
        $cities['pages'] = ceil($cities['count'] / $items_on_page);
        $cities['current_page'] = $page;
        $cities['items'] = $this->Countries_model->get_cities($page, $items_on_page, array(), $params, array(), $this->pg_language->current_lang_id);
        $this->set_api_content('data', $cities);
    }

    /**
     * Returns location data
     *
     * @param string     $type country/region/city
     * @param int|string $var  Country code, region id or city id
     */
    public function get_data()
    {
        $type = $this->input->post('type', true);
        if (!$type) {
            log_message('error', 'realestate API: Empty location type');
            $this->set_api_content('error', l('api_error_empty_location_type', 'countries'));

            return false;
        }
        $var = $this->input->post('var', true);
        if (!$var) {
            log_message('error', 'realestate API: Empty location');
            $this->set_api_content('error', l('api_error_empty_location', 'countries'));

            return false;
        }
        $data = array();
        switch ($type) {
            case 'country':
                $data['country'] = $this->Countries_model->get_country($var, $this->pg_language->current_lang_id);
                break;
            case 'region':
                $data['region'] = $this->Countries_model->get_region($var, $this->pg_language->current_lang_id);
                $data['country'] = $this->Countries_model->get_country($data['region']['country_code'], $this->pg_language->current_lang_id);
                break;
            case 'city':
                $data['city'] = $this->Countries_model->get_city($var, $this->pg_language->current_lang_id);
                $data['region'] = $this->Countries_model->get_region($data['city']['id_region'], $this->pg_language->current_lang_id);
                $data['country'] = $this->Countries_model->get_country($data['city']['country_code'], $this->pg_language->current_lang_id);
                break;
        }
        $this->set_api_content('data', $data);
    }

    /**
     * Returns districts by city
     *
     * @param integer $city_id city identifier
     * @param string  $search  search string
     * @param integer $page    page of results
     *
     * @return void
     */
    public function get_districts()
    {
        $districts = array();
        $params = array();
        $items_on_page = 400;

        $city_id = $this->input->post('city_id', true);
        if (!$city_id) {
            log_message('error', 'realestate API: Empty city id');
            $this->set_api_content('error', l('api_error_empty_city_id', 'countries'));

            return false;
        }

        $districts['city'] = $this->Countries_model->get_city($city_id, $this->pg_language->current_lang_id);
        if (!$districts['city']) {
            log_message('error', 'realestate API: Wrong city id ("' . $city_id . '")');
            $this->set_api_content('error', l('api_error_wrong_city_id', 'countries') . '("' . $city_id . '")');

            return false;
        }

        $districts['region'] = $this->Countries_model->get_region($districts['city']['id_region'], $this->pg_language->current_lang_id);
        if (!$districts['region']) {
            log_message('error', 'realestate API: Wrong region identifier ("' . $districts['city']['id_region'] . '")');
            $this->set_api_content('error', l('api_error_wrong_region_id', 'countries') . '"(' . $districts['city']['id_region'] . ')"');
        }

        $districts['country'] = $this->Countries_model->get_country($districts['city']['country_code'], $this->pg_language->current_lang_id);
        if (!$districts['country']) {
            log_message('error', 'realestate API: Wrong country code ("' . $districts['city']['country_code'] . '")');
            $this->set_api_content('error', l('api_error_wrong_country_code', 'countries') . '"(' . $districts['city']['country_code'] . ')"');
        }

        $search_string = trim(strip_tags($this->input->post('search', true)));

        $params['where']['id_city'] = $city_id;
        if (!empty($search_string)) {
            if ($this->pg_language->current_lang_id) {
                $var = 'lang_' . $this->pg_language->current_lang_id;
            } else {
                $var = 'name';
            }
            $params['where'][$var . ' LIKE'] = '%' . $search_string . '%';
        }
        $districts['count'] = $this->Countries_model->get_districts_count($params);
        $page = $this->input->post('page', true);
        $this->load->helper('sort_order');
        $page = get_exists_page_number($page, $districts['count'], $items_on_page);

        $districts['pages'] = ceil($districts['count'] / $items_on_page);
        $districts['current_page'] = $page;
        $districts['items'] = $this->Countries_model->get_districts($page, $items_on_page, array(), $params, array(), $this->pg_language->current_lang_id);
        $this->set_api_content('data', $districts);
    }

}
