<?php

namespace Pg\Modules\Import\Controllers;

/**
 * Import module
 *
 * @package     PG_RealEstate
 *
 * @copyright   Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author      Pilot Group Ltd <http://www.pilotgroup.net/>
 */
use Pg\Libraries\View;

/**
 * Import admin side controller
 *
 * @package PG_RealEstate
 * @subpackage Import
 *
 * @category    controllers
 *
 * @copyright   Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author      Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Admin_Import extends \Controller
{

    /**
     * Constructor
     *
     * @return Admin_Import
     */
    public function __construct()
    {
        parent::__construct();
        $this->load->model("Menu_model");
        $this->Menu_model->set_menu_active_item("admin_menu", "exp-import-items");
        $this->view->setHeader(l("admin_header_import", "import"));
    }

    /**
     * Selections mamagement
     *
     * @return void
     */
    public function index()
    {
        $this->data();
    }

    /**
     * Manage selections
     *
     * @param string  $order           sorting data
     * @param string  $order_direction order direction
     * @param integer $page            page of results
     *
     * @return void
     */
    public function data($order = "date_created", $order_direction = "DESC", $page = 1)
    {
        $this->load->model("Import_model");
        $this->load->model("import/models/Import_driver_model");
        $this->load->model("import/models/Import_module_model");

        $filters = array();

        $current_settings = isset($_SESSION["import_list"]) ? $_SESSION["import_list"] : array();

        $current_settings["filters"] = $filters;

        if (!isset($current_settings["order"])) {
            $current_settings["order"] = "date_add";
        }
        if (!isset($current_settings["order_direction"])) {
            $current_settings["order_direction"] = "DESC";
        }
        if (!isset($current_settings["page"])) {
            $current_settings["page"] = 1;
        }

        if (!$order) {
            $order = $current_settings["order"];
        }
        $this->view->assign("order", $order);
        $current_settings["order"] = $order;

        if (!$order_direction) {
            $order_direction = $current_settings["order_direction"];
        }
        $this->view->assign("order_direction", $order_direction);
        $current_settings["order_direction"] = $order_direction;

        $data_count = $this->Import_model->get_data_count($filters);

        if (!$page) {
            $page = $current_settings["page"];
        }
        $items_on_page = $this->pg_module->get_module_config("start", "admin_items_per_page");

        $this->load->helper("sort_order");
        $page = get_exists_page_number($page, $data_count, $items_on_page);
        $current_settings["page"] = $page;

        $_SESSION["import_list"] = $current_settings;

        $sort_links = array(
            "date_created" => site_url() . "admin/import/data/date_created/" . (($order != "date_created" xor $order_direction == "DESC") ? "ASC" : "DESC"),
        );
        $this->view->assign("sort_links", $sort_links);

        if ($data_count > 0) {
            $data = $this->Import_model->get_data_list($filters, $page, $items_on_page,
                                                       array($order => $order_direction));
            $this->view->assign("data", $data);
        }

        $this->load->helper('navigation');

        $url = site_url() . "admin/import/data/" . $order . "/" . $order_direction . "/";
        $page_data = get_admin_pages_data($url, $data_count, $items_on_page, $page, "briefPage");
        $page_data["date_format"] = $this->pg_date->get_format('date_time_literal', 'st');
        $this->view->assign("page_data", $page_data);

        $drivers = $this->Import_driver_model->get_drivers();
        foreach ($drivers as $key => $driver) {
            if (!$driver["status"]) {
                unset($drivers[$key]);
            }
        }
        $this->view->assign("is_available", count($drivers) > 0);

        $this->Menu_model->set_menu_active_item("admin_import_menu", "import_data_item");
        $this->view->render("list");
    }

    /**
     * Upload data for importing
     *
     * @param integer $import_id import identifier
     *
     * @return void
     */
    public function upload($import_id = null)
    {
        $this->load->model("Import_model");
        $this->load->model("import/models/Import_driver_model");

        if (!$import_id) {
            $import_id = $this->input->post('id', true);
        }

        if ($import_id) {
            $this->Import_model->set_format_settings(array('get_driver' => true, 'get_module' => true));
            $data = $this->Import_model->get_data_by_id($import_id, true);
            $this->Import_model->set_format_settings(array('get_driver' => false, 'get_module' => false));
        } else {
            $data = array();
        }

        if ($this->input->post("btn_save")) {
            $post_data = $this->input->post("data");

            if ($import_id) {
                $post_data['gid_driver'] = $data['gid_driver'];
                $post_data['id_object'] = $data['id_object'];
            }

            $validate_data = $this->Import_model->validate_data($import_id, $post_data);
            if (!empty($validate_data["errors"])) {
                $this->system_messages->addMessage(View::MSG_ERROR, $validate_data["errors"]);
            } else {
                $data_id = $this->Import_model->save_data($import_id, $validate_data["data"]);

                $this->load->model('import/models/Import_driver_model');
                $driver_data = $this->Import_driver_model->get_driver_by_gid($validate_data['data']['gid_driver']);

                $model_name = "import_" . $validate_data['data']['gid_driver'] . "_model";
                $this->load->model("import/models/drivers/" . $model_name, $model_name);

                $settings = (array) unserialize($validate_data['data']['settings']);

                if ($this->{$model_name}->need_upload) {
                    $upload_data = $this->Import_model->upload('import_file', $data_id);
                    $filename = $upload_data['data']['filename'];
                } else {
                    $filename = '';
                }

                $analyze_data = $this->{$model_name}->analyze($driver_data['settings'], $settings, $filename);
                if (!empty($analyze_data["errors"])) {
                    $this->system_messages->addMessage(View::MSG_ERROR, $analyze_data["errors"]);
                } else {
                    $this->Import_model->save_data($data_id, $analyze_data["data"]);

                    if (!empty($analyze_data['filename'])) {
                        $this->system_messages->addMessage(View::MSG_SUCCESS, l("success_file_uploaded", "import"));
                    } else {
                        $this->system_messages->addMessage(View::MSG_SUCCESS, l("success_selection_saved", "import"));
                    }

                    if ($import_id) {
                        $url = site_url() . "admin/import/edit/" . $data_id;
                    } else {
                        $url = site_url() . "admin/import/relations/" . $data_id;
                    }
                    redirect($url);
                }
            }
        }

        $this->view->assign('data', $data);

        $drivers = $this->Import_driver_model->get_drivers();
        foreach ($drivers as $key => $driver) {
            if (!$driver["status"]) {
                unset($drivers[$key]);
            }
        }
        $this->view->assign("drivers", $drivers);

        $this->load->model("import/models/Import_module_model");
        $modules = $this->Import_module_model->get_modules();
        $this->view->assign("modules", $modules);

        if (!empty($drivers)) {
            $current_driver = current($drivers);
            if (isset($data['gid_driver'])) {
                foreach ($drivers as $driver) {
                    if ($driver['gid'] = $data['gid_driver']) {
                        $current_driver = $driver;
                        break;
                    }
                }
            }

            $model_name = "import_" . $current_driver["gid"] . "_model";
            $this->load->model("import/models/drivers/" . $model_name, $model_name);
            $this->view->assign("driver_gid", $current_driver["gid"]);

            $settings = $this->view->fetch("upload_form_settings", "admin", "import");
            $this->view->assign("settings", $settings);
        }
        $this->view->render("upload_form");
    }

    /**
     * Get upload form by ajax
     *
     * @return void
     */
    public function ajax_get_settings()
    {
        $data = (array) $this->input->post("data", true);

        if (empty($data['gid_driver'])) {
            show_404();
        }

        $import_id = $this->input->post("id", true);
        if ($import_id) {
            $import_data = $this->Import_model->get_data_by_id($import_id);
            $impotr_data['settings'] = $data['settings'];
        } else {
            $import_data = $data;
            if (empty($import_data['settings'])) {
                $import_data['settings'] = array();
            }
        }

        $this->load->model('import/models/Import_driver_model');
        $driver_data = $this->Import_driver_model->get_driver_by_gid($import_data['gid_driver']);
        $this->view->assign('driver_data', $driver_data);

        if (empty($driver_data)) {
            show_404();
        }

        $model_name = "Import_" . $driver_data['gid'] . "_model";
        $this->load->model("import/models/drivers/" . $model_name, $model_name);

        $driver_settings = $this->{$model_name}->get_settings($driver_data['settings'], $import_data['settings']);
        $this->view->assign('driver_settings', $driver_settings);

        $this->view->assign('data', $import_data);

        $page_data = array();
        $page_data['date_format'] = $this->pg_date->get_format('date_literal', 'st');
        $page_data['date_time_format'] = $this->pg_date->get_format('date_time_literal', 'st');
        $page_data['datepicker_date_format'] = $this->pg_date->get_format('date_literal', 'ui');
        $page_data['datepicker_alt_format'] = $this->pg_date->get_format('date_numeric', 'ui');
        $this->view->assign('page_data', $page_data);

        $settings = $this->view->fetch($driver_data['gid'] . '_settings', 'admin', 'import');

        $input_types = array();

        foreach ($driver_data['input_types'] as $input_type) {
            $input_types[$input_type] = l('field_input_type_' . $input_type, 'import');
        }

        $results = array(
            'is_upload' => $driver_data['is_upload'],
            'is_integration' => $driver_data['is_integration'],
            'settings' => $settings,
            'input_types' => $input_types,
        );

        $this->view->assign($results);

        $this->view->render();
    }

    /**
     * Edit selection data
     *
     * @param integer $data_id     data identifier
     * @param string  $section_gid section guid
     * @param integer $page        page number
     *
     * @return void
     */
    public function edit($import_id = null, $section_gid = 'general', $page = 1)
    {
        $this->load->model("import/models/Import_module_model");
        $this->load->model("Import_model");

        if ($import_id) {
            $this->Import_model->set_format_settings(array('get_driver' => true, 'get_module' => true));
            $data = $this->Import_model->get_data_by_id($import_id, true);
            $this->Import_model->set_format_settings(array('get_driver' => false, 'get_module' => false));
            if (empty($data)) {
                show_404();
            }
        } else {
            $section_gid = 'general';
            $data = array();
        }

        switch ($section_gid) {
            case 'general':
                if ($this->input->post("btn_save")) {
                    $post_data = $this->input->post("data");
                    if (empty($post_data['settings'])) {
                        $post_data['settings'] = array();
                    }

                    $scheduler = (array) $this->input->post("scheduler", true);
                    if (isset($post_data["scheduler"]["type"])) {
                        if (isset($scheduler[$post_data["scheduler"]["type"]])) {
                            $post_data["scheduler"] = array_merge($post_data["scheduler"],
                                                                  $scheduler[$post_data["scheduler"]["type"]]);
                        }
                    }

                    $datepicker = (array) $this->input->post('datepicker', true);
                    foreach ($datepicker as $key => $value) {
                        if (isset($key)) {
                            $post_data['settings'][$key] = $value;
                        }
                    }

                    if ($import_id) {
                        $post_data['gid_driver'] = $data['gid_driver'];
                        $post_data['id_object'] = $data['id_object'];
                        $post_data['settings'] = array_merge($data['settings'], $post_data['settings']);
                    }

                    $validate_data = $this->Import_model->validate_data($import_id, $post_data);
                    if (!empty($validate_data["errors"])) {
                        $this->system_messages->addMessage(View::MSG_ERROR, $validate_data["errors"]);
                    } else {
                        if (!$import_id) {
                            $relations = '/relations';
                        } else {
                            $relations = '';
                        }

                        $import_id = $this->Import_model->save_data($import_id, $validate_data["data"]);
                        $result = $this->Import_model->analyze_data($import_id);
                        if (!empty($result['errors'])) {
                            $this->system_messages->addMessage(View::MSG_ERROR, $result["errors"]);
                        } else {
                            if (!empty($result['file'])) {
                                $this->system_messages->addMessage(View::MSG_SUCCESS,
                                                                   l("success_file_uploaded", "import"));
                            } else {
                                $this->system_messages->addMessage(View::MSG_SUCCESS,
                                                                   l("success_selection_saved", "import"));
                            }

                            $url = site_url() . "admin/import/edit/" . $import_id . $relations;
                            redirect($url);
                        }
                    }
                }

                if (!$import_id) {
                    $drivers = $this->Import_driver_model->get_drivers();
                    foreach ($drivers as $key => $driver) {
                        if (!$driver["status"]) {
                            unset($drivers[$key]);
                        }
                    }
                    $this->view->assign("drivers", $drivers);

                    $this->load->model("import/models/Import_module_model");
                    $modules = $this->Import_module_model->get_modules();
                    $this->view->assign("modules", $modules);

                    if (empty($drivers) || empty($modules)) {
                        show_404();
                    }

                    if (empty($data['gid_driver'])) {
                        $data['driver'] = current($drivers);
                        $data['gid_driver'] = $data['driver']['gid'];
                    } else {
                        foreach ($drivers as $key => $driver) {
                            if ($data['gid_driver'] == $driver['gid']) {
                                $data['driver'] = $driver;
                                break;
                            }
                        }
                    }

                    if (empty($data['input_type']) && !empty($data['driver']['input_types'])) {
                        $data['input_type'] = current($data['driver']['input_types']);
                    }

                    if (empty($data['settings'])) {
                        $data['settings'] = array();
                    }

                    $model_name = "import_" . $data['gid_driver'] . "_model";
                    $this->load->model("import/models/drivers/" . $model_name, $model_name);

                    $driver_settings = $this->{$model_name}->get_settings($data['driver']['settings'], $data['settings']);
                    $this->view->assign('driver_settings', $driver_settings);
                }

                $this->view->assign("driver_gid", $data['gid_driver']);
                $this->view->assign("data", $data);

                //time
                $this->view->assign("hours", range(0, 24, 1));
                $this->view->assign("minutes", range(0, 50, 10));

                $page_data = array();
                $page_data['date_format'] = $this->pg_date->get_format('date_literal', 'st');
                $page_data['date_time_format'] = $this->pg_date->get_format('date_time_literal', 'st');
                $page_data['datepicker_date_format'] = $this->pg_date->get_format('date_literal', 'ui');
                $page_data['datepicker_alt_format'] = $this->pg_date->get_format('date_numeric', 'ui');
                $this->view->assign('page_data', $page_data);
                break;
            case 'relations':
                $module_fields = $this->Import_module_model->get_module_fields($data["id_object"]);
                if ($this->input->post("btn_save")) {
                    $links = $this->input->post("link", true);
                    foreach ($data["relations"] as $index => $relation) {
                        if (!isset($links[$index])) {
                            continue;
                        }
                        if ($links[$index] != -1 && !empty($module_fields[$links[$index]]["name"])) {
                            $data["relations"][$index]["type"] = $module_fields[$links[$index]]["type"];
                            $data["relations"][$index]["link"] = $module_fields[$links[$index]]["name"];
                        } else {
                            $data["relations"][$index]["type"] = "";
                            $data["relations"][$index]["link"] = "";
                        }
                    }

                    $validate_data = $this->Import_model->validate_data($import_id,
                                                                        array('relations' => $data["relations"]));
                    if (!empty($validate_data["errors"])) {
                        $this->system_messages->addMessage(View::MSG_ERROR, $validate_data["errors"]);
                    } else {
                        $this->Import_model->save_data($import_id, $validate_data["data"]);
                        $this->system_messages->addMessage(View::MSG_SUCCESS, l("success_relation_updated", "import"));

                        $url = site_url() . "admin/import/edit/" . $import_id . "/relations/" . $page;
                        redirect($url);
                    }
                }

                $items_on_page = $this->pg_module->get_module_config('start', 'admin_items_per_page');
                $relations_count = count($data['relations']);

                $this->load->helper('sort_order');
                $page = get_exists_page_number($page, $relations_count, $items_on_page);

                $relations = array_slice($data['relations'], ($page - 1) * $items_on_page, $items_on_page, true);
                $this->view->assign('relations', $relations);

                $this->load->helper('navigation');
                $url = site_url() . 'admin/import/edit/' . $import_id . '/relations/';
                $page_data = get_admin_pages_data($url, $relations_count, $items_on_page, $page, 'briefPage');
                $this->view->assign('page_data', $page_data);

                $this->view->assign("module_fields", $module_fields);
                $this->view->assign("data", $data);
                break;
            case 'search_form':
                if ($this->input->post("btn_save")) {
                    $criteria = $this->input->post("search_data", true);
                    foreach ($data["search_data"] as $index => $value) {
                        if (!isset($criteria[$index])) {
                            continue;
                        }
                        if ($criteria[$index] != -1 && !empty($criteria[$index])) {
                            $data["search_data"][$index]["value"] = $criteria[$index];
                        } else {
                            $data["search_data"][$index]["value"] = "";
                        }
                    }
                    $validate_data = $this->Import_model->validate_data($import_id,
                                                                        array('search_data' => $data["search_data"]));
                    if (!empty($validate_data["errors"])) {
                        $this->system_messages->addMessage(View::MSG_ERROR, $validate_data["errors"]);
                    } else {
                        $this->Import_model->save_data($import_id, $validate_data["data"]);
                        $this->system_messages->addMessage(View::MSG_SUCCESS, l("success_relation_updated", "import"));

                        $url = site_url() . "admin/import/edit/" . $import_id . "/search_form/" . $page;
                        redirect($url);
                    }
                }

                $items_on_page = $this->pg_module->get_module_config('start', 'admin_items_per_page');
                $search_data_count = count($data['search_data']);

                $this->load->helper('sort_order');
                $page = get_exists_page_number($page, $search_data_count, $items_on_page);

                $search_data = array_slice($data['search_data'], ($page - 1) * $items_on_page, $items_on_page, true);
                $this->view->assign('search_data', $search_data);

                $this->load->helper('navigation');
                $url = site_url() . 'admin/import/edit/' . $import_id . '/search_form/';
                $page_data = get_admin_pages_data($url, $search_data_count, $items_on_page, $page, 'briefPage');
                $this->view->assign('page_data', $page_data);

                $this->view->assign("data", $data);
                break;
            case 'manual_import':
                foreach ($data["relations"] as $index => $relation) {
                    if (!$relation["link"]) {
                        unset($data["relations"][$index]);
                    }
                }

                $model_name = "Import_" . $data["gid_driver"] . "_model";
                $this->load->model("import/models/drivers/" . $model_name, $model_name);
                $this->view->assign("two_step", $this->{$model_name}->two_step);
                $this->view->assign("need_upload", $this->{$model_name}->need_upload);

                /* if(!$this->{$model_name}->two_step){
                  if($data["total"] > $data["processed"]) $this->parse($import_id);
                  $data["processed"] = $data["total"];
                  } */

                $imported = $this->{$model_name}->two_step ? $data['processed'] : $data["total"] - ($data['imported'] + $data['failed']);

                $this->view->assign("relations", count($data["relations"]));
                $this->view->assign("processed", $data["total"] - $data["processed"]);
                $this->view->assign("imported", $imported);

                $this->view->assign("data", $data);
                break;
        }

        $this->view->assign("section_gid", $section_gid);

        $this->view->setHeader(l("admin_header_data_edit", "import"));
        $this->view->render("edit");
    }

    /**
     * Set fields relations
     *
     * @param integer $data_id selection identifier
     *
     * @return void
     */
    public function relations($data_id)
    {
        $this->load->model("import/models/Import_module_model");
        $this->load->model("Import_model");

        $data = $this->Import_model->get_data_by_id($data_id, true);

        $module_fields = $this->Import_module_model->get_module_fields($data["id_object"]);

        $model_name = "import_" . $data['gid_driver'] . "_model";
        $this->load->model("import/models/drivers/" . $model_name, $model_name);

        if (!empty($this->{$model_name}->options)) {
            $return = $this->{$model_name}->analyze($data['settings']);
            if (!empty($return['errors'])) {
                $this->system_messages->addMessage(View::MSG_ERROR, $validate_data["errors"]);
            } else {
                $options = $this->{$model_name}->options;
                foreach ($options as $key => $option) {
                    if (isset($return['options'][$option['name']])) {
                        $options[$key]['data'] = $return['options'][$option['name']];
                    }
                }
                $this->view->assign("options", $options);

                if (!empty($return['fields'])) {
                    $data['relations'] = $data['relations'] + $return['fields'];
                    if (!empty($return['labels'])) {
                        $this->view->assign('labels', $return['labels']);
                    }
                }
            }
        }

        if ($this->input->post("btn_save")) {
            $errors = array();

            $options = $this->input->post("options", true);
            if (!empty($options)) {
                $validate_data = $this->{$model_name}->validate_settings($data['settings'], $options);
                if (!empty($validate_data['errors'])) {
                    $errors = array_merge($errors, $validate_data['errors']);
                } else {
                    $save_data["settings"] = array_merge($data['settings'], $validate_data['data']);
                }
            }

            $links = $this->input->post("link", true);
            foreach ($data["relations"] as $index => $relation) {
                if (isset($links[$index]) && $links[$index] != -1 && !empty($module_fields[$links[$index]]["name"])) {
                    $data["relations"][$index]["type"] = $module_fields[$links[$index]]["type"];
                    $data["relations"][$index]["link"] = $module_fields[$links[$index]]["name"];
                    $data["relations"][$index]["label"] = $module_fields[$links[$index]]["label"];
                } else {
                    $data["relations"][$index]["type"] = "";
                    $data["relations"][$index]["link"] = "";
                    $data["relations"][$index]["label"] = "";
                }
            }
            $save_data["relations"] = $data["relations"];
            $validate_data = $this->Import_model->validate_data($data_id, $save_data);
            $validate_data['errors'] = array_merge($errors, $validate_data['errors']);
            if (!empty($validate_data["errors"])) {
                $this->system_messages->addMessage(View::MSG_ERROR, $validate_data["errors"]);
            } else {
                $this->Import_model->save_data($data_id, $validate_data["data"]);
                $this->system_messages->addMessage(View::MSG_SUCCESS, l("success_relation_updated", "import"));

                $url = site_url() . "admin/import/relations/" . $data_id;
                redirect($url);
            }
        }

        $this->view->assign("data", $data);
        $this->view->assign("module_fields", $module_fields);

        $this->view->setHeader(l("admin_header_relations", "import"));
        $this->view->render("relations");
    }

    /**
     * Remove selections data
     *
     * @param array $ids selictions identifiers
     *
     * @return void
     */
    public function delete($ids = null)
    {
        if (!$ids) {
            $ids = $this->input->post("ids");
        }
        if (!empty($ids)) {
            $this->load->model("Import_model");
            foreach ((array) $ids as $id) {
                $this->Import_model->delete_file($id, true);
                $this->Import_model->delete_data($id);
            }
            $this->system_messages->addMessage(View::MSG_SUCCESS, l("success_data_deleted", "import"));
        }
        $url = site_url() . "admin/import";
        redirect($url);
    }

    /**
     * Parse imported data
     *
     * @param integer $data_id data identifier
     *
     * @return array
     */
    public function parse($data_id)
    {
        @set_time_limit(0);

        $this->load->model("Import_model");
        $data = $this->Import_model->get_data_by_id($data_id, true);

        try {
            $result = $this->Import_model->parse_data($data_id);
        } catch (Exception $e) {
            $result['errors'][] = $e->getMessage();
        }

        if (!empty($result['errors'])) {
            $this->system_messages->addMessage(View::MSG_ERROR, $result['errors']);
        } else {
            $this->system_messages->addMessage(View::MSG_SUCCESS, l("success_data_parsed", "import"));
        }

        $url = site_url() . "admin/import/edit/" . $data_id . "/manual_import";
        redirect($url);
    }

    /**
     *  Install Driver page
     *
     *   @return void
     */
    public function install()
    {

        ///// drivers
        $this->load->model("import/models/Import_driver_model");
        $drivers = array();

        $path = MODULEPATH . 'import/models/drivers/';
        if (!is_dir($path)) {
            show_404();
        }
        $all_files = scandir($path);
        $files = array_diff($all_files, array('.', '..'));
        foreach ($files as $file) {
            $result = explode('.', $file);
            if (!empty($result[1])) {
                $driver_gid = explode('_', $result[0]);
                $data = $this->Import_driver_model->get_driver_by_gid($driver_gid[1]);
                if (empty($data)) {
                    $driver_data = $this->Import_driver_model->loadDrivers($driver_gid[1], true);
                    if (!empty($driver_data)) {
                        $drivers[$driver_gid[1]] = $driver_data;
                    }
                }
            }
        }

        $this->view->assign('drivers', $drivers);
        $this->Menu_model->set_menu_active_item("admin_import_menu", "import_drivers_items");
        $this->view->setHeader(l('admin_header_install_list', 'import'));
        $this->view->render('list_install');
    }

    /**
     *  Driver install
     *
     *  @param string $gid
     *
     *  @return void
     */
    public function installDrivers($gid = null)
    {
        if (!is_null($gid)) {
            $this->load->model("import/models/Import_driver_model");
            $driver_data = $this->Import_driver_model->loadDrivers($gid);
            if (!empty($driver_data)) {
                $validate_data = $this->Import_driver_model->validate_driver($gid, $driver_data);
                if (!empty($validate_data["errors"])) {
                    show_404();
                }
                $this->Import_driver_model->save_driver(null, $validate_data["data"]);
                redirect(site_url() . "admin/import/drivers/");
            }
        }
    }

    /**
     * Parse imported data by ajax
     *
     * @param integer $data_id data identifier
     *
     * @return void
     */
    public function ajax_parse($data_id)
    {
        @set_time_limit(0);

        $response = array("error" => "", "processed" => 0);

        $this->load->model("Import_model");
        $data = $this->Import_model->get_data_by_id($data_id, true);

        try {
            $result = $this->Import_model->parse_data($data_id);
        } catch (Exception $e) {
            $result["errors"][] = $e->getMessage();
        }

        if (!empty($result["errors"])) {
            $response["error"] = implode("<br>", $result["errors"]);
        } else {
            $response["imported"] = $result["data"]["processed"];
        }

        $this->view->assign($response);

        $this->view->render();
    }

    /**
     * Process imported data
     *
     * @param integer $data_id data identifier
     *
     * @return void
     */
    public function make($data_id)
    {
        @set_time_limit(0);

        $this->load->model("Import_model");

        $data = $this->Import_model->get_data_by_id($data_id, true);
        $model_name = "Import_" . $data["gid_driver"] . "_model";
        $this->load->model("import/models/drivers/" . $model_name, $model_name);

        if (!$this->{$model_name}->two_step && $data["total"] - $data["processed"]) {
            $results = $this->parse($data_id);
            if (!empty($results['errors'])) {
                $this->system_messages->addMessage(View::MSG_ERROR, $results["errors"]);
            }
        }

        $results = $this->Import_model->make_import_data($data_id);
        if (!empty($results["errors"])) {
            $this->system_messages->addMessage(View::MSG_ERROR, $results["errors"]);
        } else {
            if ($results["data"]["failed"]) {
                $this->system_messages->addMessage(View::MSG_ERROR,
                                                   array(
                    l("error_data_imported", "import") . ": " . $results["data"]["imported"],
                    l("error_data_failed", "import") . ": " . $results["data"]["failed"],
                ));
            } else {
                $this->system_messages->addMessage(View::MSG_SUCCESS,
                                                   l("success_data_imported", "import") . ": " . $results["data"]["imported"]);
            }
        }

        $url = site_url() . "admin/import/edit/" . $data_id . '/manual_import';
        redirect($url);
    }

    /**
     * Process imported data action by ajax
     *
     * @param integer $import_id data identifier
     *
     * @return void
     */
    public function ajax_make($import_id)
    {
        @set_time_limit(0);

        $this->load->model("Import_model");

        $response = array("errors" => "", "imported" => 0);

        $import_data = $this->Import_model->get_data_by_id($import_id, true);
        if (!$import_data) {
            $response["error"] = l("error_empty_data", "import");

            $this->view->assign($response);

            $this->view->render();

            return;
        }

        $model_name = "Import_" . $import_data["gid_driver"] . "_model";
        $this->load->model("import/models/drivers/" . $model_name, $model_name);

        if (!$this->{$model_name}->two_step && $import_data["total"] > ($import_data["processed"] + $import_data["imported"] + $import_data["failed"])) {
            try {
                $result = $this->Import_model->parse_data($import_id);
            } catch (Exception $e) {
                $result["errors"][] = $e->getMessage();
            }

            if (!empty($result['errors'])) {
                $response["error"] = implode('<br>', $result['errors']);

                $this->view->assign($response);

                $this->view->render();

                return;
            }
            $import_data["processed"] = $result["data"]["processed"];
        }

        $result = $this->Import_model->make_import_data($import_id);
        if (!empty($result['errors'])) {
            $response['error'] = implode('<br>', $result['errors']);
        } else {
            $response["imported"] = $result["data"]["imported"];
            $response["failed"] = $result["data"]["failed"];
            $response["total"] = $this->{$model_name}->two_step ? $result["data"]["processed"] : $import_data["total"] - ($result["data"]["imported"] + $result["data"]["failed"]);
            if ($response["total"] < 1) {
                if ($response["failed"]) {
                    $response["error"] = l("error_data_imported", "import") . ": " . $result["data"]["imported"] . "<br>" .
                            l("error_data_failed", "import") . ": " . $result["data"]["failed"];
                } else {
                    $response["success"] = l("success_data_imported", "import") . ": " . $result["data"]["imported"];
                }
            }
        }

        $this->view->assign($response);

        $this->view->render();
    }

    /**
     * Manage drivers
     *
     * @return void
     */
    public function drivers()
    {
        $this->load->model("import/models/Import_driver_model");
        $drivers = $this->Import_driver_model->get_drivers();
        $this->view->assign("drivers", $drivers);
        $this->Menu_model->set_menu_active_item("admin_import_menu", "import_drivers_items");
        $this->view->render("drivers_list");
    }

    /**
     * Activate/deactivate driver
     *
     * Available status values: 1 - activate, 0 - deactivate
     *
     * @param string  $driver_gid driver guid
     * @param integer $status     driver status
     *
     * @return void
     */
    public function activate_driver($driver_gid, $status = 1)
    {
        $this->load->model("import/models/Import_driver_model");
        if ($status) {
            $connect_data = $this->Import_driver_model->connect($driver_gid);
            if (!empty($connect_data["errors"])) {
                $this->system_messages->addMessage(View::MSG_ERROR, $connect_data["errors"]);
            } else {
                $this->Import_driver_model->activate_driver($driver_gid);
            }
        } else {
            $this->Import_driver_model->deactivate_driver($driver_gid);
        }
        redirect(site_url() . "admin/import/drivers");
    }

    /**
     * Edit driver settings
     *
     * @param string $driver_gid driver guid
     *
     * @return void
     */
    public function edit_driver($driver_gid)
    {
        $this->load->model('import/models/Import_driver_model');
        $data = $this->Import_driver_model->get_driver_by_gid($driver_gid);

        if ($this->input->post("btn_save")) {
            $post_data = $this->input->post("data");

            $validate_data = $this->Import_driver_model->validate_driver($driver_gid, $post_data);
            if (!empty($validate_data["errors"])) {
                $this->system_messages->addMessage(View::MSG_ERROR, $validate_data["errors"]);
            } else {
                $this->Import_driver_model->save_driver($driver_gid, $validate_data["data"]);

                $this->system_messages->addMessage(View::MSG_SUCCESS, l("success_driver_saved", "import"));

                $url = site_url() . "admin/import/drivers";
                redirect($url);
            }
        }

        $this->view->assign("data", $data);

        $this->view->render("driver_" . $driver_gid . "_edit");
    }

}
