<?php

namespace Pg\Modules\Listings_booking\Controllers;

use Pg\Modules\Listings_booking\Models\Listings_booking_model;

/**
 * Listings booking module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
use Pg\Libraries\View;

/**
 * Listings booking admin side controller
 *
 * @package 	PG_RealEstate
 * @subpackage 	Listings booking
 *
 * @category	controllers
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Admin_Listings_booking extends \Controller
{

    /**
     * Constructor
     *
     * @return Admin_Listings
     */
    public function __construct()
    {
        parent::__construct();
        $this->load->model('Menu_model');
        $this->Menu_model->set_menu_active_item('admin_menu', 'listings_menu_item');
        $this->view->setHeader(l('admin_header_listings_booking', 'listings_booking'));
    }

    /**
     * Bookings requests management
     *
     * @param string  $status          order status
     * @param string  $order           sorting order
     * @param string  $order_direction order direction
     * @param integer $page            page of results
     *
     * @return void
     */
    public function orders($status = null, $order = 'modified', $order_direction = 'DESC', $page = 1)
    {
        $current_settings = isset($_SESSION['listings_booking']) ? $_SESSION['listings_booking'] : array();
        if (!isset($current_settings['filters'])) {
            $current_settings['filters'] = array('status' => 'wait');
        }
        if (!isset($current_settings['order'])) {
            $current_settings['order'] = 'modified';
        }
        if (!isset($current_settings['order_direction'])) {
            $current_settings['order_direction'] = 'DESC';
        }
        if (!isset($current_settings['page'])) {
            $current_settings['page'] = '1';
        }

        if ($this->input->post('filter-submit')) {
            $current_settings['filters'] = array('status' => 'wait');

            $post_data['id_listing'] = $this->input->post('id_listing', true);
            if (!empty($post_data['id_listing'])) {
                $current_settings['filters']['listings'] = $post_data['id_listing'];
            } elseif (isset($current_settings['filters']['listings'])) {
                unset($current_settings['filters']['listings']);
            }

            $post_data['id_owner'] = $this->input->post('id_owner', true);
            if (!empty($post_data['id_owner'])) {
                $current_settings['filters']['owners'] = $post_data['id_owner'];
            } elseif (isset($current_settings['filters']['owners'])) {
                unset($current_settings['filters']['owners']);
            }

            $post_data['id_user'] = $this->input->post('id_user', true);
            if (!empty($post_data['id_user'])) {
                $current_settings['filters']['users'] = $post_data['id_user'];
            } elseif (isset($current_settings['filters']['users'])) {
                unset($current_settings['filters']['users']);
            }
        }

        if ($this->input->post('filter-reset')) {
            $current_settings['filters'] = array('status' => 'wait');
        }

        $this->load->model('listings_booking/models/Listings_booking_model');

        if ($status) {
            if ($status != $current_settings['filters']['status']) {
                $page = 1;
            }
            $current_settings['filters']['status'] = $status;
        } else {
            $status = $current_settings['filters']['status'];
        }

        $filters = $current_settings['filters'];

        $order = trim(strip_tags($order));

        if (!empty($order)) {
            $current_settings['order'] = $order;
        }
        $order = $current_settings['order'];

        $this->view->assign('order', $order);

        $order_direction = strtoupper(trim(strip_tags($order_direction)));
        if (!empty($order_direction)) {
            $current_settings['order_direction'] = $order_direction;
        }
        $order_direction = $current_settings['order_direction'];
        $this->view->assign('order_direction', $order_direction);

        $page = intval($page);
        if (!empty($page)) {
            $current_settings['page'] = $page;
        }
        $page = $current_settings['page'];
        $this->view->assign('page', $page);

        if ($order_direction != 'ASC') {
            $order_direction = 'DESC';
        }

        $items_on_page = $this->pg_module->get_module_config('start', 'admin_items_per_page');
        $items_count = $this->Listings_booking_model->getPeriodsCount($filters);
        $this->load->helper('sort_order');
        $current_settings['page'] = get_exists_page_number($page, $items_count, $items_on_page);

        $_SESSION['listings_booking'] = $current_settings;

        // sorting
        $sort_data = array(
            'url' => site_url() . 'admin/listings_booking/orders/',
            'order' => $current_settings['order'],
            'direction' => $current_settings['order_direction'],
            'links' => array('modified' => l('field_date_modified', 'listings')),
        );
        $this->view->assign('sort_data', $sort_data);

        if ($items_count > 0) {
            $order_array = array();
            switch ($order) {
                case 'modified':
                    $order_array['date_modified'] = $order_direction;
                    break;
                default:
                    $order_array[$order] = $order_direction;
                    break;
            }
            $this->Listings_booking_model->set_format_settings('get_listing', true);
            $periods = $this->Listings_booking_model->get_periods_list($filters, $current_settings['page'], $items_on_page, $order_array);
            foreach ($periods as $key => $period) {
                if (!isset($period['user']['id']) && is_array($period['booking_data'])) {
                    $periods[$key]['user']['output_name'] = $period['booking_data']['fname'] .
                        ' ' . $period['booking_data']['sname'];
                }
            }
            $this->Listings_booking_model->set_format_settings('get_listing', false);
            $this->view->assign('orders', $periods);
        }
        $this->view->assign('orders_count', $items_count);

        $this->load->helper('navigation');

        $url = site_url() . 'admin/listings_booking/orders/' . $order . '/' . $order_direction . '/';
        $page_data = get_user_pages_data($url, $items_count, $items_on_page, $current_settings['page'], 'briefPage');
        $page_data['date_format'] = $this->pg_date->get_format('date_literal', 'st');
        $page_data['date_time_format'] = $this->pg_date->get_format('date_time_literal', 'st');
        $this->view->assign('page_data', $page_data);

        $status_data = array('approve' => 0, 'decline' => 0, 'wait' => 0);
        $status_data['approve'] = $this->Listings_booking_model->getPeriodsCount(array('status' => 'approve'));
        $status_data['decline'] = $this->Listings_booking_model->getPeriodsCount(array('status' => 'decline'));
        $status_data['wait'] = $this->Listings_booking_model->getPeriodsCount(array('status' => 'wait'));
        $this->view->assign('status_data', $status_data);
        $this->view->assign('status', $status);
        $this->view->assign('filters', $current_settings['filters']);
        $this->view->setHeader(l('admin_header_orders', 'listings_booking'));
        $this->Menu_model->breadcrumbs_set_parent('admin_listings_item');
        $this->view->render('orders');
    }

    /**
     * Edit booking request data
     *
     * @param integer $order_id order identifier
     *
     * @return void
     */
    public function order_edit($order_id = null)
    {
        $this->load->model('listings_booking/models/Listings_booking_model');

        $this->Listings_booking_model->set_format_settings('get_listing', true);
        $order = $this->Listings_booking_model->get_period_by_id($order_id);
        $this->Listings_booking_model->set_format_settings('get_listing', false);

        if ($order_id) {
            show_404();
        }

        if ($this->input->post('btn_save')) {
            $post_data = $this->input->post('period', true);
            $validate_data = $this->Listings_booking_model->validate_period($order_id, $post_data);
            if (!empty($validate_data['errors'])) {
                $this->system_messages->addMessage(View::MSG_ERROR, $validate_data['errors']);
            } else {
                if ($order_id) {
                    $this->Listings_booking_model->save_period($order_id, $validate_data['data']);
                    $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_order_updated', 'listings_booking'));
                } else {
                    $listing_id = $this->Listings_booking_model->save_period(null, $validate_data['data']);
                    $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_order_added', 'listings_booking'));
                }
                $url = site_url() . 'admin/listings_booking/order_edit/' . $order_id;
                $this->view->setRedirect($url);
            }
        }
        $this->view->assign('data', $order);
        $this->view->setHeader(l('admin_header_order_edit', 'listings_booking'));
        $this->Menu_model->breadcrumbs_set_parent('admin_listings_item');
        $this->view->render('order_edit');
    }

    /**
     * Remove booking request data
     *
     * @param integer $order_id order identifier
     *
     * @return void
     */
    public function order_delete($order_id)
    {
        $this->load->model('listings_booking/models/Listings_booking_model');

        if (!empty($order_id)) {
            $this->Listings_booking_model->delete_period($order_id);
            $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_order_deleted', 'listings_booking'));
        }

        $this->view->setRedirect(site_url() . 'admin/listings_booking/orders');
    }

    /**
     * Remove booking request data by ajax
     *
     * @param integer $order_id order identifier
     *
     * @return void
     */
    public function ajax_order_delete($order_id)
    {
        $return = array('error' => '', 'success' => '');

        $this->load->model('listings_booking/models/Listings_booking_model');

        if (!empty($order_id)) {
            $this->Listings_booking_model->delete_period($order_id);
            $return['success'] = l('success_order_deleted', 'listings_booking');
        } else {
            $return['error'] = 'No order';
        }

        $this->view->assign($return);
        
        $this->view->render();
    }

    /**
     * Approve booking request
     *
     * @param integer $order_id order identifier
     *
     * @return void
     */
    public function order_approve($order_id)
    {
        $this->load->model('listings_booking/models/Listings_booking_model');

        $this->Listings_booking_model->set_format_settings('get_listing', true);
        $order = $this->Listings_booking_model->get_period_by_id($order_id);
        if (!empty($order)) {
            $is_intersect_exists_orders = $this->Listings_booking_model->is_intersect_exists_orders($order['id_listing'], $order['id_user'], $order['date_start'], $order['date_end'], $order['id']);
            if ($is_intersect_exists_orders !== false) {
                $this->system_messages->addMessage(View::MSG_SUCCESS, l('error_intersect_calendar_periods_approve_request', 'listings_booking'));
                return $this->view->setRedirect(site_url() . 'admin/listings_booking/orders');
            }
        }

        if (!empty($order_id)) {
            $this->Listings_booking_model->approve_period($order_id);
            $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_order_approved', 'listings_booking'));
        }

        $this->view->setRedirect(site_url() . 'admin/listings_booking/orders');
    }

    /**
     * Decline booking request
     *
     * @param integer $order_id order identifier
     *
     * @return void
     */
    public function order_decline($order_id)
    {
        $this->load->model('listings_booking/models/Listings_booking_model');

        if (!empty($order_id)) {
            $this->Listings_booking_model->decline_period($order_id);
            $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_order_declined', 'listings_booking'));
        }

        $this->view->setRedirect(site_url() . 'admin/listings_booking/orders');
    }

    /**
     * Show booking form by ajax
     *
     * @param integer $listing_id listing identifier
     * @param integer $order_id   order identifier
     *
     * @return void
     */
    public function ajax_order_form($listing_id, $order_id = null)
    {
        $this->load->model('Listings_model');

        $this->Listings_model->set_format_settings('use_format', false);
        $listing = $this->Listings_model->get_listing_by_id($listing_id);
        $this->Listings_model->set_format_settings('use_format', true);

        if (!$listing) {
            return '';
        }

        $this->view->assign('listing', $listing);

        if ($order_id) {
            $this->load->model('listings_booking/models/Listings_booking_model');
            $this->Listings_booking_model->set_format_settings('use_format', false);
            $order = $this->Listings_booking_model->get_period_by_id($order_id);
            $this->Listings_booking_model->set_format_settings('use_format', true);
            $this->view->assign('order', $order);
        }

        $page_data['date_format'] = $this->pg_date->get_format('date_literal', 'st');
        $page_data['date_time_format'] = $this->pg_date->get_format('date_time_literal', 'st');
        $page_data['datepicker_date_format'] = $this->pg_date->get_format('date_literal', 'ui');
        $page_data['datepicker_alt_format'] = $this->pg_date->get_format('date_numeric', 'ui');
        $this->view->assign('page_data', $page_data);
        $this->view->assign('rand', rand(100000, 999999));
        $this->view->render('ajax_order_form');
    }

    /**
     * Save booking request by ajax
     *
     * @param integer $listing_id listing identifier
     * @param integer $order_id   order identifier
     *
     * @return void
     */
    public function ajax_save_order($listing_id, $order_id = null)
    {
        $result = array('error' => array(), 'success' => array(), 'id' => 0, 'data' => '');

        $this->load->model('Listings_model');

        $this->Listings_model->set_format_settings('use_format', false);
        $listing = $this->Listings_model->get_listing_by_id($listing_id);
        $this->Listings_model->set_format_settings('use_format', true);

        $this->load->model('listings_booking/models/Listings_booking_model');

        $post_period = $this->input->post('period', true);
        $post_period['id_listing'] = $listing_id;
        $post_period['id_user'] = $listing['id_user'];
        $post_period['status'] = 'book';

        switch ($listing['price_period']) {
            case Listings_booking_model::PRICE_PERIOD_DAY:
                $date_start_alt = $this->input->post('date_start_alt', true);
                if ($date_start_alt) {
                    $post_period['date_start'] = $date_start_alt;
                }

                $date_end_alt = $this->input->post('date_end_alt', true);
                if ($date_end_alt) {
                    $post_period['date_end'] = $date_end_alt;
                }
                break;
            case Listings_booking_model::PRICE_PERIOD_MONTH:
                $date_start_month = $this->input->post('date_start_month', true);
                $date_start_year = $this->input->post('date_start_year', true);
                $post_period['date_start'] = $date_start_year . '-' . $date_start_month . '-01';

                $date_end_month = $this->input->post('date_end_month', true);
                $date_end_year = $this->input->post('date_end_year', true);
                $post_period['date_end'] = $date_end_year . '-' . $date_end_month . '-01';
                break;
        }

        $validate_data = $this->Listings_booking_model->validate_period($order_id, $post_period, true);
        if (!empty($validate_data['errors'])) {
            $result['error'] = implode('<br>', $validate_data['errors']);
        } else {
            $guests = $validate_data['data']['guests'] ? $validate_data['data']['guests'] : 1;

            $validate_data['data']['price'] = $this->Listings_booking_model->get_period_price($listing, $validate_data['data']['date_start'], $validate_data['data']['date_end'], $guests);

            if ($order_id) {
                $result['success'] = l('success_order_updated', 'listings_booking');
            } else {
                $result['success'] = l('success_order_added', 'listings_booking');
            }

            $order_id = $this->Listings_booking_model->save_period($order_id, $validate_data['data']);

            $result['id'] = $validate_data['data']['id'] = $order_id;

            $this->view->assign('listing', $listing);

            $period = $this->Listings_booking_model->format_period($validate_data['data']);
            $this->view->assign('period', $period);

            $page_data['date_format'] = $this->pg_date->get_format('date_literal', 'st');
            $page_data['date_time_format'] = $this->pg_date->get_format('date_time_literal', 'st');
            $page_data['datepicker_date_format'] = $this->pg_date->get_format('date_literal', 'ui');
            $page_data['datepicker_alt_format'] = $this->pg_date->get_format('date_numeric', 'ui');
            $this->view->assign('page_data', $page_data);

            $this->view->assign('edit', 1);

            $result['data'] = $this->view->fetch('calendar_period', 'admin', 'listings_booking');
        }

        $this->view->assign($result);
        
        $this->view->render();
    }

    /**
     * Show calendar period form by ajax
     *
     * @param integer $listing_id listing identifier
     * @param integer $period_id  period identifier
     *
     * @return void
     */
    public function ajax_period_form($listing_id, $period_id = null)
    {
        $this->load->model('Listings_model');

        $this->Listings_model->set_format_settings(array('get_user' => false, 'get_location' => false));
        $listing = $this->Listings_model->get_listing_by_id($listing_id);
        $this->Listings_model->set_format_settings(array('get_user' => true, 'get_location' => true));

        if (!$listing) {
            return '';
        }

        $this->view->assign('data', $listing);

        if ($period_id) {
            $this->load->model('listings_booking/models/Listings_booking_model');
            $period = $this->Listings_booking_model->get_period_by_id($period_id);
            $this->view->assign('period', $period);
        }

        if ($this->pg_module->is_module_active('payments')) {
            $this->load->model('Currencies_model');
            $currencies = $this->Currencies_model->get_currency_list();
            $this->view->assign('currencies', $currencies);

            if (isset($listing['gid_currency']) && !empty($listing['gid_currency'])) {
                $current_price_currency = $this->Currencies_model->get_currency_by_gid($listing['gid_currency']);
                $this->view->assign('current_price_currency', $current_price_currency);
            }
        }

        $page_data['date_format'] = $this->pg_date->get_format('date_literal', 'st');
        $page_data['date_time_format'] = $this->pg_date->get_format('date_time_literal', 'st');
        $page_data['datepicker_date_format'] = $this->pg_date->get_format('date_literal', 'ui');
        $page_data['datepicker_alt_format'] = $this->pg_date->get_format('date_numeric', 'ui');
        $this->view->assign('page_data', $page_data);
        $this->view->assign('rand', rand(100000, 999999));
        $this->view->render('ajax_period_form');
    }

    /**
     * Save calendar period by ajax
     *
     * @param integer $listing_id listing identifier
     * @param integer $period_id  period identifier
     *
     * @return void
     */
    public function ajax_save_period($listing_id, $period_id = null)
    {
        $result = array('error' => array(), 'success' => array(), 'id' => 0, 'data' => '');

        $this->load->model('Listings_model');

        $this->Listings_model->set_format_settings('use_format', false);
        $listing = $this->Listings_model->get_listing_by_id($listing_id);
        $this->Listings_model->set_format_settings('use_format', true);

        $this->load->model('listings_booking/models/Listings_booking_model');

        $post_period = $this->input->post('period', true);
        $post_period['id_listing'] = $listing_id;
        $post_period['id_user'] = $listing['id_user'];

        switch ($listing['price_period']) {
            case Listings_booking_model::PRICE_PERIOD_DAY:
                $date_start_alt = $this->input->post('date_start_alt', true);
                if ($date_start_alt) {
                    $post_period['date_start'] = $date_start_alt;
                }

                $date_end_alt = $this->input->post('date_end_alt', true);
                if ($date_end_alt) {
                    $post_period['date_end'] = $date_end_alt;
                }
                break;
            case Listings_booking_model::PRICE_PERIOD_MONTH:
                $date_start_month = $this->input->post('date_start_month', true);
                $date_start_year = $this->input->post('date_start_year', true);
                $post_period['date_start'] = $date_start_year . '-' . $date_start_month . '-01';

                $date_end_month = $this->input->post('date_end_month', true);
                $date_end_year = $this->input->post('date_end_year', true);
                $post_period['date_end'] = $date_end_year . '-' . $date_end_month . '-01';
                break;
        }

        $validate_data = $this->Listings_booking_model->validate_period($period_id, $post_period);
        if (!empty($validate_data['errors'])) {
            $result['error'] = implode('<br>', $validate_data['errors']);
        } else {
            if ($period_id) {
                $result['success'] = l('success_period_updated', 'listings_booking');
            } else {
                $result['success'] = l('success_period_added', 'listings_booking');
            }

            $period_id = $this->Listings_booking_model->save_period($period_id, $validate_data['data']);
            $result['id'] = $validate_data['data']['id'] = $period_id;
            $this->view->assign('listing', $listing);
            $period = $this->Listings_booking_model->format_period($validate_data['data']);
            $this->view->assign('period', $period);
            $page_data['date_format'] = $this->pg_date->get_format('date_literal', 'st');
            $page_data['date_time_format'] = $this->pg_date->get_format('date_time_literal', 'st');
            $page_data['datepicker_date_format'] = $this->pg_date->get_format('date_literal', 'ui');
            $page_data['datepicker_alt_format'] = $this->pg_date->get_format('date_numeric', 'ui');
            $this->view->assign('page_data', $page_data);
            $this->view->assign('edit', 1);
            $result['data'] = $this->view->fetch('calendar_period', 'admin', 'listings_booking');
        }

        $this->view->assign($result);
        
        $this->view->render();
    }

    /**
     * Remove calendar period
     *
     * @param integer $period_id period identifier
     *
     * @return void
     */
    public function period_delete($period_id)
    {
        $this->load->model('listings_booking/models/Listings_booking_model');

        $this->Listings_booking_model->set_format_settings('use_format', false);
        $period = $this->Listings_booking_model->get_period_by_id($period_id);
        $this->Listings_booking_model->set_format_settings('use_format', true);

        $this->Listings_booking_model->delete_period($period_id);
        $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_period_deleted', 'listings_booking'));

        $this->view->setRedirect(site_url() . 'admin/listings_booking/edit/' . $period['id_listing'] . '/calendar/2');
    }

    /**
     * Remove calendar period by ajax
     *
     * @param integer $period_id period identifier
     *
     * @return void
     */
    public function ajax_period_delete($period_id)
    {
        $return = array('error' => '', 'success' => '');

        $this->load->model('listings_booking/models/Listings_booking_model');

        $this->Listings_booking_model->set_format_settings('use_format', false);
        $period = $this->Listings_booking_model->get_period_by_id($period_id);
        $this->Listings_booking_model->set_format_settings('use_format', true);

        $this->Listings_booking_model->delete_period($period_id);
        $return['success'] = l('success_period_deleted', 'listings_booking');

        $this->view->assign($return);
        
        $this->view->render();
    }

    /**
     * Refresh price block of booking period
     *
     * @param integer $listing_id listing identifier
     *
     * @return void
     */
    public function ajax_get_booking_price($listing_id)
    {
        $return = array('price' => 0);

        $this->load->model('Listings_model');

        $this->Listings_model->set_format_settings('use_format', false);
        $listing = $this->Listings_model->get_listing_by_id($listing_id);
        $this->Listings_model->set_format_settings('use_format', true);

        $date_start = $this->input->post('date_start', true);
        $date_end = $this->input->post('date_end', true);

        $guests = intval($this->input->post('guests', true));
        if (!$guests) {
            $guests = 1;
        }

        if (!$date_start || !$date_end) {
            if ($listing['price_negotiated']) {
                $return['price'] = l('text_negotiated_price_rent', 'listings_booking');
            } else {
                if ($listing['price_reduced'] > 0) {
                    $price_value = $listing['price_reduced'];
                } else {
                    $price_value = $listing['price'];
                }

                $this->load->helper('start');
                $price_arr = array('value' => $price_value, 'cur_gid' => $listing['gid_currency']);
                $return['price'] = l('text_price_from', 'listings_booking') . ' ' . currency_format_output($price_arr);
            }
        } else {
            $this->load->model('listings_booking/models/Listings_booking_model');
            $price_value = $this->Listings_booking_model->get_period_price($listing, $date_start, $date_end, $guests);
            if ($price_value) {
                $this->load->helper('start');
                $price_arr = array('value' => $price_value, 'cur_gid' => $listing['gid_currency']);
                $return['price'] = currency_format_output($price_arr);
            } else {
                $return['price'] = l('text_negotiated_price_rent', 'listings_booking');
            }
        }

        $this->view->assign($return);
        
        $this->view->render();
    }

}
