<?php

namespace Pg\Modules\Listings_booking\Controllers;

/**
 * Listings module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */

/**
 * Listings api side controller
 *
 * @package 	PG_RealEstate
 * @subpackage 	Listings
 *
 * @category	controllers
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Api_Listings_booking extends \Controller
{

    /**
     * Constructor
     *
     * @return Api_Listings
     */
    public function __construct()
    {
        parent::__construct();
        $this->load->model('Listings_model');
        $this->load->model('Listings_booking_model');        
        $this->Listings_model->set_format_settings('get_price_str', false);
    }

    /**
     * Calculate price of booking request
     *
     * @param integer $listing_id listing identifier
     *
     * @return void
     */
    public function get_booking_price($listing_id)
    {
        $this->Listings_model->set_format_settings('use_format', false);
        $listing = $this->Listings_model->get_listing_by_id($listing_id);
        $this->Listings_model->set_format_settings('use_format', true);

        $date_start = $this->input->post('date_start', true);
        $date_end = $this->input->post('date_end', true);

        $guests = intval($this->input->post('guests', true));
        if (!$guests) {
            $guests = 1;
        }

        if (!$date_start || !$date_end) {
            if ($listing['price_negotiated']) {
                $format_price_value = l('text_negotiated_price_rent', 'listings');
            } else {
                if ($listing['price_reduced'] > 0) {
                    $price_value = $listing['price_reduced'];
                } else {
                    $price_value = $listing['price'];
                }

                $this->load->helper('start');
                $price_arr = array('value' => $price_value, 'cur_gid' => $listing['gid_currency']);
                $format_price_value = l('text_price_from', 'listings') . ' ' . currency_format_output($price_arr);
            }
        } else {
            $price_value = $this->Listings_booking_model->get_period_price($listing, $date_start, $date_end, $guests);
            if ($price_value) {
                $this->load->helper('start');
                $price_arr = array('value' => $price_value, 'cur_gid' => $listing['gid_currency']);
                $format_price_value = currency_format_output($price_arr);
            } else {
                $format_price_value = l('text_negotiated_price_rent', 'listings');
            }
        }

        $this->set_api_content('data', strip_tags($format_price_value));
    }

    /**
     * Get bookings requests of my listings
     *
     * @param string  $status          order status
     * @param string  $order           sorting order
     * @param string  $order_direction order direction
     * @param integer $page            page of results
     *
     * @return void
     */
    public function orders($status = 'wait', $order = 'created', $order_direction = 'DESC', $page = 1)
    {        
        $current_settings = isset($_SESSION['listings_booking']) ? $_SESSION['listings_booking'] : array();
        if (!isset($current_settings['data'])) {
            $current_settings['data'] = array();
        }
        if (!isset($current_settings['order'])) {
            $current_settings['order'] = 'created';
        }
        if (!isset($current_settings['order_direction'])) {
            $current_settings['order_direction'] = 'DESC';
        }
        if (!isset($current_settings['page'])) {
            $current_settings['page'] = '1';
        }
        if (!isset($current_settings['filters'])) {
            $current_settings['filters'] = array('status' => 'wait');
        }

        $user_id = $this->session->userdata('user_id');

        $data = $this->input->post('data', true);
        if (!empty($data)) {
            $current_settings['filters'] = $data;
        }
        if ($status) {
            if ($status != $current_settings['filters']['status']) {
                $page = 1;
            }
            $current_settings['filters']['status'] = $status;
        }
        $filters = $current_settings['filters'];
        //if(isset($filters['keyword'])) $filters['keyword_mode'] = 1;
        $filters['not_user'] = $user_id;
        $filters['owners'] = $user_id;

        $order = trim(strip_tags($order));
        if (!empty($order)) {
            $current_settings['order'] = $order;
        }
        $order = $current_settings['order'];

        $order_direction = strtoupper(trim(strip_tags($order_direction)));
        if (!empty($order_direction)) {
            $current_settings['order_direction'] = $order_direction;
        }
        $order_direction = $current_settings['order_direction'];

        
        //$page = intval($page);
        $page = intval($this->input->post('page', true));        
        if (!empty($page)) {
            $current_settings['page'] = $page;
        }
        $page = $current_settings['page'];

        if ($order_direction != 'ASC') {
            $order_direction = 'DESC';
        }

        $items_on_page = $this->pg_module->get_module_config('start', 'index_items_per_page');
        $orders_count = $this->Listings_booking_model->getPeriodsCount($filters);        
        $this->load->helper('sort_order');
        $current_settings['page'] = get_exists_page_number($page, $orders_count, $items_on_page);
        $_SESSION['listings_booking'] = $current_settings;

        $orders = array();
        if ($orders_count > 0) {
            $order_array = array();
            switch ($order) {
                case 'created':
                    $order_array['date_created'] = $order_direction;
                    break;
                case 'modified':
                    $order_array['date_modified'] = $order_direction;
                    break;
                default:
                    $order_array[$order] = $order_direction;
                    break;
            }

            $this->Listings_booking_model->set_format_settings('get_listing', true);
            $orders = $this->Listings_booking_model->get_periods_list($filters, $current_settings['page'], $items_on_page, $order_array);
            
            $this->Listings_booking_model->set_format_settings('get_listing', false);

            $date_formats = $this->pg_date->get_format('date_literal', 'st');

            $this->load->helper('start');
            $this->load->helper('date_format');

            foreach ($orders as $i => $order) {
                $order['date_start_output'] = tpl_date_format($order['date_start'], $date_formats);
                $order['date_end_output'] = tpl_date_format($order['date_end'], $date_formats);
                if ($order['price']) {
                    $order['price_output'] = str_replace('&nbsp;', ' - ', strip_tags(currency_format_output(array('value' => $order['price'], 'gid' => $order['listing']['gid_currency']))));
                } else {
                    $order['price_output'] = l('text_booking_price_unknown', 'listings');
                }
                $orders[$i] = $order;
            }
        }

        $data = array(
            'settings' => $current_settings,
            'orders' => $orders,
            'orders_count' => $orders_count,
        );

        $this->set_api_content('data', $data);
        
        //$this->set_api_content('settings', $current_settings);
        //$this->set_api_content('orders', $orders);
        //$this->set_api_content('orders_count', $orders_count);
    }

    /**
     * Get my booking requests
     *
     * @return void
     */
    public function requests()
    {
        $current_settings = isset($_SESSION['listings_requests']) ? $_SESSION['listings_requests'] : array();
        if (!isset($current_settings['data'])) {
            $current_settings['data'] = array();
        }
        if (!isset($current_settings['order'])) {
            $current_settings['order'] = 'created';
        }
        if (!isset($current_settings['order_direction'])) {
            $current_settings['order_direction'] = 'DESC';
        }
        if (!isset($current_settings['page'])) {
            $current_settings['page'] = '1';
        }
        if (!isset($current_settings['filters'])) {
            $current_settings['filters'] = array();
        }

        $data = $this->input->post('data', true);
        if (!empty($data)) {
            $current_settings['filters'] = $data;
        }

        $filters = $current_settings['filters'];

        $filters['users'] = $this->session->userdata('user_id');

        $order = trim(strip_tags($this->input->post('order', true)));
        if (!empty($order)) {
            $current_settings['order'] = $order;
        }
        $order = $current_settings['order'];

        $order_direction = strtoupper(trim(strip_tags($this->input->post('order_direction'))));
        if (!empty($order_direction)) {
            $current_settings['order_direction'] = $order_direction;
        }
        $order_direction = $current_settings['order_direction'];

        $page = intval($this->input->post('page', true));
        if (!empty($page)) {
            $current_settings['page'] = $page;
        }
        $page = $current_settings['page'];

        if ($order_direction != 'ASC') {
            $order_direction = 'DESC';
        }

        $items_on_page = $this->pg_module->get_module_config('start', 'index_items_per_page');
        $requests_count = $this->Listings_booking_model->getPeriodsCount($filters);
        $this->load->helper('sort_order');
        $current_settings['page'] = get_exists_page_number($page, $requests_count, $items_on_page);
        $_SESSION['listings_requests'] = $current_settings;
        $requests = array();
        if ($requests_count > 0) {
            $order_array = array();
            switch ($order) {
                case 'modified':
                    $order_array['date_modified'] = $order_direction;
                    break;
                default:
                    $order_array[$order] = $order_direction;
                    break;
            }
            $this->Listings_booking_model->set_format_settings('get_listing', true);
            $requests = $this->Listings_booking_model->get_periods_list($filters, $current_settings['page'], $items_on_page, $order_array);
            $this->Listings_booking_model->set_format_settings('get_listing', false);

            $date_formats = $this->pg_date->get_format('date_literal', 'st');

            $this->load->helper('start');
            $this->load->helper('date_format');

            foreach ($requests as $i => $request) {
                $request['date_start_output'] = tpl_date_format($request['date_start'], $date_formats);
                $request['date_end_output'] = tpl_date_format($request['date_end'], $date_formats);
                if ($request['price']) {
                    $request['price_output'] = str_replace('&nbsp;', ' - ', strip_tags(currency_format_output(array('value' => $request['price'], 'gid' => $request['listing']['gid_currency']))));
                } else {
                    $request['price_output'] = l('text_booking_price_unknown', 'listings');
                }
                $requests[$i] = $request;
            }
        }

        $data = array(
            'settings' => $current_settings,
            'requests' => $requests,
            'requests_count' => $requests_count,
        );

        $this->set_api_content('data', $data);
    }

    /**
     * Remove booking request of my listing
     *
     * @param integer $order_id order identifier
     *
     * @return void
     */
    public function order_delete($order_id)
    {
        $user_id = $this->session->userdata('user_id');

        $this->Listings_booking_model->set_format_settings('get_listing', true);
        $order = $this->Listings_booking_model->get_period_by_id($order_id);
        $this->Listings_booking_model->set_format_settings('get_listing', false);

        if ($user_id != $order['listing']['id_user']) {
            $this->set_api_content('error', l('api_error_not_access', 'listings'));
            return;
        }

        if (!empty($order_id)) {
            $this->Listings_booking_model->delete_period($order_id);
            $this->set_api_content('success', l('success_order_deleted', 'listings'));
        }
    }

    /**
     * Remove my booking request
     *
     * @param integer $request_id request identifier
     *
     * @return void
     */
    public function request_delete($request_id)
    {
        $user_id = $this->session->userdata('user_id');

        $this->Listings_booking_model->set_format_settings('get_listing', true);
        $request = $this->Listings_booking_model->get_period_by_id($request_id);
        $this->Listings_booking_model->set_format_settings('get_listing', false);

        if ($user_id != $request['id_user']) {
            $this->set_api_content('error', l('api_error_not_access', 'listings'));
            return;
        }

        if (!empty($request_id)) {
            $this->Listings_booking_model->delete_period($request_id);
            $this->set_api_content('success', l('success_request_deleted', 'listings'));
        }
    }

    /**
     * Approve booking request of my listing
     *
     * @param integer $order_id order identifier
     *
     * @return void
     */
    public function order_approve($order_id)
    {
        $user_id = $this->session->userdata('user_id');
        $order = $this->Listings_booking_model->get_period_by_id($order_id);

        if ($user_id != $order['id_owner']) {
            $this->set_api_content('error', l('api_error_not_access', 'listings'));
            return;
        }
        if (!empty($order_id)) {
            $is_intersect = $this->Listings_booking_model->is_intersect_exists_orders(
                $order["id_listing"], 0, $order['date_start'], $order['date_end'], $order_id
            );
            if ($is_intersect) {
                $this->set_api_content('error', l('error_intersect_calendar_periods', 'listings'));
            } else {
                $this->Listings_booking_model->approve_period($order_id);
                $this->set_api_content('success', l('success_order_approved', 'listings'));
            }
        }
    }

    /**
     * Decline booking request of my listing
     *
     * @param integer $order_id order identifier
     *
     * @return void
     */
    public function order_decline($order_id)
    {
        $user_id = $this->session->userdata('user_id');

        $this->Listings_booking_model->set_format_settings('get_listing', true);
        $order = $this->Listings_booking_model->get_period_by_id($order_id);
        $this->Listings_booking_model->set_format_settings('get_listing', false);

        if ($user_id != $order['id_owner']) {
            $this->set_api_content('error', l('api_error_not_access', 'listings'));
            return;
        }

        if (!empty($order_id)) {
            $this->Listings_booking_model->decline_period($order_id);
            $this->set_api_content('success', l('success_order_declined', 'listings'));
        }
    }

    /**
     * Save booking request
     *
     * @param integer $listing_id listing identifier
     * @param integer $order_id   order identifier
     *
     * @return void
     */
    public function save_order($listing_id, $order_id = null)
    {
        $user_id = $this->session->userdata('user_id');

        $this->Listings_model->set_format_settings('use_format', false);
        $listing = $this->Listings_model->get_listing_by_id($listing_id);
        $this->Listings_model->set_format_settings('use_format', true);

        $post_period = $this->input->post('period', true);
        $post_period['id_listing'] = $listing_id;
        $post_period['id_user'] = $user_id;

        $validate_data = $this->Listings_booking_model->validate_period(
            null, $post_period, true
        );
        if (!empty($validate_data['errors'])) {
            $this->set_api_content('error', $validate_data['errors']);
        } else {
            $guests = $validate_data['data']['guests'] ? $validate_data['data']['guests'] : 1;
            $validate_data['data']['price'] = $this->Listings_booking_model->get_period_price(
                $listing, $validate_data['data']['date_start'], $validate_data['data']['date_end'], $guests
            );
            $this->Listings_booking_model->save_period(null, $validate_data['data']);
            $this->set_api_content('success', l('success_order_added', 'listings'));
        }
    }
    
    public function validate_order($listing_id)
    {
        $user_id = $this->session->userdata('user_id');

        $this->Listings_model->set_format_settings('use_format', false);
        $listing = $this->Listings_model->get_listing_by_id($listing_id);
        $this->Listings_model->set_format_settings('use_format', true);

        $post_period = $this->input->post('period', true);
        $post_period['id_listing'] = $listing_id;
        $post_period['id_user'] = $user_id;

        $validate_data = $this->Listings_booking_model->validate_period(
            null, $post_period, true
        );
        if (!empty($validate_data['errors'])) {
            $this->set_api_content('error', $validate_data['errors']);
        } else {            
            $this->set_api_content('success', l('success_date_available', 'listings_booking'));
        }
    }
        
    /**
     * Save calendar period
     *
     * @param integer $listing_id listing identifier
     * @param integer $period_id  period identifier
     *
     * @return void
     */
    public function save_period($listing_id, $period_id = null)
    {
        $user_id = $this->session->userdata('user_id');

        $this->Listings_model->set_format_settings('use_format', false);
        $listing = $this->Listings_model->get_listing_by_id($listing_id);
        $this->Listings_model->set_format_settings('use_format', true);

        if (!$listing || $listing['id_user'] != $user_id) {
            $this->set_api_content('error', l('api_error_not_access', 'listings'));
            return;
        }

        $post_period = $this->input->post('period', true);
        $post_period['id_listing'] = $listing_id;
        $post_period['id_user'] = $user_id;

        switch ($listing['price_type']) {
            case 1:
                $date_start_alt = $this->input->post('date_start_alt', true);
                if ($date_start_alt) {
                    $post_period['date_start'] = $date_start_alt;
                }

                $date_end_alt = $this->input->post('date_end_alt', true);
                if ($date_end_alt) {
                    $post_period['date_end'] = $date_end_alt;
                }
                break;
            case 2:
                $date_start_month = $this->input->post('date_start_month', true);
                $date_start_year = $this->input->post('date_start_year', true);
                $post_period['date_start'] = $date_start_year . '-' . $date_start_month . '-01';

                $date_end_month = $this->input->post('date_end_month', true);
                $date_end_year = $this->input->post('date_end_year', true);
                $post_period['date_end'] = $date_end_year . '-' . $date_end_month . '-01';
                break;
        }

        $validate_data = $this->Listings_booking_model->validate_period(
            $period_id, $post_period
        );
        if (!empty($validate_data['errors'])) {
            $this->set_api_content('error', $validate_data['errors']);
        } else {
            $period_id = $this->Listings_booking_model->save_period(
                $period_id, $validate_data['data']
            );
            $this->set_api_content('success', l('success_period_added', 'listings'));
            $this->set_api_content('id', $period_id);
            $data = $this->Listings_booking_model->format_period($validate_data['data']);
            $this->set_api_content('data', $data);
        }
    }

    /**
     * Remove calendar period
     *
     * @param integer $period_id period identifier
     *
     * @return void
     */
    public function period_delete($period_id)
    {
        $user_id = $this->session->userdata('user_id');
        if (!$period_id) {
            $this->set_api_content('error', l('api_error_not_access', 'listings'));
            return;
        }

        $this->Listings_booking_model->set_format_settings('use_format', false);
        $period = $this->Listings_booking_model->get_period_by_id($period_id);
        $this->Listings_booking_model->set_format_settings('use_format', true);

        if ($user_id != $period['id_user']) {
            $this->set_api_content('error', l('api_error_not_access', 'listings'));
            return;
        }

        $this->Listings_booking_model->delete_period($period_id);
        $this->set_api_content('success', l('success_period_deleted', 'listings'));
    }

    /**
     * Booking listing
     *
     * @param integer $listing_id listing identifier
     * @param integer $order_id   order identifier
     *
     * @return void
     */
    public function order($listing_id, $order_id = null)
    {
        $user_id = $this->session->userdata('user_id');

        $listing = $this->Listings_model->get_listing_by_id($listing_id);

        $booking = $listing['id_user'] != $user_id;

        $post_period = $this->input->post('period', true);
        $post_period['id_listing'] = $listing_id;
        $post_period['id_user'] = $user_id;
        if (!$booking) {
            $post_period['status'] = 'book';
        }

        $validate_data = $this->Listings_booking_model->validate_period(
            $order_id, $post_period, $booking
        );
        if (!empty($validate_data['errors'])) {
            $this->set_api_content('errors', $validate_data['errors']);
        } else {
            $guests = $validate_data['data']['guests'] ? $validate_data['data']['guests'] : 1;
            $validate_data['data']['price'] = $this->Listings_booking_model->get_period_price(
                $listing, $validate_data['data']['date_start'], $validate_data['data']['date_end'], $guests
            );
            if ($order_id) {
                $this->set_api_content('success', l('success_order_updated', 'listings'));
            } else {
                $this->set_api_content('success', l('success_order_added', 'listings'));
            }
            $order_id = $this->Listings_booking_model->save_period(
                $order_id, $validate_data['data'], $booking
            );
            $result['id'] = $validate_data['data']['id'] = $order_id;
        }
    }

}
