<?php

use Pg\Modules\Memberships\Models\Memberships_model;

/**
 * Memberships module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
/**
 * Memberships helpers
 *
 * @package 	PG_RealEstate
 * @subpackage 	Memberships
 *
 * @category	helpers
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
if (!function_exists('memberships_list')) {

    /**
     * Block of memberships
     *
     * Usage in template lite: {block name=memberships_list module=memberships}
     *
     * @param array memberships
     * @param int id
     * @param string gid
     * @param int my
     * @param bool show_purchased
     *
     * @return string
     */
    function memberships_list($params = array())
    {
        $ci = &get_instance();
        $where = array();
        if (!empty($params['memberships'])) {
            $memberships = $params['memberships'];
        } else {
            $where['is_active'] = 1;
            if (!empty($params['gid'])) {
                $where['gid'] = $params['gid'];
            }
            if (!empty($params['id'])) {
                $where['id'] = $params['id'];
            }
            if ($ci->session->userdata('auth_type') == 'user') {
                $where['user_type'] = $ci->session->userdata('user_type');
            }

            if (!empty($params['template_gid'])) {
                $ci->load->model('Services_model');

                $where_s = array(
                    'where' => array(
                        'type'   => 'membership',
                        'status' => 1,
                    ),
                    'where_in' => array(
                        'template_gid' => (array) $params['template_gid'],
                    ),
                );
                
                $services = $ci->Services_model->get_service_list($where_s);
                if (!empty($services)) {
                    $membership_ids = array();
                    foreach ($services as $service) {
                        $memberships_ids[] = $service['id_membership'];
                    }
                    $where['ids'] = array_unique($memberships_ids);
                }
            }

            $ci->load->model('Memberships_model');
            $ci->Memberships_model->setFormatSettings('get_services', true);
            $memberships = $ci->Memberships_model->formatMemberships(
                $ci->Memberships_model->getMembershipsList($where, null, null, array('price' => 'ASC'))
            );
        }
        
        if (empty($memberships)) {
            return '';
        } 

        $all_templates = Memberships_model::getTemplatesByMemberships($memberships);

        $highlight = array();
        $user_memberships_by_id = array();

        if (!empty($all_templates)) {
            $user_id = $ci->session->userdata('user_id');

            $ci->load->model('memberships/models/Memberships_users_model');
            $user_memberships = $ci->Memberships_users_model->getUserMembershipsList(
                    null, array('where' => array('id_user' => $user_id, 'is_active' => 1))
            );

            foreach ($user_memberships as $user_membership) {
                if (isset($params['highlight_purchased'])) {
                    $highlight[$user_membership['id_membership']] = $params['highlight_purchased'];
                }
                $user_memberships_by_id[$user_membership['id_membership']] = $user_membership;
            }
        }

        $ci->view->assign('all_templates', $all_templates);
        $ci->view->assign('memberships_count', count($memberships));
        $ci->view->assign('block_memberships', $memberships);
        $ci->view->assign('highlight', $highlight);
        $ci->view->assign('user_memberships', $user_memberships_by_id);
        $page_data["date_format"] = $ci->pg_date->get_format('date_time_literal', 'st');
        $ci->view->assign("page_data", $page_data);

        return $ci->view->fetch('helper_memberships_list', 'user', Memberships_model::MODULE_GID);
    }
}

if (!function_exists('user_memberships_list')) {

    /**
     * Block of user memberships
     *
     * Usage in template lite: {block name=user_memberships_list module=memberships}
     *
     * @param array $params block parameters
     *
     * @return string
     */
    function user_memberships_list($params = array())
    {
        $ci = &get_instance();
        if (!$ci->session->userdata('auth_type') === 'user') {
            return '';
        }
        $id_user = $ci->session->userdata('user_id');
        if (!$id_user) {
            return '';
        }
        $order_by['till_date'] = 'DESC';

        if (!empty($params['memberships'])) {
            $user_memberships = $params['memberships'];
        } else {
            $where['where']['id_user'] = $id_user;
            $ci->load->model('Memberships_model');
            $ci->load->model('memberships/models/Memberships_users_model');
            $ci->Memberships_model->setFormatSettings('get_services', true);
            $user_memberships = $ci->Memberships_users_model->getUserMembershipsList($order_by, $where);
            $ci->Memberships_model->setFormatSettings('get_services', false);
        }

        if (!empty($user_memberships)) {
            $sort_user_memberships = array('active' => array(), 'inactive' => array());
            foreach ($user_memberships as $key => $user_memberships) {
                if ($user_memberships['is_active']) {
                    $sort_user_memberships['active'][] = $user_memberships;
                } else {
                    $sort_user_memberships['inactive'][] = $user_memberships;
                }
            }
            $sorted_user_memberships = array_merge($sort_user_memberships['active'], $sort_user_memberships['inactive']);
        }

        $date_formats = array(
            'date_format'      => $ci->pg_date->get_format('date_literal', 'st'),
            'date_time_format' => $ci->pg_date->get_format('date_time_literal', 'st'),
        );
        $ci->view->assign('block_user_memberships', $sorted_user_memberships);
        $ci->view->assign('block_user_memberships_date_formats', $date_formats);

        return $ci->view->fetch('helper_user_memberships_list', 'user', Memberships_model::MODULE_GID);
    }
}
