<?php

namespace Pg\Modules\Mobile\Controllers;

/**
 * Mobile module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */

/**
 * Mobile version API controller
 *
 * @package 	PG_RealEstate
 * @subpackage 	Mobile
 *
 * @category	controllers
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Api_Mobile extends \Controller
{

    /**
     * Constructor
     * 
     * @return Api_Mobile
     */
    public function __construct()
    {
        parent::__construct();
        $this->load->model('Listings_model');
        $this->Listings_model->set_format_settings('get_price_str', false);        
    }

    /**
     * Init mobile version
     * 
     * @param integer $lang_id language identifier
     *
     * @return void
     */
    public function init()
    {
        $this->load->library('pg_theme');
        $theme_data = $this->pg_theme->format_theme_settings($this->router->class);
        $data['data']['site_url'] = site_url();
        $data['data']['logo'] = $theme_data['mini_logo']['path'];
        $data['data']['mapProvider'] = $this->pg_module->get_module_config('mobile', 'geomap_driver');
        $data['data']['indexView'] = $this->pg_module->get_module_config('mobile', 'index_view_block');
        $data['data']['is_demo'] = $_ENV['DEMO_MODE'] ? 1 : 0;

        $data['data']['is_payments'] = $this->pg_module->is_module_active('users_payments');
        $data['data']['is_services'] = $this->pg_module->is_module_active('services');
        $data['data']['is_users_payments'] = $this->pg_module->is_module_active('users_payments');
        $data['data']['is_listings_booking'] = $this->pg_module->is_module_active('listings_booking');
        $data['data']['is_social_networking'] = $this->pg_module->is_module_active('social_networking');
        $data['data']['is_users'] = $this->pg_module->is_module_active('users');


        // social networking
        $params = array('where' => array('status' => 1));
        $order = array('id' => 'ASC');
        if ($data['data']['is_social_networking']) {
            $this->load->model('social_networking/models/Social_networking_services_model');
            $services = $this->Social_networking_services_model->get_services_list($order, $params);
            if (!$services) {
                $services = array();
            }
            foreach ($services as $service) {
                $data['data']['social_widgets'][] = $service['gid'];
            }
        }
        $current_lang = $this->pg_language->get_lang_by_id($this->pg_language->current_lang_id);
        $data['language'] = $current_lang['code'];

        $download_apps_used = $this->pg_module->get_module_config('mobile', 'download_apps_used');
        if ($download_apps_used) {
            $download_apps = array();
            $download_app_android = $this->pg_module->get_module_config('mobile', 'download_apps_android');
            if ($download_app_android) {
                $download_apps[] = 'android';
            }
            $download_app_ios = $this->pg_module->get_module_config('mobile', 'download_apps_ios');
            if ($download_app_ios) {
                $download_apps[] = 'iphone|ipad';
            }
            if (!empty($download_apps)) {
                $data['data']['download_apps'] = implode('|', $download_apps);
            }
        }

        $theme_type = $this->pg_theme->get_current_theme_type();
        $active_settings = $this->pg_theme->return_active_settings($theme_type);
        $theme = $active_settings["theme"];
        $scheme = $active_settings["scheme"];
        $data['data']['css_url'] = $this->pg_theme->theme_default_url . $theme . '/sets/' . $scheme . '/css/mobile-' . $current_lang['rtl'] . '.css';
        $data['data']['rtl'] = $current_lang['rtl'];

        $data['login'] = $this->session->userdata('auth_type') == 'user';
        $data['data']['user_types'] = array();
        $data['data']['login_enabled'] = false;
        $data['data']['register'] = array();
        $data['data']['register_confirm'] = array();
        
        //Map Settings for mobapp
        if ($this->pg_module->is_module_active('dynamic_block_map')) {            
            $this->load->model('Dynamic_block_map_model');
            $data['data']['map_settings'] = $this->Dynamic_block_map_model->getMapSettings();            
        }

        if ($data['data']['is_users']) {
            $this->load->model('Users_model');
            $user_types = $this->Users_model->getUserTypes();
            foreach ($user_types as $user_type) {
                $enabled = $this->pg_module->get_module_config('users', $user_type . '_login_enabled');
                if ($enabled) {
                    $data['data']['login_enabled'] = true;
                }

                $enabled = $this->pg_module->get_module_config('users', $user_type . '_register_enabled');
                if ($enabled) {
                    $data['data']['register'][] = $user_type;
                }

                $enabled = $this->pg_module->get_module_config('users', $user_type . '_register_mail_confirm');
                if ($enabled) {
                    $data['data']['register_confirm'][] = $user_type;
                }

                $enabled = $this->pg_module->get_module_config('users', $user_type . '_search_enabled');
                if ($enabled) {
                    $data['data']['user_types'][] = $user_type;
                }
            }
        }

        $this->load->model('Listings_model');
        $data['data']['operation_types'] = $this->Listings_model->get_operation_types(true);

        $this->view->assign('data', $data);
    }

    /**
     * Get token
     *
     * @return void
     */
    public function get_token()
    {
        $this->load->model('Mobile_model');

        $data = array(
            'email' => trim(strip_tags($this->input->get_post('email', true))),
            'password' => trim(strip_tags($this->input->get_post('password', true))),
        );
        if (!empty($data['email']) || !empty($data['password'])) {
            $this->load->model('users/models/Auth_model');
            $validate = $this->Auth_model->validate_login_data($data);
            if (!empty($validate["errors"])) {
                $errors = array(array_shift($validate["errors"]));
                $this->session->sess_destroy();
                $this->view->assign('errors', $errors);
            } else {
                $login_return = $this->Auth_model->login_by_email_password($validate["data"]["email"],
                                                                           $this->Users_model->encode_password($validate["data"]["password"]));
                if (!empty($login_return["errors"])) {
                    $errors = $login_return["errors"];
                    $this->view->assign('errors', $errors);
                    $this->session->sess_destroy();
                } else {
                    $this->load->model('Mobile_model');
                    $user_id = $this->session->userdata('user_id');
                    $secret = $this->Mobile_model->save_user_secret($user_id);

                    $lang_id = $this->session->userdata('lang_id');
                    $current_lang = $this->pg_language->get_lang_by_id($lang_id);

                    if (empty($current_lang)) {
                        $lang_id = $this->pg_language->current_lang_id = $lang_id;
                        $current_lang = $this->pg_language->get_lang_by_id($lang_id);
                    } else {
                        $this->pg_language->current_lang_id = $lang_id;
                    }

                    $data = array(
                        'token' => $this->session->sess_create_token(),
                        'user_id' => $user_id,
                        'secret' => $secret,
                        'language' => $current_lang['code'],
                        'l' => $this->pg_language->pages->return_module('mobile', $lang_id),
                        'd' => $this->pg_language->ds->return_module('mobile', $lang_id),
                    );
                    $this->view->assign('data', $data);
                }
            }
        } else {
            $data = array(
                'user_id' => 0,
                'secret' => '',
            );

            $user_id = intval($this->input->get_post('user_id', true));
            $secret = trim(strip_tags($this->input->get_post('user_secret', true)));

            if ($user_id && $secret) {
                $is_user_secret = $this->Mobile_model->is_user_secret($user_id, $secret);
                if ($is_user_secret) {
                    $this->load->model('Users_model');
                    $this->Users_model->set_format_settings('get_safe', false);
                    $user = $this->Users_model->get_user_by_id($user_id);
                    $this->Users_model->set_format_settings('get_safe', true);
                    if (!empty($user)) {
                        $this->load->model('users/models/Auth_model');
                        $login_return = $this->Auth_model->login_by_email_password($user['email'], $user['password']);
                        if (empty($login_return["errors"])) {
                            $data = array(
                                'user_id' => $user_id,
                                'secret' => $secret,
                            );
                        }
                    }
                }
            }

            $lang_id = $this->session->userdata('lang_id');
            $current_lang = $this->pg_language->get_lang_by_id($lang_id);

            if (empty($current_lang)) {
                $lang_id = $this->pg_language->current_lang_id = $lang_id;
                $current_lang = $this->pg_language->get_lang_by_id($lang_id);
            } else {
                $this->pg_language->current_lang_id = $lang_id;
            }

            $data['language'] = $current_lang['code'];
            $data['l'] = $this->pg_language->pages->return_module('mobile', $lang_id);
            $data['d'] = $this->pg_language->ds->return_module('mobile', $lang_id);
            $data['token'] = $this->session->sess_create_token();
            $this->view->assign('data', $data);
        }
    }

    /**
     * Process backend request
     *
     * @return void
     */
    public function backend()
    {
        $data = array();

        if ($this->session->userdata('auth_type') == 'user') {
            $user_id = $this->session->userdata('user_id');
        } else {
            $user_id = 0;
        }

        $modules = $this->input->post('gids', true);
        if ($modules) {
            foreach ((array) $modules as $module) {
                switch ($module) {
                    case 'mailbox':
                        if (!$user_id) {
                            break;
                        }

                        if ($this->pg_module->is_module_active('Mailbox_model')) {
                            break;
                        }

                        $this->load->model('Mailbox_model');
                        $data['mailbox']['new_messages_count'] = 0;
                        $chats = $this->Mailbox_model->get_chats_list(array('where' => array('id_com_user' => $user_id)));
                        foreach ($chats as $chat) {
                            $data['mailbox']['new_messages_count'] += $chat['new'];
                        }
                        break;
                    case 'message':
                        if (!$user_id) {
                            break;
                        }

                        if ($this->pg_module->is_module_active('Mailbox_model')) {
                            break;
                        }

                        $this->load->model('Mailbox_model');
                        $chats = $this->input->post('chats', true);

                        if (!$chats) {
                            return;
                        }
                        foreach ($chats as $chat) {
                            $data['mailbox']['chats'][$chat]['messages'] = array();
                        }

                        $date_format = $this->pg_date->get_format('date_time_literal', 'st');

                        $this->load->helper('date_format');

                        $messages = $this->Mailbox_model->get_messages(array('where' => array('is_new' => 1), 'where_in' => array('id_chat' => (array) $chats)));
                        foreach ($messages as $message) {
                            $message['date_add_output'] = tpl_date_format($message['date_add'], $date_format);
                            $data['mailbox']['chats'][$message['id_chat']]['messages'][] = $message;
                        }
                        foreach ($data['mailbox']['chats'] as $chat_id => $chat_data) {
                            if (empty($chat_data['messages'])) {
                                continue;
                            }
                            $ids = array();
                            foreach ($chat_data['messages'] as $message) {
                                $ids[] = $message['id'];
                            }
                            $this->Mailbox_model->set_read_messages($chat_id, $ids);
                        }
                        break;
                }
            }
        }
        $this->view->assign('data', $data);
        $this->view->render();
    }

    /**
     * Get data for index page
     *
     * @return void
     */
    public function index()
    {
        $data = array();

        $type = $this->pg_module->get_module_config('mobile', 'index_type_block');
        $view = $this->pg_module->get_module_config('mobile', 'index_view_block');
        $count = $this->pg_module->get_module_config('mobile', 'index_count_block');

        $filters = array('active' => 1);
        $order_by = array();

        $data['type'] = $type;
        $data['view'] = $view;

        switch ($type) {
            case 'latest':
                $order_by = array('date_created' => 'DESC');
                break;
            case 'featured':
                $filters['featured'] = 1;
                $order_by = array('RAND()' => '');
                break;
            case 'sale':
                $filters['type'] = 'sale';
                $order_by = array('date_created' => 'DESC');
                break;
            case 'rent':
                $filters['type'] = 'rent';
                $order_by = array('date_created' => 'DESC');
                break;
        }

        $this->load->model('Listings_model');

        $this->Listings_model->set_format_settings('get_user', false);
        $listings = $this->Listings_model->get_listings_list($filters, 1, $count, $order_by);
        $this->Listings_model->set_format_settings('get_user', true);

        foreach ($listings as $i => $listing) {
            $data['listings'][$i] = array(
                'listing_id' => $listing['id'],
                'logo_image' => $listing['media']['photo']['thumbs'],
            );

            unset($listing['id']);
            unset($listing['logo_image']);
            unset($listing['media']);
            unset($listing['listing_file']);
            unset($listing['listing_file_date']);
            unset($listing['listing_video']);
            unset($listing['listing_video_image']);
            unset($listing['listing_video_data']);

            foreach ($listing as $name => $value) {
                $data['listings'][$i]['listing_' . $name] = $value;
            }
        }

        $this->view->assign('data', $data);
        $this->view->render();
    }

    /**
     * Get languages data
     *
     * @param integer $lang_id language identifier
     *
     * @return void
     */
    public function languages($lang_id = null)
    {
        if (!$lang_id || !$this->pg_language->is_active($lang_id)) {
            $lang_id = $this->pg_language->current_lang_id;
        }

        $lang = $this->pg_language->get_lang_by_id($lang_id);
        if (empty($lang)) {
            $lang_id = $this->pg_language->current_lang_id;
            $lang = $this->pg_language->get_lang_by_id($lang_id);
        }

        $data = array(
            'l' => $this->pg_language->pages->return_module('mobile', $lang_id),
            'd' => $this->pg_language->ds->return_module('mobile', $lang_id),
        );

        $this->load->library('pg_theme');
        $theme_type = $this->pg_theme->get_current_theme_type();
        $active_settings = $this->pg_theme->return_active_settings($theme_type);
        $theme = $active_settings["theme"];
        $scheme = $active_settings["scheme"];
        $data['css_url'] = $this->pg_theme->theme_default_url . $theme . '/sets/' . $scheme . '/css/mobile-' . $lang['rtl'] . '.css';

        $data['rtl'] = $lang['rtl'];

        $this->view->assign('data', $data);
        $this->view->render();
    }

    /**
     * Authenticate by social networks
     *
     * @return void
     */
    public function social_login()
    {
        $data = array("applications" => array(), "unapplications" => array());

        if ($this->pg_module->get_module_config('mobile', 'use_social_auth') && $this->pg_module->is_module_installed('users_connections')) {
            $this->load->model('Users_connections_model');
            $this->load->model('social_networking/models/Social_networking_services_model');

            $user_id = $this->session->userdata('user_id');

            $params = array('where' => array('oauth_status' => 1));
            $services = $this->Social_networking_services_model->get_services_list(null, $params);
            foreach ($services as $id => $val) {
                $connection = $this->Users_connections_model->get_connection_by_user_id($val['id'], $user_id);
                if ($connection && isset($connection['id'])) {
                    $data["unapplications"][$id] = $val;
                } else {
                    $data["applications"][$id] = $val;
                }
                $data["applications"][$id]['link'] = site_url() . 'mobile/oauth_login/' . $id;
            }
        }

        $this->view->assign('data', $data);
        $this->view->render();
    }

    /**
     * Get share link of social networks
     *
     * @return void
     */
    public function social_share_listing()
    {
        $data = array('services' => array());
        $this->load->model('social_networking/models/Social_networking_pages_model');
        $page_data = $this->Social_networking_pages_model->get_page_data('mobile', 'social_share_listing');
        if (!empty($page_data['data'])) {
            $params = array('where' => array('status' => 1));
            $order = array('id' => 'ASC');
            $this->load->model('social_networking/models/Social_networking_services_model');
            $services = $this->Social_networking_services_model->get_services_list($order, $params);
            if (!$services) {
                $services = array();
            }
            foreach ($services as $id => $value) {
                if (isset($page_data['data']['share'][$value['gid']])) {
                    $data['services'][] = array(
                        'gid' => $value['gid'],
                        'app_key' => $value['app_key'],
                        'name' => $value['name'],
                        'action' => site_url() . 'mobile/social_share_listing',
                    );
                }
            }
        }
        $this->view->assign('data', $data);
        $this->view->render();
    }

    /**
     * Check authorization by user connection
     *
     * @return void
     */
    public function oauth_check()
    {
        $data = array();

        if ($this->pg_module->is_module_installed('users_connections')) {
            $service_id = $this->input->post('service_id', true);
            $service_user_id = $this->input->post('user_id', true);

            $this->load->model('Users_connections_model');
            $connection = $this->Users_connections_model->get_connection_by_data($service_id, $service_user_id);
            if ($connection && isset($connection['id'])) {
                $data['login'] = 1;
            } else {
                $data['login'] = 0;
            }
        } else {
            $data['login'] = 0;
        }

        $this->view->assign('data', $data);
        $this->view->render();
    }

    /**
     * Get authorization link for social service
     *
     * @param string $service_id service identifier
     *
     * @return void
     */
    public function oauth_link($service_id)
    {
        $data = array();

        if ($this->pg_module->is_module_installed('users_connections')) {
            $this->load->model('Users_connections_model');
            $this->load->model('social_networking/models/Social_networking_services_model');

            $user_id = $this->session->userdata('user_id');

            $params = array('where' => array('id' => $service_id, 'oauth_status' => 1));
            $services = $this->Social_networking_services_model->get_services_list(null, $params);
            if (!empty($services)) {
                $data['link'] = site_url() . 'mobile/oauth_register/' . $service_id;
            } else {
                $data['link'] = '';
            }
        } else {
            $data['link'] = '';
        }

        $this->view->assign('data', $data);
        $this->view->render();
    }

    /**
     * Check authorization by phone
     *
     * @return void
     */
    public function oauth_phone_check()
    {
        $data = array();

        if ($this->pg_module->is_module_installed('users_connections')) {
            $service_id = $this->input->post('service_id', true);
            $service_user_id = $this->input->post('user_id', true);

            $this->load->model('Users_connections_model');
            $connection = $this->Users_connections_model->get_connection_by_phone($service_id, $service_user_id);
            if ($connection && isset($connection['id'])) {
                $data['login'] = 1;
            } else {
                $data['login'] = 0;
            }
        }

        $this->view->assign('data', $data);
        $this->view->render();
    }

    /**
     * Make registeration by phone
     *
     * @return void
     */
    public function oauth_phone_register()
    {
        $data = array('login' => 0);

        if ($this->pg_module->is_module_installed('users_connections')) {
            $service_id = $this->input->post('service_id', true);
            $service_user_id = $this->input->post('user_id', true);
            $user_type = $this->input->post('user_type', true);
            $service_user_fname = $this->input->post('user_fname', true);
            $service_user_sname = $this->input->post('user_sname', true);
            $service_user_email = $this->input->post('user_email', true);

            if (!$service_user_fname) {
                $service_user_fname = 'first_name';
            }
            if (!$service_user_sname) {
                $service_user_sname = 'last_name';
            }
            if (!$service_user_email) {
                $service_user_email = 'unknown' . rand(100000, 999999) . '@mail.com';
            }

            $this->load->model('Users_connections_model');
            $connection = $this->Users_connections_model->get_connection_by_data($service_id, $service_user_id);
            if ($connection && isset($connection['id'])) {
                $this->Users_connections_model->delete_connection($connection['id']);
                $user_id = $connection['user_id'];

                $this->load->model("users/models/Auth_model");
                $auth_data = $this->Auth_model->login($user_id);
                if (empty($auth_data["errors"])) {
                    $connection = array(
                        'service_id' => $service_id,
                        'user_id' => $user_id,
                        'data' => $service_user_id,
                    );
                    $this->Users_connections_model->save_connection(null, $connection);
                }
            }

            $params = array();
            $params["where"]["email"] = $service_user_email;
            $this->load->model("Users_model");
            $count = $this->Users_model->get_users_count($params);
            if ($count > 0) {
                $service_user_email = rand(100, 300) . $service_user_email;
            }

            $lang_id = $this->pg_language->current_lang_id;
            $this->load->model('users/models/Users_groups_model');
            $group_id = $this->Users_groups_model->get_default_group_id();

            $user_data = array(
                'fname' => $service_user_fname,
                'sname' => $service_user_sname,
                'email' => $service_user_email,
                'confirm' => 1,
                'user_type' => $user_type,
                'status' => '1',
                'lang_id' => $lang_id,
                'group_id' => $group_id,
            );

            if ($service_id && $service_user_id) {
                // Грузим модели
                $this->load->model('social_networking/models/Social_networking_services_model');
                $this->load->model('users_connections/models/Users_connections_model');
                // Данные
                $service = $this->Social_networking_services_model->get_service_by_id($service_id);
                if ($service) {
                    $user_id = $this->Users_model->save_user(null, $user_data);

                    $this->load->model("users/models/Auth_model");
                    $auth_data = $this->Auth_model->login($user_id);

                    if (!empty($auth_data["errors"])) {
                        $service_user_id = 0;
                    } else {
                        $connection = array(
                            'service_id' => $service_id,
                            'user_id' => $user_id,
                            'data' => $service_user_id,
                            'date_end' => date("Y-m-d H:i:s", time() + $result['result']['expires_in']),
                        );
                        $this->Users_connections_model->save_connection(null, $connection);

                        $data['login'] = 1;
                    }
                }
            }
        }

        $this->view->assign('data', $data);
        $this->view->render();
    }

    /**
     * Get applications data
     *
     * @return void
     */
    public function apps()
    {
        $lang_id = $this->pg_language->current_lang_id;
        $data['download_apps_used'] = $this->pg_module->get_module_config('mobile', 'download_apps_used');
        $data['download_apps_android'] = $this->pg_module->get_module_config('mobile', 'download_apps_android');
        $data['download_apps_ios'] = $this->pg_module->get_module_config('mobile', 'download_apps_ios');
        $data['download_apps_background'] = $this->pg_module->get_module_config('mobile', 'download_apps_background');
        $data['download_apps_title'] = $this->pg_language->pages->get_string('mobile', 'download_apps_title');
        $data['download_apps_text'] = $this->pg_language->pages->get_string('mobile', 'download_apps_text');

        $this->load->model('Mobile_model');
        $data = $this->Mobile_model->format_settings($data);

        $this->view->assign('data', $data);
    }
    
    public function oauthLogin()
    {
        $service_id = $this->input->post('service_id', true);
        $service_user_id = $this->input->post('service_user_id', true);
        $data = array();
        $errors = array();
        if (empty($service_id)) {
            $errors[] = 'empty_service_gid';
        }
        if (empty($service_user_id)) {
            $errors[] = 'empty_service_user_id';
        }
        if (empty($errors)) {
            $this->load->model('social_networking/models/Social_networking_services_model');
            $service = $this->Social_networking_services_model->get_service_by_id($service_id);
        }
        if (empty($service)) {
            $errors[] = 'wrong_service_gid';
        }
        if (empty($errors)) {
            $this->load->model('users_connections/models/Users_connections_model');
            $connection = $this->Users_connections_model->get_connection_by_data(
                $service['id'],
                $service_user_id
            );
            if (empty($connection)) {
                $errors[] = 'connection_not_found';
            } else {
                $this->load->model('users/models/Auth_model');
                $auth_data = $this->Auth_model->login($connection['user_id']);
                $data['auth_data'] = $auth_data;
                if (!empty($auth_data['errors'])) {
                    $errors = $auth_data['errors'];
                } elseif ($auth_data['invalid_data']) {
                    $errors[] = 'invalid_data';
                } elseif ($auth_data['blocked']) {
                    $errors[] = 'blocked';
                } elseif (!empty($auth_data["login"])) {
                    $data['token'] = $this->session->sess_create_token();
                }
            }
        }

        $this->view->assign('data', $data);
        $this->view->assign('errors', $errors);
    }

}
