<?php

namespace Pg\Modules\News\Models;

/**
 * News module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */

if (!defined("BASEPATH")) {
    exit("No direct script access allowed");
}

if (!defined('NEWS_EVENTS_TABLE')) {
    define("NEWS_EVENTS_TABLE", DB_PREFIX . "news_events");
}

/**
 * News events model
 *
 * @package 	PG_RealEstate
 * @subpackage 	News
 *
 * @category	models
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class News_events_model extends \Model
{
    /**
     * CodeIgniter object
     *
     * @var object
     */
    protected $ci;

    /**
     * Properties of update module in data source
     *
     * @var array
     */
    protected $fields = array(
        'id',
        'module',
        'model',
        'save_callback',
        'delete_callback',
    );

    /**
     * Updates cache
     *
     * @var array
     */
    protected $updates = array();

    /**
     * Class constructor
     *
     * @return News_events_model
     */
    public function __construct()
    {
        parent::__construct();
        $this->ci = &get_instance();

        $this->updates = $this->_get_updates_list();
    }

    /**
     * Return updates modules as array
     *
     * @return array
     */
    protected function _get_updates_list()
    {
        $this->ci->db->select(implode(', ', $this->fields));
        $this->ci->db->from(NEWS_EVENTS_TABLE);
        $results = $this->ci->db->get()->result_array();
        if (!empty($results) && is_array($results)) {
            return $results;
        }

        return array();
    }

    /**
     * Add module
     *
     * @param array $module_data module data
     *
     * @return void
     */
    public function add_module($module_data)
    {
        $this->ci->db->insert(NEWS_EVENTS_TABLE, $module_data);
    }

    /**
     * Remove module
     *
     * @param string $module_gid module guid
     *
     * @return void
     */
    public function delete_module($module_gid)
    {
        $this->ci->db->where("module", $module_gid);
        $this->ci->db->delete(NEWS_EVENTS_TABLE);
    }

    /**
     * Call functions on save news
     *
     * @param array $news_old_data news old data
     * @param array $news_new_data news new data
     *
     * @return void
     */
    public function onsave_news($news_old_data, $news_new_data)
    {
        foreach ($this->updates as $update) {
            if (strtolower($update['module']) . '_model' != strtolower($update['model'])) {
                $model_path = $update['module'] . '/models/' . $update['model'];
            } else {
                $model_path = $update['model'];
            }
            $this->ci->load->model($model_path);
            $this->ci->{$update['model']}->{$update['save_callback']}($news_old_data, $news_new_data);
        }
    }

    /**
     * Call functions on remove news
     *
     * @param array $news_data news data
     *
     * @return void
     */
    public function ondelete_news($news_data)
    {
        foreach ($this->updates as $update) {
            if (strtolower($update['module']) . '_model' != strtolower($update['model'])) {
                $model_path = $update['module'] . '/models/' . $update['model'];
            } else {
                $model_path = $update['model'];
            }
            $this->ci->load->model($model_path);
            $this->ci->{$update['model']}->{$update['remove_callback']}($news_data);
        }
    }
}
