<?php

namespace Pg\Modules\Polls\Controllers;

/**
 * Polls module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */

/**
 * Polls user side controller
 *
 * @package 	PG_RealEstate
 * @subpackage 	Polls
 *
 * @category	controllers
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Polls extends \Controller
{

    /**
     * Class constructor
     *
     * @return Polls
     */
    public function __construct()
    {
        parent::__construct();
        $this->load->model('Polls_model');
    }

    /**
     * Load poll data by ajax
     *
     * @param integer $poll_id        poll identifier
     * @param integer $one_poll_place once output to user
     *
     * @return void
     */
    public function ajax_poll($poll_id = 0, $one_poll_place = null)
    {
        $this->load->helper('polls');
        $this->view->output(show_poll_place_block(array('id_poll' => $poll_id, 'one_poll_place' => $one_poll_place)));
        $this->view->render();
    }

    /**
     * Save answer to poll by ajax
     *
     * @return void
     */
    public function ajax_save_result()
    {
        $language = $this->pg_language->current_lang_id;

        $poll = $this->Polls_model->get_poll_by_id($this->input->post('poll_id', true));
        if (!$poll) {
            return false;
        }
        $this->load->helper('polls');
        $this->load->helper('cookie');
        $this->load->library('user_agent');

        // Array
        $poll_answer_id = $this->input->post('answer', true);
        $poll_comment = $this->input->post('poll_comment', true);
        $user_id = $this->session->userdata('user_id');

        $cookie = array(
            'name' => 'polls[' . $poll['id'] . ']',
            'value' => '0',
            'expire' => time() + '86500',
            'domain' => COOKIE_SITE_SERVER,
            'path' => '/' . SITE_SUBFOLDER,
        );
        set_cookie($cookie);

        // Save
        $respond['user_id'] = floor($user_id);
        $respond['poll_id'] = $poll['id'];
        $respond['agent'] = $this->agent->agent_string();
        $respond['ip'] = $this->input->ip_address();
        $respond['date_add'] = date("Y-m-d H:i:s");

        if (is_array($poll_answer_id)) {
            foreach ($poll_answer_id as $answer) {
                $respond['answer_' . $answer] = isset($respond['answer_' . $answer]) ? $respond['answer_' . $answer] + 1 : 1;
            }
        } else {
            $respond['answer_' . $poll_answer_id] = isset($respond['answer_' . $poll_answer_id]) ? $respond['answer_' . $poll_answer_id] + 1 : 1;
        }

        $respond['comment'] = (string) $poll_comment;
        $this->Polls_model->save_respond($respond);

        $this->view->assign('cur_lang', $language);
        $this->view->assign('language', $poll['language']);

        $max_answers = $this->pg_module->get_module_config('polls', 'max_answers');
        $max_answers = $max_answers ? $max_answers : 10;
        $max_results = 0;

        for ($i = 1; $i <= $max_answers; ++$i) {
            if (isset($poll['results'][$i])) {
                $max_results = $max_results + floor($poll['results'][$i]);
            }
        }

        switch ($poll['sorter']) {
            case 2:
                arsort($poll['results']);
                break;
            case 1:
                asort($poll['results']);
                break;
        }

        $this->view->assign('poll_data', $poll);
        $this->view->assign('user_id', $user_id);
        $this->view->assign('max_results', $max_results);
        $this->view->assign('max_answers', $max_answers);

        $this->view->output(show_poll_place_block(array('id_poll' => $poll['id'],
            'one_poll_place' => true, 'show_results' => true)));

        $this->view->render();
    }

    /**
     * Show active polls
     *
     * @return void
     */
    public function index()
    {
        $language = $this->pg_language->current_lang_id;

        $params = array();
        $params['where']['status'] = 1;
        $params['where_in']['language'] = array('0', $language);
        $params['where']['show_results'] = 1;
        $params['where_in']['poll_type'] = array(0);

        $this->load->model('Menu_model');

        if ($this->session->userdata("auth_type") == "user") {
            $this->load->model('Users_model');
            $user_types = $this->Users_model->getUserTypes();
            $user_types = array_flip($user_types);

            $user_type = $this->session->userdata("user_type");
            $params['where_in']['poll_type'][] = 1;
            $params['where_in']['poll_type'][] = $user_types[$user_type] + 3;
        } else {
            $user_type = 'guest';
            $params['where_in']['poll_type'][] = 2;
        }

        $this->Menu_model->breadcrumbs_set_parent('guest-main-polls-item');

        $polls = $this->Polls_model->get_polls_list(null, null, array('date_add' => 'ASC'), $params);
        $this->view->assign('language', $language);

        $this->load->model('Menu_model');
        $this->Menu_model->breadcrumbs_set_active(l('header_polls_results', 'polls'));
        $this->pg_theme->add_js('polls.js', 'polls');
        $this->view->assign('polls', $polls);
        $this->view->render('list_polls');
    }

}
