<?php

namespace Pg\Modules\Properties\Controllers;

/**
 * Properties module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
use Pg\Libraries\View;

/**
 * Properties admin side controller
 *
 * @package 	PG_RealEstate
 * @subpackage 	Properties
 *
 * @category	controllers
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Admin_Properties extends \Controller
{

    /**
     * Constructor
     *
     * @return Admin_Properties
     */
    public function __construct()
    {
        parent::__construct();
        $this->load->model('Menu_model');
        $this->load->model('Properties_model');
        $this->Menu_model->set_menu_active_item('admin_menu', 'content_items');
    }

    /**
     * Manage properties
     *
     * @return void
     */
    public function index()
    {
        $lang_id = $this->pg_language->current_lang_id;

        $this->pg_language->ds->return_module($this->Properties_model->module_gid, $lang_id);
        $ds = $this->pg_language->ds->lang[$lang_id][$this->Properties_model->module_gid];

        $category_gids = $this->Properties_model->categories;
        $property_gids = $this->Properties_model->properties;

        foreach ($category_gids as $gid) {
            $categories[$gid] = $ds[$gid];
        }
        $this->view->assign('categories', $categories);

        foreach ($property_gids as $gid) {
            $properties[$gid] = $ds[$gid];
            $properties[$gid]['status'] = $this->pg_module->get_module_config('listings', $gid . '_enabled');
        }
        $this->view->assign('properties', $properties);

        $this->view->assign('property_prefix', $this->Properties_model->property_prefix);

        $this->view->setHeader(l('admin_header_properties', 'properties'));
        $this->view->render('menu');
    }

    /**
     * Manage property items
     *
     * @param string  $ds_gid  data source guid
     * @param integer $lang_id language identifier
     *
     * @return void
     */
    public function property($ds_gid, $lang_id = null)
    {
        if (!$ds_gid) {
            redirect(site_url() . 'admin/properties/index');
        } elseif (!$lang_id || !array_key_exists($lang_id, $this->pg_language->languages)) {
            $lang_id = $this->pg_language->current_lang_id;
        }
        $reference = $this->pg_language->ds->get_reference($this->Properties_model->module_gid, $ds_gid, $lang_id);
        $header = l('admin_header_properties', 'properties') . ' : ' . $reference['header'];

        $this->view->setBackLink(site_url() . 'admin/properties/index');
        $this->view->setHeader($header);

        $this->view->assign('current_gid', $ds_gid);
        $this->view->assign('langs', $this->pg_language->languages);
        $this->view->assign('current_lang_id', $lang_id);
        $this->view->assign('reference', $reference);
        $this->view->assign('module_gid', $this->Properties_model->module_gid);
        $this->view->assign('ds_gid', $ds_gid);
        $this->view->render('list');
    }

    /**
     * Manage property items
     *
     * @param integer $lang_id    language identifier
     * @param string  $ds_gid     data source guid
     * @param string  $option_gid option guid
     *
     * @return void
     */
    public function property_items($lang_id, $ds_gid, $option_gid = null)
    {
        if (!$lang_id || !array_key_exists($lang_id, $this->pg_language->languages)) {
            $lang_id = $this->pg_language->current_lang_id;
        }

        $reference = $this->pg_language->ds->get_reference($this->Properties_model->module_gid, $ds_gid, $lang_id);
        if ($option_gid) {
            $add_flag = false;
            foreach ($this->pg_language->languages as $lid => $lang) {
                $r = $this->pg_language->ds->get_reference($this->Properties_model->module_gid, $ds_gid, $lid);
                $lang_data[$lid] = $r["option"][$option_gid];
            }
        } else {
            $option_gid = "";
            $lang_data = array();
            $add_flag = true;
        }

        if ($this->input->post('btn_save')) {
            $lang_data = $this->input->post('lang_data', true);

            if (empty($option_gid)) {
                if (!empty($reference["option"])) {
                    $array_keys = array_keys($reference["option"]);
                } else {
                    $array_keys = array(0);
                }
                $option_gid = max($array_keys) + 1;
            }

            $default_lang_id = $lang_id ? $lang_id : $this->pg_language->current_lang_id;
            if (!isset($lang_data[$default_lang_id]) || empty($lang_data[$default_lang_id])) {
                $this->system_messages->addMessage(View::MSG_ERROR, l('error_empty_property_item', 'properties'));
            } else {
                $reference = $this->pg_language->ds->get_reference($this->Properties_model->module_gid, $ds_gid,
                                                                   $default_lang_id);
                $reference["option"][$option_gid] = $lang_data[$default_lang_id];
                $this->pg_language->ds->set_module_reference($this->Properties_model->module_gid, $ds_gid, $reference,
                                                             $default_lang_id);
                foreach ($this->pg_language->languages as $lid => $ldata) {
                    if ($lid == $default_lang_id) {
                        continue;
                    }
                    $reference = $this->pg_language->ds->get_reference($this->Properties_model->module_gid, $ds_gid,
                                                                       $lid);
                    if (!isset($lang_data[$lid]) || empty($lang_data[$lid])) {
                        $reference["option"][$option_gid] = $lang_data[$default_lang_id];
                    } else {
                        $reference["option"][$option_gid] = $lang_data[$lid];
                    }
                    $this->pg_language->ds->set_module_reference($this->Properties_model->module_gid, $ds_gid,
                                                                 $reference, $lid);
                }

                if ($add_flag) {
                    $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_added_property_item', 'properties'));
                } else {
                    $this->system_messages->addMessage(View::MSG_SUCCESS,
                                                       l('success_updated_property_item', 'properties'));
                }

                $url = site_url() . "admin/properties/property/" . $ds_gid . "/" . $lang_id;
                redirect($url);
            }
        }

        $this->view->assign('lang_data', $lang_data);
        $this->view->assign('langs', $this->pg_language->languages);
        $this->view->assign('current_lang_id', $lang_id);
        $this->view->assign('current_module_gid', $this->Properties_model->module_gid);
        $this->view->assign('current_gid', $ds_gid);
        $this->view->assign('option_gid', $option_gid);
        $this->view->assign('add_flag', $add_flag);

        $this->view->setHeader(l('admin_header_property_item_edit', 'properties'));
        $this->view->render('edit_ds_item');
    }

    /**
     * Remove data source item
     *
     * @param string $gid        data source guid
     * @param string $option_gid option guid
     *
     * @return void
     */
    public function ajax_ds_item_delete($gid, $option_gid)
    {
        if ($gid && $option_gid) {
            foreach ($this->pg_language->languages as $lid => $lang) {
                $reference = $this->pg_language->ds->get_reference($this->Properties_model->module_gid, $gid, $lid);
                if (isset($reference["option"][$option_gid])) {
                    unset($reference["option"][$option_gid]);
                    $this->pg_language->ds->set_module_reference($this->Properties_model->module_gid, $gid, $reference,
                                                                 $lid);
                }
            }
        }

        return;
    }

    /**
     * Resort data source items
     *
     * @param string $gid data source guid
     *
     * @return void
     */
    public function ajax_ds_item_save_sorter($gid)
    {
        $sorter = $this->input->post("sorter");
        foreach ($sorter as $parent_str => $items_data) {
            foreach ($items_data as $item_str => $sort_index) {
                $option_gid = str_replace("item_", "", $item_str);
                $sorter_data[$sort_index] = $option_gid;
            }
        }

        if (empty($sorter_data)) {
            return;
        }
        ksort($sorter_data);
        $this->pg_language->ds->set_reference_sorter($this->Properties_model->module_gid, $gid, $sorter_data);

        return;
    }

    /**
     * Activate/deactivate property
     *
     * Available status values: 1 - activate, 0 - deactivate
     *
     * @param string  $property_gid property guid
     * @param integer $status       property status
     *
     * @return void
     */
    public function activate($property_gid, $status)
    {
        if (!$status) {
            $can_deactivate = false;
            $property_gids = $this->Properties_model->properties;
            foreach ($property_gids as $gid) {
                if ($gid == $property_gid) {
                    continue;
                }
                $property_enabled = $this->pg_module->get_module_config('listings', $gid . '_enabled');
                if ($property_enabled) {
                    $can_deactivate = true;
                    break;
                }
            }
            if (!$can_deactivate) {
                $this->system_messages->addMessage(View::MSG_ERROR, l('error_category_deactivate', 'properties'));
                redirect(site_url() . 'admin/properties/index');
            }
        }
        $this->pg_module->set_module_config('listings', $property_gid . '_enabled', $status ? 1 : 0);
        redirect(site_url() . 'admin/properties/index');
    }

    /**
     * Edit property data
     *
     * @param string $property_gid property guid
     *
     * @return void
     */
    public function edit($property_gid)
    {
        if (!in_array($property_gid, $this->Properties_model->properties)) {
            show_404();
        }

        $property_gid = str_replace('property_type_', '', $property_gid);

        foreach ($this->pg_language->languages as $lid => $lang) {
            $categories = $this->pg_language->ds->get_reference($this->Properties_model->module_gid, 'property_types',
                                                                $lid);
            $lang_data[$lid] = $categories['option'][$property_gid];
        }

        if ($this->input->post('btn_save')) {
            $errors = array();

            $lang_data = $this->input->post('lang_data', true);

            foreach ($this->pg_language->languages as $lid => $ldata) {
                $lang_data[$lid] = trim(strip_tags($lang_data[$lid]));
                if (empty($lang_data[$lid])) {
                    $errors[] = l('error_empty_category_name', 'properties');
                    break;
                }
            }

            if (!empty($errors)) {
                $this->system_messages->addMessage(View::MSG_ERROR, $errors);
            } else {
                foreach ($this->pg_language->languages as $lid => $ldata) {
                    $reference = $this->pg_language->ds->get_reference($this->Properties_model->module_gid,
                                                                       'property_types', $lid);
                    $reference["option"][$property_gid] = $lang_data[$lid];
                    $this->pg_language->ds->set_module_reference($this->Properties_model->module_gid, 'property_types',
                                                                 $reference, $lid);

                    $reference = $this->pg_language->ds->get_reference($this->Properties_model->module_gid,
                                                                       'property_type_' . $property_gid, $lid);
                    $reference["header"] = $lang_data[$lid];
                    $this->pg_language->ds->set_module_reference($this->Properties_model->module_gid,
                                                                 'property_type_' . $property_gid, $reference, $lid);
                }
                $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_category_updated', 'properties'));
                redirect(site_url() . 'admin/properties/index');
            }
        }

        $this->view->assign('lang_data', $lang_data);
        $this->view->assign('langs', $this->pg_language->languages);

        $this->view->render('edit');
    }

}
