<?php

namespace Pg\Modules\Services\Controllers;

/**
 * Services module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
use Pg\Libraries\View;

/**
 * Services admin side controller
 *
 * @package 	PG_RealEstate
 * @subpackage 	Services
 *
 * @category	controllers
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Admin_Services extends \Controller
{

    /**
     * Constructor
     *
     * @return Admin_Services
     */
    public function __construct()
    {
        parent::__construct();

        $this->load->model('Menu_model');
        $this->Menu_model->set_menu_active_item('admin_menu', 'payments_menu_item');
        $this->load->model("Services_model");
    }

    /**
     * Manage services
     *
     * @return void
     */
    public function index()
    {
        $order_by["gid"] = "ASC";
        $query = array('where' => array('type' => 'tariff',),);
        $services = $this->Services_model->get_service_list($query, null, $order_by);
        foreach ($services as $key => $service) {
            if ($service['type'] != 'tariff') {
                unset($services[$key]);
            }
        }

        $this->view->assign('services', $services);
        $services_count = count($services);

        $this->load->helper('navigation');

        $page_data = get_admin_pages_data(site_url() . "admin/services/index/", $services_count, $services_count, 1,
                                          'briefPage');
        $page_data["date_format"] = $this->pg_date->get_format('date_time_literal', 'st');

        $addable_templates = $this->Services_model->get_template_count(array("where" => array("moveable" => "1")));
        if ($addable_templates > 0) {
            $page_data["add_service_link"] = 1;
        }

        $this->view->assign('page_data', $page_data);
        $this->Menu_model->set_menu_active_item('admin_payments_menu', 'services_menu_item');
        $this->view->setHeader(l('admin_header_services_list', 'services'));
        $this->view->render('list');
    }

    /**
     * Edit service data
     *
     * @param integer $service_id service identifier
     *
     * @return void
     */
    public function edit($service_id = null)
    {
        $errors = false;

        if (!empty($service_id)) {
            $data = $this->Services_model->get_service_by_id($service_id);
            foreach ($this->pg_language->languages as $lang_id => $lang_data) {
                $validate_lang[$lang_id] = l('service_name_' . $data["id"], 'services', $lang_id);
            }
            $template = $this->Services_model->get_template_by_gid($data["template_gid"]);
            $this->view->assign('template', $template);
        } else {
            $data = array();
            $params["where"]["moveable"] = "1";
            $templates = $this->Services_model->get_template_list($params);
            $template = $this->Services_model->format_template(current($templates));
            $this->view->assign('templates', $templates);
        }

        if ($this->input->post('btn_save')) {
            $post_data = array(
                "gid" => $this->input->post("gid", true),
                "pay_type" => $this->input->post("pay_type", true),
                "template_gid" => $this->input->post("template_gid", true),
                "user_type_disabled" => $this->input->post("user_type_disabled", true),
                "status" => $this->input->post("status", true),
                "price" => $this->input->post("price", true),
                "data_admin" => $this->input->post("data_admin", true),
            );
            $langs = $this->input->post("langs", true);
            $validate_data = $this->Services_model->validate_service($service_id, $post_data, $langs);
            if (!empty($validate_data["errors"])) {
                $errors = $validate_data["errors"];
                $validate_lang = $langs;
                $data = array_merge($data, $validate_data["data"]);
            } else {
                $this->Services_model->save_service($service_id, $validate_data["data"], $validate_data['langs_data']);

                $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_update_service_data', 'services'));
                redirect(site_url() . "admin/services");
            }
        }
        if (!empty($data)) {
            $t = $this->Services_model->format_service(array(0 => $data));
            $data = $t[0];
        }

        // languages
        $this->view->assign('languages', $this->pg_language->languages);
        $this->view->assign('languages_count', count($this->pg_language->languages));
        $this->view->assign('cur_lang', $this->pg_language->current_lang_id);
        $this->view->assign('pay_type_lang', ld('pay_type', 'services'));

        $this->view->assign(
                'template_block',
                $this->_get_template_admin_param_block(
                        $template, 
                        isset($data["price"]) ? $data["price"] : 0,
                        isset($data["data_admin"]) ? $data["data_admin"] : array()
                )
        );

        if (!empty($validate_lang)) {
            $this->view->assign('validate_lang', $validate_lang);
        }

        $this->view->assign('data', $data);

        if (!empty($errors)) {
            $this->system_messages->addMessage(View::MSG_ERROR, $errors);
        }

        $this->load->model('Users_model');
        $user_types = $this->Users_model->getUserTypes();
        if (!empty($user_types)) {
            $this->view->assign('user_types', $user_types);
        }

        $this->Menu_model->set_menu_active_item('admin_payments_menu', 'services_menu_item');
        $this->view->setHeader(l('admin_header_services_list', 'services'));
        $this->view->render('edit');
    }

    /**
     * Get service parameters block by ajax
     *
     * @param string $template_gid template guid
     *
     * @return void
     */
    public function ajax_get_template_admin_param_block($template_gid)
    {
        $template = $this->Services_model->get_template_by_gid($template_gid);

        $this->view->output($this->_get_template_admin_param_block($template));

        $this->view->render();
    }

    /**
     * Get service parameters block (internal)
     *
     * @param array $template template object
     * @param float $price    template price
     * @param array $values   parameters data
     *
     * @return string
     */
    protected function _get_template_admin_param_block($template, $price = 0, $values = "")
    {
        if (!empty($values)) {
            foreach ($template["data_admin_array"] as $gid => $data) {
                if (!empty($values[$gid])) {
                    $template["data_admin_array"][$gid]["value"] = $values[$gid];
                }
            }
        }
        $this->view->assign('price', $price);
        $this->view->assign('template_data', $template);

        return $this->view->fetch('block_service_param');
    }

    /**
     * Remove service
     *
     * @param integer $service_id service identifier
     *
     * @return void
     */
    public function delete($service_id)
    {
        $this->Services_model->delete_service($service_id);
        $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_delete_service', 'services'));
        redirect(site_url() . "admin/services");
    }

    /**
     * Activate/deactivate service
     *
     * Available service values: 1 - activate, 0 - deactivate
     *
     * @param integer $service_id service identifier
     * @param integer $status     active status
     *
     * @return void
     */
    public function activate($service_id, $status = 1)
    {
        $this->Services_model->save_service($service_id, array("status" => $status));
        $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_update_service_data', 'services'));
        redirect(site_url() . "admin/services");
    }

    /**
     * Manage service templates
     *
     * @return void
     */
    public function templates()
    {
        $order_by["gid"] = "ASC";
        $templates = $this->Services_model->get_template_list(array(), null, $order_by);
        $this->view->assign('templates', $templates);
        $templates_count = count($templates);

        $this->load->helper('navigation');

        $page_data = get_admin_pages_data(site_url() . "admin/services/templates/", $templates_count, $templates_count,
                                          1, 'briefPage');
        $page_data["date_format"] = $this->pg_date->get_format('date_time_literal', 'st');
        $this->view->assign('page_data', $page_data);

        $this->Menu_model->set_menu_active_item('admin_payments_menu', 'services_menu_item');
        $this->view->setHeader(l('admin_header_templates_list', 'services'));
        $this->view->render('list_templates');
    }

    /**
     * Edit template data
     *
     * @param string $template_id template identifier
     *
     * @return void
     */
    public function template_edit($template_id = null)
    {
        if (!empty($template_id)) {
            $data = $this->Services_model->get_template_by_id($template_id);
            foreach ($this->pg_language->languages as $lang_id => $lang_data) {
                $validate_lang[$lang_id] = l('template_name_' . $data["id"], 'services', $lang_id);
            }
        } else {
            $data = array();
        }

        if ($this->input->post('btn_save')) {
            $post_data = array(
                "gid" => $this->input->post("gid", true),
                "callback_module" => $this->input->post("callback_module", true),
                "callback_model" => $this->input->post("callback_model", true),
                "callback_buy_method" => $this->input->post("callback_buy_method", true),
                "callback_activity_method" => $this->input->post("callback_activity_method", true),
                "callback_validate_method" => $this->input->post("callback_validate_method", true),
                "price_type" => $this->input->post("price_type", true),
                "moveable" => $this->input->post("moveable", true),
                "data_admin" => unserialize($this->input->post("data_admin", true)),
                "data_user" => unserialize($this->input->post("data_user", true)),
            );
            $langs = $this->input->post("langs", true);

            $validate_data = $this->Services_model->validate_template($template_id, $post_data, $langs);
            if (!empty($validate_data["errors"])) {
                $this->system_messages->addMessage(View::MSG_ERROR, $validate_data["errors"]);
                $validate_lang[] = $langs;
            } else {
                $this->Services_model->save_template($template_id, $validate_data["data"], $validate_data['langs_data']);

                $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_update_template_data', 'services'));
                redirect(site_url() . "admin/services/templates");
            }
        }

        ///// languages
        $this->view->assign('languages', $this->pg_language->languages);
        $this->view->assign('languages_count', count($this->pg_language->languages));
        $this->view->assign('cur_lang', $this->pg_language->current_lang_id);
        $this->view->assign('price_type_lang', ld('price_type', 'services'));

        if (!empty($validate_lang)) {
            $this->view->assign('validate_lang', $validate_lang);
        }

        $this->view->assign('data', $data);
        $this->Menu_model->set_menu_active_item('admin_payments_menu', 'services_menu_item');
        $this->view->setHeader(l('admin_header_templates_list', 'services'));
        $this->view->render('edit_templates');
    }

    /**
     * Remove template
     *
     * @param integer $template_id template idnetifier
     *
     * @return void
     */
    public function template_delete($template_id)
    {
        $this->Services_model->delete_template($template_id);
        $this->system_messages->addMessage(View::MSG_SUCCESS, l('success_delete_template', 'services'));
        redirect(site_url() . "admin/services/templates");
    }

}
