<?php

namespace Pg\Modules\Services\Controllers;

/**
 * Services module
 *
 * @package     PG_RealEstate
 *
 * @copyright   Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author      Pilot Group Ltd <http://www.pilotgroup.net/>
 */
use Pg\Libraries\View;

/**
 * Services user side controller
 *
 * @package     PG_RealEstate
 * @subpackage  Services
 *
 * @category    controllers
 *
 * @copyright   Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author      Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Services extends \Controller
{

    /**
     * Constructor
     *
     * @return Services
     */
    public function __construct()
    {
        parent::__construct();
        $this->load->model("Services_model");
    }

    /**
     * Display form to activate service
     *
     * @param string $service_gid    service GUID
     * @param array  $user_form_data user's filled data
     *
     * @return void
     */

    public function createServiceData($service_gid, $services = false)
    {
        $data = array();
        $user_form_data = array();
        $user_form_data = json_decode($this->input->post("data_user", true), true);

        if ($service_gid) {
            $data = $this->Services_model->get_service_by_gid($service_gid);
            $t = $this->Services_model->format_service(array($data));
            $data = $t[0];
            $data["template"] = $this->Services_model->format_template($data["template"]);
            $price =  $this->input->post('price', true);
            if ($price) {
                if (!$services) {
                    $data["price"] = $price;
                }
            }
            if (!empty($data["data_admin_array"])) {
                foreach ($data["template"]["data_admin_array"] as $gid => $temp) {
                    if (!empty($data["data_admin_array"][$gid])) {
                        $data["template"]["data_admin_array"][$gid]["value"] = $data["data_admin_array"][$gid];
                    }
                }
            }
            if (!empty($data["template"]["data_user_array"])) {
                foreach ($data["template"]["data_user_array"] as $gid => $temp) {
                    $value = "";
                    if ($temp["type"] == "hidden") {
                        $value = $this->input->post($gid, true);
                        if (!$value) {
                            if ($temp['gid'] == 'user') {
                                $value = $this->session->userdata('user_id');
                            } elseif (isset($temp['value'])) {
                                $value = $temp['value'];
                            }
                        }
                    }
                    if (!empty($user_form_data[$gid])) {
                        $value = $user_form_data[$gid];
                    }
                    $data["template"]["data_user_array"][$gid]["value"] = $value;
                }
            }
            if (empty($user_form_data) && isset($data['template']['data_user_array'])) {
                foreach ($data['template']['data_user_array'] as $value) {
                    if ($value['gid'] == 'user') {
                        $user_form_data[$value['gid']] = $this->session->userdata('user_id');
                    }
                    if ($value['gid'] == 'id_listing') {
                        $listings_for_buy = $this->input->post('listings', true);
                        if ($listings_for_buy) {
                            $listings_for_buy = json_decode($listings_for_buy);
                            if (count($listings_for_buy) == 1) {
                                $user_form_data[$value['gid']] = $listings_for_buy[0];
                            } elseif (count($listings_for_buy) > 1) {
                                foreach ($listings_for_buy as $key_list => $value_list) {
                                   $user_form_data[$value['gid']][$key_list] = $value_list;
                                }
                            } else {
                                return false;
                            }
                        } else {
                            $user_form_data[$value['gid']] = $this->input->post('id_listing', true);
                        }
                    } else {
                       $user_form_data[$value['gid']] = $value['value'];
                    }
                }
            } elseif (empty($user_form_data) && $data["template"]["data_admin_array"]) {
                $user_form_data['id_listing'] = $this->input->post('id_listing', true);
            }
            $data['user_form_data'] = $user_form_data;
            return $data;
        } else {
            return false;
        }
    }

    public function form($service_gid = null, $user_form_data = array())
    {
        $service_redirect = $this->session->userdata('service_redirect');

        $services_gids = array();

        $service_public_id = $this->input->post('activate_public', true);
        if ($service_public_id) {
            $this->load->model("services/models/Services_users_model");
            $listing_id = $this->input->post('id_listing', true);
            if ($listing_id) {
                $this->load->model("Listings_model");
                $listing = $this->Listings_model->get_listing_by_id($listing_id);
                $services_gids = $this->input->post('services', true);
                foreach ($services_gids as $key => $value) {
                    if ($value == 'listings_post_service') {
                        unset($services_gids[$key]);
                    }
                }
                if ($listing && $listing['status'] == 0) {
                    if (strtotime($listing['date_expire']) > time()) {
                        $this->Listings_model->activate_listing($listing['id'], 1);
                        if (empty($services_gids)) {
                            $redirect = !empty($service_redirect) ? $service_redirect : 'listings/my';
                            $this->view->setRedirect($redirect);
                        }
                    } else {
                        $service = $this->Services_users_model->get_service_by_id($service_public_id);
                        if ($service) {
                            $this->load->model("Users_services_model");
                            $user_data = array('id_listing' => $listing_id);
                            $this->Users_services_model->service_post_activate($service, $user_data);
                            if (empty($services_gids)) {
                                $redirect = !empty($service_redirect) ? $service_redirect : 'listings/my';
                                $this->view->setRedirect($redirect);
                            }
                        }
                    }
                } elseif (empty($services_gids)) {
                    $redirect = !empty($service_redirect) ? $service_redirect : 'listings/my';
                    $this->view->setRedirect($redirect);
                }
            }
        }

        if (!$this->pg_module->is_module_installed('users')) {
            show_404();
            return;
        }

        if (!$service_gid) {
            $service_gid = $this->input->post('service_gid', true);
        }

        $services_data = json_decode($this->input->post('services_data', true), true);

        if (empty($services_data)) {
            if (!$service_gid) {
                if (empty($services_gids)) {
                    $services_gids = $this->input->post('services', true);
                    if (empty($services_gids)) {
                        show_404();
                        return;
                    }
                }
            }
        } else {
            $services_data['price'] = $this->input->post('price', true);
        }

        $listings = null;
        $data = array();
        $user_form_data = array();


        $listings = json_decode($this->input->post('listings', true));

        $user_id = $this->session->userdata('user_id');

        $this->load->model('users/models/Auth_model');
        $this->Auth_model->update_user_session_data($user_id);

        $back_url = !empty($service_redirect) ? $service_redirect : 'users/account_services/';

        if (!empty($services_gids)) {
            $data = array('services' => array());
            if (count($services_gids) > 1) {
                foreach ($services_gids as $key => $value) {
                    $data['services'][$key] = $this->createServiceData($value, true);
                }
                $data['pay_type'] = 2;
                $data['user_form_data'] = null;
                $data['price'] = $this->input->post('price', true);
                foreach ($data['services'] as $value) {
                    $data['user_form_data'] = $value['user_form_data'];
                    $data["template"]["price_type"] = $value["template"]["price_type"];
                    break;
                }
                $this->view->assign('services', '1');
            } elseif (count($services_gids) == 1) {
                foreach ($services_gids as $value) {
                    $service_gid = $value;
                }
            }
        }
        if ($service_gid) {
            $data = $this->createServiceData($service_gid);
        }
        if (!empty($listings)) {
            $this->view->assign('listings', $listings);
            $this->view->assign('listings_count', count($listings));
        }
        if ($service_gid && empty($listings)) {
            $this->view->assign('service', '1');
        }
        if (empty($services_data)) {
            $user_form_data = $data['user_form_data'];
            unset($data['user_form_data']);
        } else {
            foreach ($services_data['services'] as $key => $value) {
                $user_form_data['services'][$key ] = $value['gid'];
            }
        }

        if ($this->input->post('data_user', true)) {
            $data_user = json_decode($this->input->post("data_user", true), true);
            $user_form_data = array_merge($user_form_data, $data_user);
        }

        if ($this->input->post('id_listing', true)) {
            $id_listing = $this->input->post('id_listing', true);
            $this->view->assign('id_listing', $id_listing);
            if (!empty($services_data)) {
                $user_form_data['id_listing'] = $id_listing;
            }
        }
        if ($this->input->post('btn_buy')) {
                $activate_immediately = (bool) $this->input->post('activate_immediately');
                if ($services_data) {
                    foreach ($services_data['services'] as $key => $value) {
                        $service_return[$key] = $this->Services_model->validate_service_payment($value["id"], $user_form_data, $value['price']);
                    }
                    foreach ($service_return as $value) {
                        if (!empty($value["errors"])) {
                            $service_return["errors"] = $value["errors"];
                        }
                    }
                    if (empty($service_return["errors"])) {
                        unset($service_return);
                        $service_return = array( 'errors' => array(),
                            'data' => array('price' => $services_data['price'],
                            'data_user' => $user_form_data)
                        );
                    }
                } else {
                    $service_return = $this->Services_model->validate_service_payment($data["id"], $user_form_data, $data["price"]);
                }

                if (!empty($service_return["errors"])) {
                    $this->system_messages->addMessage(View::MSG_ERROR, $service_return["errors"]);
                } else {
                    if (!empty($service_return['data']['data_user']['services'])) {
                        foreach ($services_data['services'] as $key_serv => $value_serv) {
                            $origin_return[$key_serv] = $this->Services_model->validate_service_original_model($value_serv["id"], $user_form_data, $user_id, $value_serv["price"]);
                        }
                        foreach ($origin_return as $value) {
                            if (!empty($value["errors"])) {
                                $origin_return["errors"] = $value["errors"];
                            }
                        }
                    } else {
                        $origin_return = $this->Services_model->validate_service_original_model($data["id"], $user_form_data, $user_id, $data["price"]);
                    }

                    if (!empty($origin_return["errors"])) {
                        $this->system_messages->addMessage(View::MSG_ERROR, $origin_return["errors"]);
                    } else {
                        $system_gid = $this->input->post('system_gid', true);
                        if (empty($system_gid)) {
                            $this->system_messages->addMessage(View::MSG_ERROR, l('error_select_payment_system', 'services'));
                        } else {
                            if ($system_gid == 'account') {
                                $base_currency_gid = $this->pg_module->get_module_config("start", "base_currency");
                                $this->load->model('Currencies_model');
                                $base_currency = $this->Currencies_model->get_currency_default(true);
                                $this->load->helper('start');
                                if (!empty($services_data)) {
                                    foreach ($services_data['services'] as $key => $value) {
                                        $data["price"] = get_currency_value(array('value' => $value["price"], 'cur_gid' => $base_currency['gid'], 'to_gid' => $base_currency_gid));
                                        //$user_form_data = array('id_listing' => $this->input->post('id_listing', true));
                                        $temp_return[$key] = $this->Services_model->account_payment($value["id"], $user_id, $user_form_data, $data["price"], $activate_immediately);
                                    }
                                    $return = true;
                                    foreach ($temp_return as $value) {
                                        if ($value !== true) {
                                            $return = $value;
                                        }
                                    }
                                } else {
                                    $data["price"] = get_currency_value(array('value' => $data["price"], 'cur_gid' => $base_currency['gid'], 'to_gid' => $base_currency_gid));
                                    $return = $this->Services_model->account_payment($data["id"], $user_id, $user_form_data, $data["price"], $activate_immediately);
                                }
                                if ($return !== true) {
                                    $this->system_messages->addMessage(View::MSG_ERROR, $return);
                                } else {
                                    $this->system_messages->addMessage(View::MSG_SUCCESS,
                                                                       l('success_services_apply', 'services'));
                                    $redirect = !empty($service_redirect) ? $service_redirect : 'listings/my';
                                    $this->session->set_userdata(array('service_redirect' => ''));
                                    $this->load->model('users/models/Auth_model');
                                    $this->Auth_model->update_user_session_data($user_id);
                                    $this->view->setRedirect($redirect);
                                }
                            } elseif ($system_gid != 'account') {
                                $base_currency_gid = $this->pg_module->get_module_config("start", "base_currency");
                                $this->load->model('Currencies_model');
                                $base_currency = $this->Currencies_model->get_currency_default(true);
                                $this->load->helper('start');
                                if (!empty($services_data)) {
                                    $data["price"] = 0;
                                    $id_listing = [];
                                    foreach ($services_data['services'] as $key => $value) {
                                        $data["price"] += get_currency_value(array('value' => $value["price"], 'cur_gid' => $base_currency['gid'], 'to_gid' => $base_currency_gid));
                                    }
                                    //$user_form_data = array('id_listing' => $this->input->post('id_listing', true));
                                    $this->Services_model->system_payment($system_gid, $user_id, null, $user_form_data, $data["price"]);
                                } else {
                                    $this->Services_model->system_payment($system_gid, $user_id, $data["id"], $user_form_data, $data["price"]);
                                }

                                $redirect = !empty($service_redirect) ? $service_redirect : 'listings/my';
                                $this->session->set_userdata(array('service_redirect' => ''));
                                $this->load->model('users/models/Auth_model');
                                $this->Auth_model->update_user_session_data($user_id);
                                $this->view->setRredirect($redirect);
                            }
                        }
                    }
                }
            }

            // get payments types
            $data["free_activate"] = false;

            if ($data["price"] <= 0) {
                $data["free_activate"] = true;
            }
            if ($data["pay_type"] == 1 || $data["pay_type"] == 2) {
                $this->load->helper('payments');
                $this->load->model("Users_payments_model");
                $data["user_account"] = $this->Users_payments_model->get_user_account($user_id);
                if ($data["user_account"] <= 0 && $data["price"] > 0) {
                    $data["disable_account_pay"] = true;
                } elseif ($data["template"]["price_type"] == 1 || $data["template"]["price_type"] == 3) {
                    $base_currency_gid = $this->pg_module->get_module_config('start', 'base_currency');
                    $this->load->model('Currencies_model');
                    $base_currency = $this->Currencies_model->get_currency_default(true);
                    $this->load->helper('start');
                    $price_value = get_currency_value(array('value' => $data["price"], 'cur_gid' => $base_currency['gid'], 'to_gid' => $base_currency_gid));
                    if ($price_value > $data["user_account"]) {
                        $data["disable_account_pay"] = true;
                    }
                }
            }

            if ($data["pay_type"] == 2 || $data["pay_type"] == 3) {
                $is_install_payments = $this->pg_module->is_module_active('payments');
                if ($is_install_payments) {
                    $this->load->model("payments/models/Payment_systems_model");
                    $billing_systems = $this->Payment_systems_model->get_active_system_list();
                    $this->view->assign('billing_systems', $billing_systems);
                }
            }

            $this->load->model('Currencies_model');
            $this->view->assign('base_currency', $this->Currencies_model->get_currency_default(true));

            $this->load->model('Menu_model');
            $this->Menu_model->breadcrumbs_set_parent('my_payments_item');
            $this->Menu_model->breadcrumbs_set_active(l('checkout_header', 'services'));
            $is_payments_installed = $this->pg_module->is_module_active('users_payments');
            if ($service_gid) {
                $this->view->assign('service_gid', $service_gid);
            }
            if ($services_gids) {
                $this->view->assign('services_gids', $services_gids);
                $this->view->assign('sum', $this->input->post('price', true));
            }


            $this->view->assign('user_data', $user_form_data);
            $this->view->assign('back_url', $back_url);
            $this->view->assign('is_module_active', $is_payments_installed);
            $this->view->assign('data', $data);
            $this->view->render('service_form');

    }

    public function selectListings($service_gid, $reset_storage = false, $page = 1, $item_on_page = 5)
    {

            if (!$service_gid) {
                show_404();
                return;
            }
            if ($this->input->get('page', true)) {
                $page = $this->input->get('page', true);
            }

            $user_type = $this->session->userdata('user_type');
            $this->Menu_model->breadcrumbs_set_parent($user_type . '_my_profile_services_item');
            $service = $this->Services_model->get_service_by_gid($service_gid);
            if (empty($service['status'])) {
                show_404();
                return;
            }
            $t = $this->Services_model->format_service(array($service));
            $service = $t[0];
            $user_id = $this->session->userdata('user_id');
            $this->load->model('Listings_model');
            $filters = array('id_user' => $user_id, 'status' => 1, 'sold' => 0 );
            $items_count = $this->Listings_model->get_listings_count($filters);
            $order = array('date_activity' => 'ASC');
            $this->Listings_model->applyFormatSettings(array('get_views' => true));
            $listings = $this->Listings_model->get_listings_list($filters, $page, $item_on_page, $order, true, true);
            $this->Listings_model->revertFormatSettings();
            if ( $items_count == 1) {
                $this->form($service_gid, null, $listings);
                return;
            }
            if ( $items_count == 0 ) {
                $no_listings = true;
            } else {
                $no_listings = false;
            }
            if ($items_count > $item_on_page) {
                $this->load->helper('navigation');
                $page_data = get_user_pages_data('services/selectListings/'.$service_gid.'/false', $items_count, 5, $page, 'briefPage');
                $this->view->assign('page_data', $page_data);
            } else {
                $this->view->assign('no_pagination', '1');
            }
            $this->load->model('Users_model');
            $this->view->assign('user_account', $this->Users_model->get_user_account($user_id));
            $this->load->model('Currencies_model');
            $this->view->assign('base_currency', $this->Currencies_model->get_currency_default(true));
            $this->view->assign('action', 'services/form/' . $service_gid);
            $this->view->assign('reset_storage', $reset_storage);
            $this->view->assign('no_listings', $no_listings);
            $this->view->assign('service', $service);
            $this->view->assign('data', $listings);
            $this->view->render('service_form_listings');
    }
}
