<?php

namespace Pg\Modules\Services\Models;

/**
 * Services module
 *
 * @package     PG_RealEstate
 *
 * @copyright   Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author      Pilot Group Ltd <http://www.pilotgroup.net/>
 */
if (!defined('BASEPATH')) {
    exit('No direct script access allowed');
}

define('SERVICES_TEMPLATES_TABLE', DB_PREFIX . 'services_templates');
define('SERVICES_TABLE', DB_PREFIX . 'services');
define('SERVICES_LOG_TABLE', DB_PREFIX . 'services_log');

use Pg\Modules\Services\Models\Services_users_model;

/**
 * Services main model
 *
 * @package     PG_RealEstate
 * @subpackage  Services
 *
 * @category    models
 *
 * @copyright   Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author      Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Services_model extends \Model
{

    /**
     * Link to CodeIgniter object
     *
     * @var object
     */
    protected $ci;

    /**
     * Template properties in data source
     *
     * data_admin = array("key" => "type") type = text/string/int/price/checkbox
     * data_user = array("key" => "type") type = text/string/int/price/checkbox/hidden (hidden - user form controller will be wait get or post data)
     *
     * @var array
     */
    protected $template_fields = array(
        'id',
        'gid',
        'callback_module',
        'callback_model',
        'callback_buy_method',
        'callback_activate_method',
        'callback_validate_method',
        'price_type',
        'data_admin',
        'data_user',
        'date_add',
        'moveable',
        'is_membership',
        'data_membership',
        'data_membership_service',
    );

    /**
     * Service properties in data source
     *
     * @var array
     */
    protected $service_fields = array(
        "id",
        "gid",
        "template_gid",
        "type",
        'user_type_disabled',
        'user_type_disabled_code',
        "pay_type",
        "status",
        "price",
        "data_admin",
        "id_membership",
        "file_name",
        "date_add",
    );

    /**
     * Templates cache by identifiers
     *
     * @var array
     */
    protected $cache_template_by_id = array();

    /**
     * Templates cache by GUIDs
     *
     * @var array
     */
    protected $cache_template_by_gid = array();

    /**
     * Services cache by identifiers
     *
     * @var array
     */
    protected $cache_service_by_id = array();

    /**
     * Services cache by guids
     *
     * @var array
     */
    protected $cache_service_by_gid = array();

    /**
     * Class contructor
     *
     * @return Service_model
     */
    public function __construct()
    {
        parent::__construct();
        $this->ci = &get_instance();
    }

    /**
     * Return template object by identifier
     *
     * @param integer $id template identifier
     *
     * @return array
     */
    public function get_template_by_id($id)
    {
        if (empty($this->cache_template_by_id[$id])) {
            $result = $this->ci->db->select(implode(", ", $this->template_fields))->from(SERVICES_TEMPLATES_TABLE)->where("id", $id)->get()->result_array();
            $return = (!empty($result)) ? $this->format_template($result[0]) : array();
            $this->cache_template_by_id[$id] = $this->cache_template_by_gid[$return["gid"]] = $return;
        }

        return $this->cache_template_by_id[$id];
    }

    /**
     * Return template object by GUID
     *
     * @param string $gid template guid
     *
     * @return array
     */
    public function get_template_by_gid($gid)
    {
        if (empty($this->cache_template_by_gid[$gid])) {
            $result = $this->ci->db->select(implode(", ", $this->template_fields))
                    ->from(SERVICES_TEMPLATES_TABLE)
                    ->where("gid", $gid)
                    ->get()->result_array();
            $return = (!empty($result)) ? $this->format_template($result[0]) : array();
            if (empty($return)) {
                return array();
            }
            $this->cache_template_by_gid[$gid] = $this->cache_template_by_id[$return["id"]] = $return;
        }

        return $this->cache_template_by_gid[$gid];
    }

    /**
     * Format template object
     *
     * @param array $data template data
     *
     * @return array
     */
    public function format_template($data)
    {
        $templates = $this->format_templates(array($data));
        return array_shift($templates);
    }

    /**
     * Format templates' data
     *
     * @param array $data templates' data
     *
     * @return array
     */
    public function format_templates($data)
    {
        foreach ($data as $key => $template) {
            if (!empty($template["data_admin"])) {
                $temp = unserialize($template["data_admin"]);
                if (!empty($temp)) {
                    foreach ($temp as $param => $type) {
                        $template["data_admin_array"][$param] = array(
                            "gid" => $param,
                            "type" => $type,
                            "name" => l('admin_param_name_' . $template["id"] . "_" . $param, 'services'),
                        );
                    }
                }
            }
            if (!empty($template["data_user"])) {
                $temp = unserialize($template["data_user"]);
                if (!empty($temp)) {
                    foreach ($temp as $param => $type) {
                        $template["data_user_array"][$param] = array(
                            "gid" => $param,
                            "type" => $type,
                            "name" => l('admin_param_name_' . $template["id"] . "_" . $param, 'services'),
                        );
                    }
                }
            }
            if (!empty($template["data_membership"])) {
                $temp = unserialize($template["data_membership"]);
                if (!empty($temp)) {
                    foreach ($temp as $param => $type) {
                        $template["data_membership_array"][$param] = array(
                            "gid" => $param,
                            "type" => $type,
                            "name" => l('admin_param_name_' . $template["id"] . "_" . $param, 'services'),
                            "name_lang_gid" => 'admin_param_name_' . $template["id"] . "_" . $param,
                        );
                    }
                } else {
                    $template["data_membership_array"] = array();
                }
            } else {
                $template["data_membership_array"] = array();
            }

            if (!empty($template["data_membership_service"])) {
                $template["data_membership_service_array"] = unserialize($template["data_membership_service"]);
            } else {
                $template["data_membership_service_array"] = array();
            }

            $template["name"] = l('template_name_' . $template["id"], 'services');

            $data[$key] = $template;
        }

        return $data;
    }

    /**
     * Return templates objects as array
     *
     * @param array $params            filter parameters
     * @param array $filter_object_ids filter identifiers
     * @param array $order_by          sorting data
     *
     * @return array
     */
    public function get_template_list($params = array(), $filter_object_ids = null, $order_by = null)
    {
        $this->ci->db->select(implode(", ", $this->template_fields))->from(SERVICES_TEMPLATES_TABLE);

        if (isset($params["where"]) && is_array($params["where"]) && count($params["where"])) {
            foreach ($params["where"] as $field => $value) {
                $this->ci->db->where($field, $value);
            }
        }

        if (isset($params["where_in"]) && is_array($params["where_in"]) && count($params["where_in"])) {
            foreach ($params["where_in"] as $field => $value) {
                $this->ci->db->where_in($field, $value);
            }
        }

        if (isset($params["where_sql"]) && is_array($params["where_sql"]) && count($params["where_sql"])) {
            foreach ($params["where_sql"] as $value) {
                $this->ci->db->where($value);
            }
        }

        if (isset($filter_object_ids) && is_array($filter_object_ids) && count($filter_object_ids)) {
            $this->ci->db->where_in("id", $filter_object_ids);
        }

        if (is_array($order_by) && count($order_by) > 0) {
            foreach ($order_by as $field => $dir) {
                $this->ci->db->order_by($field . " " . $dir);
            }
        }

        $results = $this->ci->db->get()->result_array();
        if (!empty($results) && is_array($results)) {
            foreach ($results as $k => $r) {
                $return[$r["gid"]] = $r;
                $return[$r["gid"]]["name"] = l('template_name_' . $r["id"], 'services');
            }
            return $return;
        }

        return array();
    }

    /**
     * Return number of templates objects
     *
     * @param array $params            filter parameters
     * @param array $filter_object_ids filter identifiers
     *
     * @return integer
     */
    public function get_template_count($params = array(), $filter_object_ids = null)
    {
        $this->ci->db->select("COUNT(*) AS cnt")->from(SERVICES_TEMPLATES_TABLE);

        if (isset($params["where"]) && is_array($params["where"]) && count($params["where"])) {
            foreach ($params["where"] as $field => $value) {
                $this->ci->db->where($field, $value);
            }
        }

        if (isset($params["where_in"]) && is_array($params["where_in"]) && count($params["where_in"])) {
            foreach ($params["where_in"] as $field => $value) {
                $this->ci->db->where_in($field, $value);
            }
        }

        if (isset($params["where_sql"]) && is_array($params["where_sql"]) && count($params["where_sql"])) {
            foreach ($params["where_sql"] as $value) {
                $this->ci->db->where($value);
            }
        }

        if (isset($filter_object_ids) && is_array($filter_object_ids) && count($filter_object_ids)) {
            $this->ci->db->where_in("id", $filter_object_ids);
        }

        $results = $this->ci->db->get()->result_array();
        if (!empty($results) && is_array($results)) {
            return intval($results[0]["cnt"]);
        }

        return 0;
    }

    /**
     * Validate template object for storing in data source
     *
     * @param integer $id         template identifier
     * @param array   $data       template data
     * @param array   $langs_data languages data
     *
     * @return array
     */
    public function validate_template($id, $data, $langs_data = null)
    {
        $return = array("errors" => array(), "data" => array(), 'langs_data' => array());

        if (isset($data["gid"])) {
            $data["gid"] = strip_tags($data["gid"]);
            $data["gid"] = preg_replace("/[^a-z0-9\-_]+/i", '', $data["gid"]);

            $return["data"]["gid"] = $data["gid"];

            if (empty($return["data"]["gid"])) {
                $return["errors"][] = l('error_template_code_incorrect', 'services');
            }
        }

        if (isset($data["callback_module"])) {
            $return["data"]["callback_module"] = $data["callback_module"];
        }

        if (isset($data["callback_model"])) {
            $return["data"]["callback_model"] = $data["callback_model"];
        }

        if (isset($data["callback_buy_method"])) {
            $return["data"]["callback_buy_method"] = $data["callback_buy_method"];
        }

        if (isset($data["callback_activate_method"])) {
            $return["data"]["callback_activate_method"] = $data["callback_activate_method"];
        }

        if (isset($data["callback_validate_method"])) {
            $return["data"]["callback_validate_method"] = $data["callback_validate_method"];
        }

        if (isset($data["price_type"])) {
            $return["data"]["price_type"] = intval($data["price_type"]);
        }

        if (isset($data["moveable"])) {
            $return["data"]["moveable"] = intval($data["moveable"]);
        }

        if (isset($data["data_admin"])) {
            $return["data"]["data_admin"] = serialize($data["data_admin"]);
        }

        if (isset($data["data_user"])) {
            $return["data"]["data_user"] = serialize($data["data_user"]);
        }

        if (!empty($langs_data)) {
            foreach ($this->pg_language->languages as $lid => $lang_data) {
                if (!isset($langs_data[$lid])) {
                    $return['errors'][] = l('error_empty_template_name', "services");
                    break;
                } else {
                    $return["langs_data"][$lid] = trim(strip_tags($langs_data[$lid]));
                    if (empty($return["langs_data"][$lid])) {
                        $return['errors'][] = l('error_empty_template_name', "services");
                        break;
                    }
                }
            }
        }

        if (isset($data["is_membership"])) {
            $return["data"]["is_membership"] = intval($data["is_membership"]);
        }

        if (isset($data["data_membership"])) {
            $return["data"]["data_membership"] = serialize($data["data_membership"]);
        }

        if (isset($data["data_membership_service"])) {
            $return["data"]["data_membership_service"] = serialize($data["data_membership_service"]);
        }

        return $return;
    }

    /**
     * Save template object to data source
     *
     * @param integer $id   template identifier
     * @param array   $data template data
     * @param array   $name template name by languages
     *
     * @return integer
     */
    public function save_template($id, $data, $name = null)
    {
        if (is_null($id)) {
            $data["date_add"] = date("Y-m-d H:i:s");
            $this->ci->db->insert(SERVICES_TEMPLATES_TABLE, $data);
            $id = $this->ci->db->insert_id();
        } else {
            $this->ci->db->where('id', $id);
            $this->ci->db->update(SERVICES_TEMPLATES_TABLE, $data);
        }

        if (!empty($name)) {
            $languages = $this->ci->pg_language->languages;
            if (!empty($languages)) {
                $lang_ids = array_keys($languages);
                $this->ci->pg_language->pages->set_string_langs('services', "template_name_" . $id, $name, $lang_ids);
            }
        }
        unset($this->cache_template_by_id[$id]);
        if (!empty($data["gid"])) {
            unset($this->cache_template_by_gid[$data["gid"]]);
        }

        return $id;
    }

    /**
     * Remove template object from data source by identifier
     *
     * @param integer $id template identifier
     *
     * @return void
     */
    public function delete_template($id)
    {
        $this->ci->db->where("id", $id);
        $this->ci->db->delete(SERVICES_TEMPLATES_TABLE);
        $this->ci->pg_language->pages->delete_string("services", "template_name_" . $id);

        return;
    }

    /**
     * Remove template object from data source by guid
     *
     * @param string $gid template guid
     *
     * @return void
     */
    public function delete_template_by_gid($gid)
    {
        $template = $this->get_template_by_gid($gid);
        if (empty($template)) {
            return;
        }
        $this->delete_template($template["id"]);

        return;
    }

    /**
     * Return service object by identifier
     *
     * @param integer $id service identifier
     *
     * @return array
     */
    public function get_service_by_id($id)
    {
        if (empty($this->cache_service_by_id[$id])) {
            $fields = implode(", ", $this->service_fields);

            $result = $this->ci->db->select($fields)
                ->from(SERVICES_TABLE)
                ->where("id", $id)
                ->get()
                ->result_array();

            $return = (!empty($result)) ? $result[0] : array();

            if (!empty($return["data_admin"])) {
                $return["data_admin_array"] = unserialize($return["data_admin"]);
            } else {
                $return["data_admin_array"] = array();
            }

            $this->cache_service_by_id[$id] = $this->cache_service_by_gid[$return["gid"]] = $return;
        }

        return $this->cache_service_by_id[$id];
    }

    /**
     * Return service object by guid
     *
     * @param string $gid service guid
     *
     * @return array
     */
    public function get_service_by_gid($gid)
    {
        if (empty($this->cache_service_by_gid[$gid])) {
            $result = $this->ci->db->select(implode(", ", $this->service_fields))
                    ->from(SERVICES_TABLE)
                    ->where("gid", $gid)
                    ->get()->result_array();
            if (empty($result)) {
                return array();
            }

            $return = $result[0];
            if (!empty($return["data_admin"])) {
                $return["data_admin_array"] = unserialize($return["data_admin"]);
            }

            $this->cache_service_by_gid[$gid] = $this->cache_service_by_id[$return["id"]] = $return;
        }

        return $this->cache_service_by_gid[$gid];
    }

    /**
     * Return services objects from data source as array
     *
     * @param array $params            filter criteria
     * @param array $filter_object_ids filter identifiers
     * @param array $order_by          sorting data
     *
     * @return array
     */
    public function get_service_list($params = array(), $filter_object_ids = null, $order_by = null)
    {
        $this->ci->db->select(SERVICES_TABLE . '.' . implode(", " . SERVICES_TABLE . '.', $this->service_fields));
        $this->ci->db->from(SERVICES_TABLE);

        if (isset($params['join']) && is_array($params['join']) && count($params['join'])) {
            foreach ($params['join'] as $join) {
                $this->ci->db->join($join['table'], $join['condition'], $join['type']);
            }
        }

        if (isset($params["where"]) && is_array($params["where"]) && count($params["where"])) {
            foreach ($params["where"] as $field => $value) {
                $this->ci->db->where($field, $value);
            }
        }

        if (isset($params["where_in"]) && is_array($params["where_in"]) && count($params["where_in"])) {
            foreach ($params["where_in"] as $field => $value) {
                $this->ci->db->where_in($field, $value);
            }
        }

        if (isset($params["where_sql"]) && is_array($params["where_sql"]) && count($params["where_sql"])) {
            foreach ($params["where_sql"] as $value) {
                $this->ci->db->where($value, null, false);
            }
        }

        if (isset($filter_object_ids) && is_array($filter_object_ids) && count($filter_object_ids)) {
            $this->ci->db->where_in("id", $filter_object_ids);
        }

        if (is_array($order_by) && count($order_by) > 0) {
            foreach ($order_by as $field => $dir) {
                $this->ci->db->order_by($field . " " . $dir);
            }
        }

        $results = $this->ci->db->get()->result_array();

        if (!empty($results) && is_array($results)) {
            return $this->format_service($results);
        }

        return array();
    }

    /**
     * Format service object
     *
     * @param array $data service data
     *
     * @return array
     */
    public function format_service($data)
    {
        $template_gids = array();
        foreach ($data as $service) {
            if (empty($template_gids) || !in_array($service["template_gid"], $template_gids)) {
                $template_gids[] = $service["template_gid"];
            }
        }

        $params["where_in"]["gid"] = $template_gids;
        $templates = $this->get_template_list($params);

        foreach ($data as $k => $service) {
            $data[$k]["template"] = $this->format_template($templates[$service["template_gid"]]);

            if (!empty($data[$k]['user_type_disabled'])) {
                $data[$k]['user_type_disabled_array'] = (array) unserialize($data[$k]['user_type_disabled']);
            } else {
                $data[$k]['user_type_disabled_array'] = array();
            }

            $data[$k]["data_admin"] = unserialize($service["data_admin"]);

            if (isset($service["id"])) {
                $data[$k]["name"] = l('service_name_' . $service["id"], 'services');
            }
        }

        return $data;
    }

    /**
     * Return number of service objects from data source
     *
     * @param array $params            filter criteria
     * @param array $filter_object_ids filter identifiers
     *
     * @return integer
     */
    public function get_service_count($params = array(), $filter_object_ids = null)
    {
        $this->ci->db->select("COUNT(*) AS cnt")->from(SERVICES_TABLE);

        if (isset($params['join']) && is_array($params['join']) && count($params['join'])) {
            foreach ($params['join'] as $join) {
                $this->ci->db->join($join['table'], $join['condition'], $join['type']);
            }
        }

        if (isset($params["where"]) && is_array($params["where"]) && count($params["where"])) {
            foreach ($params["where"] as $field => $value) {
                $this->ci->db->where($field, $value);
            }
        }

        if (isset($params["where_in"]) && is_array($params["where_in"]) && count($params["where_in"])) {
            foreach ($params["where_in"] as $field => $value) {
                $this->ci->db->where_in($field, $value);
            }
        }

        if (isset($params["where_sql"]) && is_array($params["where_sql"]) && count($params["where_sql"])) {
            foreach ($params["where_sql"] as $value) {
                $this->ci->db->where($value, null, false);
            }
        }

        if (isset($filter_object_ids) && is_array($filter_object_ids) && count($filter_object_ids)) {
            $this->ci->db->where_in("id", $filter_object_ids);
        }

        $results = $this->ci->db->get()->result_array();
        if (!empty($results) && is_array($results)) {
            return intval($results[0]["cnt"]);
        }

        return 0;
    }

    /**
     * Validate service object for storing in data source
     *
     * @param integer $id         service identifier
     * @param array   $data       service data
     * @param array   $langs_data languages data
     *
     * @return array
     */
    public function validate_service($id, $data, $langs_data = null)
    {
        $return = array("errors" => array(), "data" => array(), 'langs_data' => array());

        if (isset($data["gid"])) {
            $data["gid"] = strip_tags($data["gid"]);
            $data["gid"] = preg_replace("/[^a-z0-9\-_]+/i", '', $data["gid"]);

            $return["data"]["gid"] = $data["gid"];

            if (empty($return["data"]["gid"])) {
                $return["errors"][] = l('error_service_code_incorrect', 'services');
            } else {
                $param["where"]["gid"] = $return["data"]["gid"];
                if ($id) {
                    $param["where"]["id <>"] = $id;
                }
                $gid_counts = $this->get_service_count($param);
                if ($gid_counts > 0) {
                    $return["errors"][] = l('error_service_code_exists', 'services');
                }
            }
        }

        if (isset($data["template_gid"])) {
            $return["data"]["template_gid"] = $data["template_gid"];
        }

        if (isset($data["type"])) {
            $return["data"]["type"] = strval($data["type"]);
        }

        if (isset($data['user_type_disabled'])) {
            if (!is_array($data['user_type_disabled'])) {
                $data['user_type_disabled'] = array();
            }
            if (empty($data['user_type_disabled'])) {
                //$return["errors"][] = l('error_usertypes_empty', 'services');
            }
            $return['data']['user_type_disabled'] = serialize($data['user_type_disabled']);
            $return['data']['user_type_disabled_code'] = $this->_userTypesToDec($data['user_type_disabled']);
        } elseif (!$id) {
            //$return["errors"][] = l('error_usertype_empty', 'services');
        }

        if (isset($data["pay_type"])) {
            $return["data"]["pay_type"] = intval($data["pay_type"]);
        }

        if (isset($data["status"])) {
            $return["data"]["status"] = intval($data["status"]);
        }

        if (isset($data["price"])) {
            $return["data"]["price"] = floatval($data["price"]);
        }

        if (isset($data["data_admin"]) && !empty($data["data_admin"])) {
            $template_data = $this->get_template_by_gid($data["template_gid"]);
            foreach ($data["data_admin"] as $key => $value) {
                switch ($template_data["data_admin_array"][$key]["type"]) {
                    case "string": $value = trim(strip_tags($value));
                        break;
                    case "int": $value = intval($value);
                        break;
                    case "price": $value = sprintf("%01.2f", floatval($value));
                        break;
                    case "text": break;
                    case "checkbox": $value = (intval($value) > 0) ? 1 : 0;
                        break;
                }
                $data["data_admin"][$key] = $value;
            }
            $return["data"]["data_admin"] = serialize($data["data_admin"]);
        }

        if (isset($data["id_membership"]) && !empty($data["id_membership"])) {
            $return["data"]["id_membership"] = intval($data["id_membership"]);
        }

        if (!empty($langs_data)) {
            $default_lang_id = $this->ci->pg_language->current_lang_id;
            if (!isset($langs_data[$default_lang_id])) {
                $return['errors'][] = l('error_empty_service_name', "services");
            } else {
                $return["langs_data"][$default_lang_id] = trim(strip_tags($langs_data[$default_lang_id]));
                if (empty($return["langs_data"][$default_lang_id])) {
                    $return['errors'][] = l('error_empty_service_name', "services");
                } else {
                    foreach ($this->ci->pg_language->languages as $lid => $lang_data) {
                        if ($lid == $default_lang_id) {
                            continue;
                        }
                        if (!isset($langs_data[$lid])) {
                            $return["langs_data"][$lid] = $return["langs_data"][$default_lang_id];
                        } else {
                            $return["langs_data"][$lid] = trim(strip_tags($langs_data[$lid]));
                            if (empty($return["langs_data"][$lid])) {
                                $return["langs_data"][$lid] = $return["langs_data"][$default_lang_id];
                            }
                        }
                    }
                }
            }
        }

        return $return;
    }

    /**
     * Save service object to data source
     *
     * @param integer $id   service identifier
     * @param array   $data service data
     * @param array   $name service name by languages
     *
     * @return integer
     */
    public function save_service($id, $data, $name = array())
    {
        if (is_null($id)) {
            $data["date_add"] = date("Y-m-d H:i:s");
            $this->ci->db->insert(SERVICES_TABLE, $data);
            $id = $this->ci->db->insert_id();
        } else {
            $this->ci->db->where('id', $id);
            $this->ci->db->update(SERVICES_TABLE, $data);
        }
        if (!empty($name)) {
            $languages = $this->ci->pg_language->languages;
            if (!empty($languages)) {
                $lang_ids = array_keys($languages);
                $this->ci->pg_language->pages->set_string_langs('services', "service_name_" . $id, $name, $lang_ids);
            }
        }

        unset($this->cache_service_by_id[$id]);
        if (!empty($data["gid"])) {
            unset($this->cache_service_by_gid[$data["gid"]]);
        }

        return $id;
    }

    /**
     * Remove service object from data source by identifier
     *
     * @param integer $id service identifier
     *
     * @return void
     */
    public function delete_service($id)
    {
        $this->ci->db->where("id", $id);
        $this->ci->db->delete(SERVICES_TABLE);
        $this->ci->pg_language->pages->delete_string("services", "service_name_" . $id);

        return;
    }

    /**
     * Remove service object from data source by guid
     *
     * @param string $gid service guid
     *
     * @return void
     */
    public function delete_service_by_gid($gid)
    {
        $template = $this->get_service_by_gid($gid);
        if (empty($template)) {
            return;
        }
        $this->delete_service($template["id"]);
        return;
    }

    /**
     * Logging of service used
     *
     * @param integer $id_user    user identifier
     * @param integer $id_service service identifier
     * @param array   $user_data  user data
     *
     * @return void
     */
    public function add_service_log($id_user, $id_service, $user_data)
    {
        $data = array(
            "id_user" => $id_user,
            "id_service" => $id_service,
            "user_data" => serialize($user_data),
            "date_add" => date("Y-m-d H:i:s"),
        );
        $this->ci->db->insert(SERVICES_LOG_TABLE, $data);
    }

    /**
     * Validate service payment object
     *
     * @param integer $id_service service identifier
     * @param array   $user_data  user data
     * @param float   $price      service price
     *
     * @return array
     */
    public function validate_service_payment($id_service, $user_data, $price)
    {
        $return = array("errors" => array(), "data" => array());
        $service_data = $this->get_service_by_id($id_service);
        $template_data = $this->get_template_by_gid($service_data["template_gid"]);

        $return["data"]["price"] = $price = floatval($price);

        if (!empty($template_data["data_user_array"])) {
            foreach ($template_data["data_user_array"] as $gid => $param) {
                $value = isset($user_data[$gid]) ? $user_data[$gid] : "";
                switch ($param["type"]) {
                    case "string": $value = trim(strip_tags($value));
                        break;
                    case "int": $value = intval($value);
                        break;
                    case "price": $value = sprintf("%01.2f", floatval($value));
                        break;
                    case "text": break;
                    case "checkbox": $value = (intval($value) > 0) ? 1 : 0;
                        break;
                    case "hidden":
                        if (empty($value)) {
                            $return["errors"][] = l('error_parametr_incorrect', 'services') . $param["name"];
                        }
                        break;
                }

                $return["data"]["data_user"][$gid] = $value;
            }
        }

        return $return;
    }

    /**
     * Validate user data by module callback
     *
     * @param integer $id_service service identifier
     * @param array   $user_data  user data
     * @param integer $id_user    user identifier
     * @param float   $price      service price
     *
     * @return array
     */
    public function validate_service_original_model($id_service, $user_data, $id_user, $price)
    {
        $service_data = $this->get_service_by_id($id_service);
        $template_data = $this->get_template_by_gid($service_data["template_gid"]);

        $model_name = ucfirst($template_data["callback_model"]);
        $model_path = strtolower($template_data["callback_module"] . "/models/") . $model_name;
        $this->ci->load->model($model_path);
        $return = $this->ci->$model_name->{$template_data["callback_validate_method"]}(
            $id_user, $user_data, $service_data, $price
        );

        return $return;
    }

    /**
     * Service payment by write off from account
     *
     * @param integer $id_service service identifier
     * @param integer $id_user    user identifier
     * @param array   $user_data  user data
     * @param float   $price      service price
     *
     * @return boolean
     */
    public function account_payment($id_service, $id_user, $user_data, $price)
    {
        if ($this->pg_module->is_module_active('users_payments')) {
            $this->ci->load->model("Users_payments_model");

            $service_data = $this->get_service_by_id($id_service);
            $t = $this->format_service(array($service_data));
            $service_data = $t[0];
            $message = l('service_payment', 'services') . $service_data["name"];

            $return = $this->ci->Users_payments_model->write_off_user_account($id_user, $price, $message);
            if ($return === true) {
                $payment_data = array(
                    "id_user" => $id_user,
                    "amount" => $price,
                    "payment_data" => array("id_service" => $id_service, "user_data" => $user_data),
                );
                $this->payment_service_status($payment_data, 1);
                return true;
            } else {
                return $return;
            }
        }

        return false;
    }

    /**
     * Service payment by payment system
     *
     * @param string  $system_gid payment system guid
     * @param integer $id_user    user identifier
     * @param integer $id_service service identifier
     * @param array   $user_data  user data
     * @param float   $price      service price
     *
     * @return boolean
     */
    public function system_payment($system_gid, $id_user, $id_service, $user_data, $price)
    {
        // TODO: check payment is for service
        if ($id_service) {
            $service_data = $this->get_service_by_id($id_service);
            $t = $this->format_service(array($service_data));
            $service_data = $t[0];
            $name = l('service_payment', 'services') . $service_data["name"];
        } else {
            $name = l('services_payment', 'services');
        }

        $this->ci->load->model("Currencies_model");
        $base_currency = $this->ci->Currencies_model->get_currency_default(true);

        $payment_data["name"] = $name;
        $payment_data["id_service"] = $id_service;
        $payment_data["user_data"] = $user_data;

        $this->ci->load->helper('payments');
        send_payment('services', $id_user, $price, $base_currency['gid'], $system_gid, $payment_data, true);
    }

    /**
     * Set status of payment service
     *
     * This method is callback for payment module
     *
     * @param array   $payment_data   payment data
     * @param integer $payment_status payment status
     *
     * @return void
     */
    public function payment_service_status($payment_data, $payment_status)
    {
        if ($payment_status != 1) {
            return;
        }

        if (empty($payment_data["payment_data"]["id_service"]) && !empty($payment_data["payment_data"]["user_data"]["services"])) {
            foreach ($payment_data["payment_data"]["user_data"]["services"] as $service_gid) {
                $this->applyService($service_gid, $payment_data, $payment_status);
            }
        } else {
            $user_id = $payment_data["id_user"];
            $service_id = $payment_data["payment_data"]["id_service"];

            $service_data = $this->get_service_by_id($service_id);
            $template = $this->get_template_by_gid($service_data["template_gid"]);

            $service_admin_data = isset($service_data["data_admin_array"]) ? $service_data["data_admin_array"] : array();

            if (!empty($payment_data["payment_data"]["user_data"])) {
                $user_data = $payment_data["payment_data"]["user_data"];
            } else {
                $user_data = array();
            }

            $price = $payment_data["amount"];

            $model_name = ucfirst($template["callback_model"]);
            $model_path = strtolower($template["callback_module"] . "/models/") . $model_name;
            $this->ci->load->model($model_path);

            if (!empty($payment_data['payment_data']['id_users_package'])) {
                $users_package_id = intval($payment_data['payment_data']['id_users_package']);
            } else {
                $users_package_id = 0;
            }

            if (!empty($payment_data['payment_data']['id_users_membership'])) {
                $users_membership_id = intval($payment_data['payment_data']['id_users_membership']);
            } else {
                $users_membership_id = 0;
            }

            $is_activate_service = true;

            if ($users_membership_id) {
                $this->ci->load->model('Memberships_model');
                $this->ci->load->model('Memberships_users_model');
                $user_membership_data = $this->ci->Memberships_users_model->getUserMembershipById($users_membership_id);
                $user_membership_data = $this->ci->Memberships_users_model->formatMembership($user_membership_data);
                $service_admin_data['period'] = $this->ci->Memberships_model->getMembershipDays($user_membership_data['membership_info']);
                $service_admin_data['period_count'] = $user_membership_data['membership_info']['period_count'];
                $service_admin_data['period_type'] = $user_membership_data['membership_info']['period_type'];
            }

            $service_data['data_admin'] = serialize($service_admin_data);
            if (isset($service_data['data_admin_array'])) {
                unset($service_data['data_admin_array']);
            }

            $template['data_admin'] = serialize(!empty($template['data_admin_array']) ? $template['data_admin_array'] : array());
            $template['data_user'] = serialize(!empty($template['data_user_array']) ? $template['data_user_array'] : array());
            $template['data_membership'] = serialize(!empty($template['data_membership_array']) ? $template['data_membership_array'] : array());
            $template['data_membership_service'] = serialize(!empty($template['data_membership_service_array']) ? $template['data_membership_service_array'] : array());

            $user_service_data = array(
                'id_user' => $user_id,
                'service_gid' => $service_data['gid'],
                'template_gid' => $template['gid'],
                'service_data' => $service_data,
                'template_data' => $template,
                'payment_data' => $payment_data,
                'id_users_package' => $users_package_id,
                'id_users_membership' => $users_membership_id,
                'status' => 1,
            );

            $this->ci->load->model('services/models/Services_users_model');
            $user_service_id = $this->ci->Services_users_model->save_service(null, $user_service_data);

            $user_service = $this->ci->Services_users_model->get_service_by_id($user_service_id);

            $is_activate_service = $this->ci->$model_name->{$template["callback_buy_method"]}(
                $user_service, $user_data, $price
            );

            if ($is_activate_service) {
                $user_service = $this->ci->Services_users_model->get_service_by_id($user_service_id);
                $this->ci->$model_name->{$template["callback_activate_method"]}($user_service, $user_data, $price);
            }

            $this->add_service_log($user_id, $service_id, $user_data);
        }
    }

    private function applyService($service_gid, $payment_data, $payment_status)
    {
        $user_id = $payment_data["id_user"];

        $service_data = $this->get_service_by_gid($service_gid);
        $template = $this->get_template_by_gid($service_data["template_gid"]);

        $service_admin_data = isset($service_data["data_admin_array"]) ? $service_data["data_admin_array"] : array();

        if (!empty($payment_data["payment_data"]["user_data"])) {
            $user_data = $payment_data["payment_data"]["user_data"];
        } else {
            $user_data = array();
        }

        $price = $payment_data["amount"];

        $model_name = ucfirst($template["callback_model"]);
        $model_path = strtolower($template["callback_module"] . "/models/") . $model_name;
        $this->ci->load->model($model_path);

        if (!empty($payment_data['payment_data']['id_users_package'])) {
            $users_package_id = intval($payment_data['payment_data']['id_users_package']);
        } else {
            $users_package_id = 0;
        }

        if (!empty($payment_data['payment_data']['id_users_membership'])) {
            $users_membership_id = intval($payment_data['payment_data']['id_users_membership']);
        } else {
            $users_membership_id = 0;
        }

        $is_activate_service = true;

        if ($users_membership_id) {
            $this->ci->load->model('Memberships_model');
            $this->ci->load->model('Memberships_users_model');
            $user_membership_data = $this->ci->Memberships_users_model->getUserMembershipById($users_membership_id);
            $user_membership_data = $this->ci->Memberships_users_model->formatMembership($user_membership_data);
            $service_admin_data['period'] = $this->ci->Memberships_model->getMembershipDays($user_membership_data['membership_info']);
            $service_admin_data['period_count'] = $user_membership_data['membership_info']['period_count'];
            $service_admin_data['period_type'] = $user_membership_data['membership_info']['period_type'];
        }

        $service_data['data_admin'] = serialize($service_admin_data);
        if (isset($service_data['data_admin_array'])) {
            unset($service_data['data_admin_array']);
        }

        $template['data_admin'] = serialize(!empty($template['data_admin_array']) ? $template['data_admin_array'] : array());
        $template['data_user'] = serialize(!empty($template['data_user_array']) ? $template['data_user_array'] : array());
        $template['data_membership'] = serialize(!empty($template['data_membership_array']) ? $template['data_membership_array'] : array());
        $template['data_membership_service'] = serialize(!empty($template['data_membership_service_array']) ? $template['data_membership_service_array'] : array());

        $user_service_data = array(
            'id_user' => $user_id,
            'service_gid' => $service_data['gid'],
            'template_gid' => $template['gid'],
            'service_data' => $service_data,
            'template_data' => $template,
            'payment_data' => $payment_data,
            'id_users_package' => $users_package_id,
            'id_users_membership' => $users_membership_id,
            'status' => 1,
        );

        $this->ci->load->model('services/models/Services_users_model');
        $user_service_id = $this->ci->Services_users_model->save_service(null, $user_service_data);

        $user_service = $this->ci->Services_users_model->get_service_by_id($user_service_id);

        $is_activate_service = $this->ci->$model_name->{$template["callback_buy_method"]}(
            $user_service, $user_data, $price
        );

        if ($is_activate_service) {
            $user_service = $this->ci->Services_users_model->get_service_by_id($user_service_id);
            $this->ci->$model_name->{$template["callback_activate_method"]}($user_service, $user_data, $price);
        }

        $this->add_service_log($user_id, $service_data['id'], $user_data);
    }

    /**
     * Check service is used
     *
     * @param string $gid service guid
     *
     * @return integer
     */
    public function is_service_active($gid = '')
    {
        $s = $this->get_service_by_gid($gid);

        return intval($s['status']);
    }

    /**
     * Exoprt languages data from services module
     *
     * @param array $items     data guids
     * @param array $langs_ids languages identifiers
     *
     * @return array
     */
    public function export_langs($items, $langs_ids = null)
    {
        foreach ($items as $type => $gids) {
            if ($type != 'param') {
                $method = 'get_' . $type . '_by_gid';
                foreach ($gids as $gid) {
                    $element = $this->$method($gid);
                    $services[$gid] = $type . '_name_' . $element['id'];
                }
            } else {
                $method = 'get_template_by_gid';
                foreach ($gids as $template => $param_gids) {
                    $element = $this->$method($template);
                    if (is_array($param_gids)) {
                        foreach ($param_gids as $k => $v) {
                            $services[$template . "_" . $v] = 'admin_param_name_' . $element['id'] . '_' . $v;
                        }
                    } else {
                        $services[$template . "_" . $param_gids] = 'admin_param_name_' . $element['id'] . '_' . $param_gids;
                    }
                }
            }
        }
        $langs_db = $this->ci->pg_language->export_langs('services', $services, $langs_ids);
        $lang_codes = array_keys($langs_db);
        foreach ($langs_ids as $lang_code) {
            //$lang_data[$lang_code] = array_combine(array_keys($services), $langs_db[$lang_code]);
            foreach ($services as $key => $value) {
                $lang_data[$key][$lang_code] = $langs_db[$value][$lang_code];
            }
        }

        return $lang_data;
    }

    /**
     * Import languages data to services module
     *
     * @param array $services   data guids
     * @param array $langs_data languages data
     *
     * @return boolean
     */
    public function update_langs($services, $langs_data)
    {
        foreach ($services as $type => $gids) {
            if ($type != 'param') {
                $method = 'get_' . $type . '_by_gid';
                foreach ($gids as $gid) {
                    $element = $this->$method($gid);
                    $lang_data = $langs_data[$gid];
                    $this->ci->pg_language->pages->set_string_langs('services', $type . '_name_' . $element['id'], $lang_data, array_keys($lang_data));
                }
            } else {
                $method = 'get_template_by_gid';
                foreach ($gids as $template => $param_gids) {
                    $element = $this->$method($template);
                    if (is_array($param_gids)) {
                        foreach ($param_gids as $k => $v) {
                            if (empty($langs_data[$template . "_" . $v])) {
                                log_message('error', '(services) Empty langs data (' . $template . "_" . $v . ')');
                                continue;
                            }
                            $lang_data = $langs_data[$template . "_" . $v];
                            $this->ci->pg_language->pages->set_string_langs('services', 'admin_param_name_' . $element['id'] . '_' . $v, $lang_data, array_keys($lang_data));
                        }
                    } else {
                        $lang_data = $langs_data[$template . "_" . $param_gids];
                        $this->ci->pg_language->pages->set_string_langs('services', 'admin_param_name_' . $element['id'] . '_' . $param_gids, $lang_data, array_keys($lang_data));
                    }
                }
            }
        }

        return true;
    }

    /**
     * Return active services by templates guids
     *
     * @param integer $user_id        user identifier
     * @param array   $templates_gids templates guids
     *
     * @return array
     */
    public function get_active_services($user_id, $templates_gids)
    {
        $services_params["where_in"]["template_gid"] = (array) $templates_gids;
        $services_params["where"]["id_membership"] = 0;
        $services_params["where"]["status"] = 1;
        $services = $this->get_service_list($services_params);

        return $services;
    }

    /**
     * Return active services by user type and templates guids
     *
     * @param integer $user_type      user type
     * @param array   $templates_gids templates guids
     *
     * @return array
     */
    public function get_user_type_active_services($user_type, array $templates_gids = [])
    {
        if (!empty($templates_gids)) {
            $services_params["where_in"]["template_gid"] = (array) $templates_gids;
        }
        $services_params["where"]["id_membership"] = 0;

        $criteria = $this->ci->Services_model->getUserTypeCriteria($user_type);
        $services_params = array_merge_recursive($services_params, $criteria);
        $services_params["where"]["status"] = 1;
        
        // TODO: частным лицам не показываем сервис featured
        if ($this->ci->session->userdata('auth_type') == 'user' &&
            $this->ci->session->userdata('user_type') == 'private') {
            $services_params["where"]["template_gid !="] = 'featured_template';
        }
        
        return $this->get_service_list($services_params);
    }

    /**
     * Return membership's services as array
     *
     * @param integer $membership_id membership identifier
     *
     * @return array
     */
    public function getMembershipServices($membership_id)
    {
        $where = array('where' => array('id_membership' => $membership_id));

        return $this->get_service_list($where);
    }

    /**
     * Create services for membership
     *
     * @param integer $membership_id membership identifier
     *
     * @return void
     */
    public function create_membership_services($membership_id)
    {
        $service_counter = $this->ci->pg_module->get_module_config('services', 'service_counter');
        $service_counter = intval($service_counter);

        $templates_data = $this->get_template_list();
        $templates_data = $this->format_templates($templates_data);
        foreach ($templates_data as $template_data) {
            if (!$template_data['is_membership']) {
                continue;
            }

            $data_admin = array();

            foreach ($template_data['data_membership_array'] as $name => $type) {
                if (isset($template_data['data_membership_service_array'][$name])) {
                    $data_admin[$name] = $template_data['data_membership_service_array'][$name];
                } else {
                    switch ($type) {
                        case 'int':
                            $data_admin[$name] = 0;
                            break;
                        default:
                            $data_admin[$name] = '';
                            break;
                    }
                }
            }

            $lds = array();

            if (!empty($template_data['lds_array'])) {
                foreach ($template_data['lds_array'] as $name => $value) {
                    $lds[$name] = $value;
                }
            }

            ++$service_counter;

            $service_data = array(
                'gid' => 'service_' . $service_counter,
                'template_gid' => $template_data['gid'],
                'pay_type' => 1,
                'type' => 'membership',
                'data_admin' => serialize($data_admin),
                'status' => 0,
                'id_membership' => $membership_id,
            );
            $service_id = $this->save_service(null, $service_data);

            $lang_data = array();

            foreach ($this->ci->pg_language->languages as $lid => $data) {
                $lang_data[$lid] = l('template_name_' . $template_data['id'], 'services', $lid);
            }

            $this->ci->pg_language->pages->set_string_langs('services', 'service_name_' . $service_id, $lang_data, array_keys($lang_data));
        }

        $this->ci->pg_module->set_module_config('services', 'service_counter', $service_counter);
    }

    /**
     * Remove services from memberships
     *
     * @param integer $membership_id membership identifier
     *
     * @return void
     */
    public function deleteMembershipServices($membership_id)
    {
        $services_data = $this->get_service_list(array('where' => array('id_membership' => $membership_id)));
        foreach ($services_data as $service_data) {
            $this->delete_service($service_data['id']);
        }
    }

    /**
     * Update status of memberships services
     *
     * Available status: 1 - active, 0 - inactive
     *
     * @param integer $membership_id       membership identifier
     * @param integer $status              activity status
     * @param array   $active_services_ids active memebership services
     *
     * @return void
     */
    public function updateMembershipServicesStatus($membership_id, $status, $active_services_ids = array())
    {
        $services_data = $this->get_service_list(array('where' => array('id_membership' => $membership_id)));
        foreach ($services_data as $service_data) {
            $service_status = ($status && in_array($service_data['id'], $active_services_ids)) ? 1 : 0;
            $save_data = array('status' => $service_status);
            $this->save_service($service_data['id'], $save_data);
        }
    }

    /**
     * Update data of memberships services
     *
     * @param integer $membership_id           membership identifier
     * @param array   $membership_service_data service data
     *
     * @return void
     */
    public function updateMembershipServicesData($membership_id, $membership_service_data)
    {
        $services_data = $this->get_service_list(array('where' => array('id_membership' => $membership_id)));
        foreach ($services_data as $service_data) {
            $data = $membership_service_data[$service_data['id']];
            $data['template_gid'] = $service_data['template_gid'];
            $validate_data = $this->validate_service($service_data['id'], $data);
            if (!empty($validate_data['errors']) || empty($validate_data['data'])) {
                continue;
            }
            $this->save_service($service_data['id'], $validate_data['data']);
        }
    }

    /**
     * Update status of service
     *
     * Available status: 1 - active, 0 - inactive
     *
     * @param integer $service_id service identifier
     * @param integer $status     activity status
     *
     * @return void
     */
    public function updateServiceStatus($service_id, $status)
    {
        $this->save_service($service_id, array('status' => $status));
    }

    /**
     * Convert data from array to numeric
     *
     * @param array $data membership user types
     *
     * @return integer
     */
    private function _userTypesToDec($data)
    {
        $lang_id = $this->ci->pg_language->current_lang_id;

        $this->ci->load->model('Users_model');
        $user_types = $this->ci->Users_model->getUserTypes();
        if (empty($user_types)) {
            return 0;
        }

        $binary_string = '';
        foreach ($user_types as $user_type) {
            $binary_string = (in_array($user_type, $data) ? "1" : "0") . $binary_string;
        }

        return bindec($binary_string);
    }

    /**
     * Return criteria by user type
     *
     * @param string/array $user_type user type
     *
     * @return array
     */
    public function getUserTypeCriteria($user_type)
    {
        if (empty($user_type)) {
            return array();
        }

        if (!is_array($user_type)) {
            $user_type = array($user_type);
        }

        $temp = $this->_userTypesToDec($user_type);

        return array("where_sql" => array("user_type_disabled_code&" . $temp . "!=" . $this->ci->db->escape($temp)));
    }


    public function add_icons()
    {
        $icon_dir = MODULEPATH . "services/install/icons/";
        $this->ci->load->model('services/models/Services_model');

        $allFiles = scandir($icon_dir);
        $files = array_diff($allFiles, array('.', '..'));

        foreach ($files as $file) {
            $path = $icon_dir . $file;
            if (file_exists($path)) {
                if (!is_dir('uploads' . '/services-logo/')) {
                    mkdir('uploads' . '/services-logo/', 0777, true);
                }
                $to_copy = 'uploads' . '/services-logo/' . $file;
                copy($path, $to_copy);
                $gid = 'listings_' . strstr($file, '.', true) .'_service';
                $this->ci->Services_model->saveIcon($gid, $file);
            }
        }
    }

    public function saveIcon($gid, $file_name)
    {
        $needle = 'lift_up';
        $pos = strripos($gid, $needle);
        if ($file_name && $gid) {
            $data = array('file_name' => $file_name);
            if ($pos === false) {
                $this->db->where('gid', $gid);
                $this->db->update(SERVICES_TABLE, $data);
            } else {
                $this->db->query('UPDATE ' . SERVICES_TABLE . ' SET  file_name = "' . $data['file_name'] . '" WHERE  gid LIKE  "%' . $needle . '%"');
            }
        }
    }

}
