<?php

namespace Pg\Modules\Site_map\Models;

/**
 * Site map module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */

/**
 * Site map install model
 *
 * @package 	PG_RealEstate
 * @subpackage 	Site map
 *
 * @category	models
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Site_map_install_model extends \Model
{

    /**
     * Link to CodeIgniter object
     *
     * @var object
     */
    protected $ci;

    /**
     * Menu configuration
     *
     * @var array
     */
    protected $menu = array(
        'user_footer_menu' => array(
            'action' => 'none',
            'name' => '',
            'items' => array(
                'footer-menu-map-item' => array('action' => 'create', 'link' => 'site_map/', 'status' => 1, 'sorter' => 5),
            ),
        ),
    );

    /**
     * Constructor
     *
     * @return Site_map_install_model object
     */
    public function __construct()
    {
        parent::__construct();
        $this->ci = &get_instance();
    }

    /**
     * Install data of menu model
     *
     * @return void
     */
    public function install_menu()
    {
        $this->ci->load->helper('menu');
        foreach ($this->menu as $gid => $menu_data) {
            $this->menu[$gid]['id'] = linked_install_set_menu($gid, $menu_data["action"], $menu_data["name"]);
            linked_install_process_menu_items($this->menu, 'create', $gid, 0, $this->menu[$gid]["items"]);
        }
    }

    /**
     * Import languages of menu module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return boolean
     */
    public function install_menu_lang_update($langs_ids = null)
    {
        if (empty($langs_ids)) {
            return false;
        }
        $langs_file = $this->ci->Install_model->language_file_read('site_map', 'menu', $langs_ids);

        if (!$langs_file) {
            log_message('info', 'Empty menu langs data');
            return false;
        }

        $this->ci->load->helper('menu');

        foreach ($this->menu as $gid => $menu_data) {
            linked_install_process_menu_items($this->menu, 'update', $gid, 0, $this->menu[$gid]["items"], $gid, $langs_file);
        }

        return true;
    }

    /**
     * Export languages of menu module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return array
     */
    public function install_menu_lang_export($langs_ids)
    {
        if (empty($langs_ids)) {
            return false;
        }
        $this->ci->load->helper('menu');

        $return = array();
        foreach ($this->menu as $gid => $menu_data) {
            $temp = linked_install_process_menu_items($this->menu, 'export', $gid, 0, $this->menu[$gid]["items"], $gid, $langs_ids);
            $return = array_merge($return, $temp);
        }

        return array("menu" => $return);
    }

    /**
     * Uninstall data of menu module
     *
     * @return void
     */
    public function deinstall_menu()
    {
        $this->ci->load->helper('menu');
        foreach ($this->menu as $gid => $menu_data) {
            if ($menu_data['action'] == 'create') {
                linked_install_set_menu($gid, 'delete');
            } else {
                linked_install_delete_menu_items($gid, $this->menu[$gid]['items']);
            }
        }
    }

    /**
     * Install module data
     *
     * @return void
     */
    public function _arbitrary_installing()
    {
        $seo_data = array(
            'module_gid' => 'site_map',
            'model_name' => 'Site_map_model',
            'get_settings_method' => 'get_seo_settings',
            'get_rewrite_vars_method' => 'request_seo_rewrite',
            'get_sitemap_urls_method' => 'get_sitemap_xml_urls',
        );
        $this->ci->pg_seo->set_seo_module('site_map', $seo_data);

        // add seo link
        $this->ci->load->model('Seo_model');

        $xml_data = $this->ci->Seo_model->get_xml_route_file_content();

        $data = array(
            'noindex' => 1,
            'title' => '',
            'keyword' => '',
            'description' => '',
            'header' => '',
            'og_title' => '',
            'og_type' => '',
            'og_descrtiption' => '',
            'url_template' => '[text:map]',
        );
        $this->ci->pg_seo->set_settings('user', 'site_map', 'index', $data);

        $xml_data['site_map']['index'] = $this->ci->pg_seo->url_template_transform('site_map', 'index', $data["url_template"], 'base', 'xml');

        $this->ci->Seo_model->set_xml_route_file_content($xml_data);
        $this->ci->Seo_model->rewrite_route_php_file();
    }

    /**
     * Import module languages
     *
     * @param array $langs_ids array languages identifiers
     *
     * @return void
     */
    public function _arbitrary_lang_install($langs_ids = null)
    {
        $langs_file = $this->ci->Install_model->language_file_read("site_map", "arbitrary", $langs_ids);
        if (!$langs_file) {
            log_message("info", "Empty site_map arbitrary langs data");
            return false;
        }

        $post_data = array(
            "title" => $langs_file["seo_tags_index_title"],
            "keyword" => $langs_file["seo_tags_index_keyword"],
            "description" => $langs_file["seo_tags_index_description"],
            "header" => $langs_file["seo_tags_index_header"],
            "og_title" => $langs_file["seo_tags_index_og_title"],
            "og_type" => $langs_file["seo_tags_index_og_type"],
            "og_description" => $langs_file["seo_tags_index_og_description"],
        );
        $this->ci->pg_seo->set_settings("user", "site_map", "index", $post_data);
    }

    /**
     * Export module languages
     *
     * @param array $langs_ids languages identifiers
     *
     * @return array
     */
    public function _arbitrary_lang_export($langs_ids = null)
    {
        if (empty($langs_ids)) {
            return false;
        }

        // arbitrary
        $settings = $this->ci->pg_seo->get_settings("user", "site_map", "index", $langs_ids);
        $arbitrary_return["seo_tags_index_title"] = $settings["title"];
        $arbitrary_return["seo_tags_index_keyword"] = $settings["keyword"];
        $arbitrary_return["seo_tags_index_description"] = $settings["description"];
        $arbitrary_return["seo_tags_index_header"] = $settings["header"];
        $arbitrary_return["seo_tags_index_og_title"] = $settings["og_title"];
        $arbitrary_return["seo_tags_index_og_type"] = $settings["og_type"];
        $arbitrary_return["seo_tags_index_og_description"] = $settings["og_description"];

        return array("arbitrary" => $arbitrary_return);
    }

    /**
     * Uninstall module data
     *
     * @return void
     */
    public function _arbitrary_deinstalling()
    {
        // remove seo link
        $this->ci->load->model('Seo_model');

        $xml_data = $this->ci->Seo_model->get_xml_route_file_content();

        $data = array(
            'noindex' => 0,
            'title' => '',
            'keyword' => '',
            'description' => '',
            'header' => '',
            'og_title' => '',
            'og_type' => '',
            'og_descrtiption' => '',
            'url_template' => '',
        );
        $this->ci->pg_seo->set_settings('user', 'site_map', 'index', $data);

        $xml_data['site_map']['index'] = $this->ci->pg_seo->url_template_transform('site_map', 'index', $data["url_template"], 'base', 'xml');

        $this->ci->Seo_model->set_xml_route_file_content($xml_data);
        $this->ci->Seo_model->rewrite_route_php_file();

        $this->ci->pg_seo->delete_seo_module('site_map');
    }

}
