<?php

namespace Pg\Modules\Start\Controllers;

/**
 * Start module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */

/**
 * Start api side controller
 *
 * @package 	PG_RealEstate
 * @subpackage 	Start
 *
 * @category	controllers
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Api_Start extends \Controller
{

    /**
     * Constructor
     *
     * @return Api_Start
     */
    public function __construct()
    {
        parent::__construct();
    }

    /**
     * Get search tabs
     *
     * @return void
     */
    public function get_search_tabs()
    {
        $operations = array();

        if ($this->pg_module->is_module_active('listings')) {
            $this->load->model('Listings_model');
            $operation_types = $this->Listings_model->get_operation_types(true);
            foreach ($operation_types as $operation_type) {
                $operations[] = array('module' => 'listings', 'type' => $operation_type, 'name' => l('header_' . $operation_type . '_listings', 'listings'));
            }
        }

        if ($this->pg_module->is_module_active('users')) {
            $this->load->model('Users_model');
            $user_types = $this->Users_model->getUserTypes();
            foreach ($user_types as $user_type) {
                $enabled = $this->pg_module->get_module_config('users', $user_type . '_search_enabled');
                if ($enabled) {
                    $operations[] = array('module' => 'users', 'type' => $user_type, 'name' => l($user_type, 'users'));
                }
            }
        }

        $this->set_api_content('operations', $operations);
    }

    /**
     * Get active languages
     *
     * @return void
     */
    public function get_languages()
    {
        $data['languages'] = array_values($this->pg_language->languages);
        $data['current_lang'] = $this->pg_language->current_lang_id;
                
        $this->set_api_content('data', $data);
    }

    /**
     * Get active currencies
     *
     * @return void
     */
    public function get_currencies()
    {
        $currencies = array();
        if ($this->pg_module->is_module_active('currencies')) {
            $this->load->model('Currencies_model');
            $currencies = $this->Currencies_model->get_currency_list();
        }
        $this->set_api_content('currencies', $currencies);
    }

    public function startUser()
    {

        $this->load->model('Start_model');
        $user = $this->Start_model->getUserDefaultOnSite();
        $data['user'] = $user;
        $this->set_api_content('data', (array) $data);
    }

    /**
     * Change user currency
     * 
     * @param integer $currency_id currency identifier
     * @return void
     */
    public function change_currency($currency_id)
    {
        $currency_id = intval($currency_id);
        $this->session->set_userdata("currency_id", $currency_id);

        if ($this->pg_module->is_module_active('users')) {
            if ($this->session->userdata("auth_type") == "user") {
                $user_id = $this->session->userdata("user_id");
                $save_data["id_currency"] = $currency_id;
                $this->Users_model->save_user($user_id, $save_data);
            }
        }

        $this->set_api_content('result', 'ok');
    }

    /**
     * Change user language 
     * 
     * @param integer $lang_id language identifier
     * @return void
     */
    public function change_language($lang_id)
    {
        $lang_id = intval($lang_id);

        $old_code = $this->pg_language->languages[$this->pg_language->current_lang_id]["code"];
        $this->pg_language->current_lang_id = $lang_id;
        $code = $this->pg_language->languages[$lang_id]["code"];
        $_SERVER["HTTP_REFERER"] = str_replace("/" . $old_code . "/", "/" . $code . "/", $_SERVER["HTTP_REFERER"]);
        $site_url = str_replace("/" . $code . "/", "", site_url());

        $this->session->set_userdata("lang_id", $lang_id);

        if ($this->pg_module->is_module_active('users')) {
            if ($this->session->userdata("auth_type") == "user") {
                $user_id = $this->session->userdata("user_id");
                $save_data["lang_id"] = $lang_id;
                $this->load->model('Users_model');
                $this->Users_model->save_user($user_id, $save_data);
            }
        }

        $this->set_api_content('result', 'ok');
    }

}
