<?php

namespace Pg\Modules\Subscriptions\Models;

/**
 * Subscriptions module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2016 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
if (!defined('SUBSCRIPTIONS_TABLE')) {
    define('SUBSCRIPTIONS_TABLE', DB_PREFIX . 'subscriptions');
}

/**
 * Subscriptions main model
 *
 * @package 	PG_RealEstate
 * @subpackage 	Subscriptions
 *
 * @category	models
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Subscriptions_model extends \Model
{

    /**
     * Link to CodeIgniter object
     *
     * @var object
     */
    protected $ci;

    /**
     * Subscription object properties in data source
     *
     * @var array
     */
    protected $attrs = array(
        'id',
        'id_template',
        'subscribe_type',
        'id_content_type',
        'scheduler',
        'subscriber_type',
    );

    /**
     * Class constructor
     *
     * @return Subscriptions_model
     */
    public function __construct()
    {
        parent::__construct();
        $this->ci = &get_instance();
    }

    /**
     * Return number of subsriptions objects
     *
     * @param array $params            criteria of sql query to data source
     * @param array $filter_object_ids filter subscriptions identifiers
     *
     * @return integer
     */
    public function get_subscriptions_count($params = array(), $filter_object_ids = null)
    {
        $this->ci->db->select('COUNT(*) AS cnt')->from(SUBSCRIPTIONS_TABLE);

        if (isset($params["where"]) && is_array($params["where"]) && count($params["where"])) {
            foreach ($params["where"] as $field => $value) {
                $this->ci->db->where($field, $value);
            }
        }

        if (isset($params["where_in"]) && is_array($params["where_in"]) && count($params["where_in"])) {
            foreach ($params["where_in"] as $field => $value) {
                $this->ci->db->where_in($field, $value);
            }
        }

        if (isset($params["where_sql"]) && is_array($params["where_sql"]) && count($params["where_sql"])) {
            foreach ($params["where_sql"] as $value) {
                $this->ci->db->where($value);
            }
        }

        if (isset($filter_object_ids) && is_array($filter_object_ids) && count($filter_object_ids)) {
            $this->ci->db->where_in("id", $filter_object_ids);
        }

        $results = $this->ci->db->get()->result_array();
        if (!empty($results) && is_array($results)) {
            return intval($results[0]["cnt"]);
        }

        return 0;
    }

    /**
     * Return subscriptions objects as array
     *
     * @param integer $page              page of results
     * @param integer $items_on_page     items per page
     * @param array   $order_by          sorting data
     * @param array   $params            criteria of query to data source
     * @param integer $filter_object_ids filter subscriptions idnetifiers
     *
     * @return array
     */
    public function get_subscriptions_list($page = null, $items_on_page = null, $order_by = null, $params = array(), $filter_object_ids = null)
    {
        $this->ci->db->select(implode(", ", $this->attrs))->from(SUBSCRIPTIONS_TABLE);

        if (isset($params["where"]) && is_array($params["where"]) && count($params["where"])) {
            foreach ($params["where"] as $field => $value) {
                $this->ci->db->where($field, $value);
            }
        }

        if (isset($params["where_in"]) && is_array($params["where_in"]) && count($params["where_in"])) {
            foreach ($params["where_in"] as $field => $value) {
                $this->ci->db->where_in($field, $value);
            }
        }

        if (isset($params["where_sql"]) && is_array($params["where_sql"]) && count($params["where_sql"])) {
            foreach ($params["where_sql"] as $value) {
                $this->ci->db->where($value);
            }
        }

        if (isset($filter_object_ids) && is_array($filter_object_ids) && count($filter_object_ids)) {
            $this->ci->db->where_in("id", $filter_object_ids);
        }

        if (is_array($order_by) && count($order_by) > 0) {
            foreach ($order_by as $field => $dir) {
                if (in_array($field, $this->attrs)) {
                    $this->ci->db->order_by($field . " " . $dir);
                }
            }
        }

        if (!is_null($page)) {
            $page = intval($page) ? intval($page) : 1;
            $this->ci->db->limit($items_on_page, $items_on_page * ($page - 1));
        }

        $data = array();
        $results = $this->ci->db->get()->result_array();
        if (!empty($results) && is_array($results)) {
            foreach ($results as $r) {
                $r['scheduler'] = $this->get_scheduler_format($r['scheduler']);
                $data[] = $this->format_subscription($r, true);
            }
        }

        return $data;
    }

    /**
     * Format subscription object
     *
     * @param array   $data      subscription data
     * @param boolean $get_langs extract data by language
     *
     * @return array
     */
    public function format_subscription($data, $get_langs = false)
    {
        $data["name_i"] = "subscription_" . $data["id"];
        $data["name"] = ($get_langs) ? (l($data["name_i"], 'subscriptions')) : "";
        if ($data['scheduler']['type'] == 1) {
            $data["scheduler_format"] = l('manual', 'subscriptions');
        }
        if ($data['scheduler']['type'] == 2) {
            $data["scheduler_format"] = l('in_time', 'subscriptions') . ' ' . $data['scheduler']['date'] . ' ' . $data['scheduler']['hours'] . ':' . $data['scheduler']['minutes'];
        }
        if ($data['scheduler']['type'] == 3) {
            $data["scheduler_format"] = l('every_time', 'subscriptions') . ' ' . l($data['scheduler']['period'],
                                                                                   'subscriptions') . ' ' . l('since',
                                                                                                              'subscriptions') . ' ' . $data['scheduler']['date'] . ' ' . $data['scheduler']['hours'] . ':' . $data['scheduler']['minutes'];
        }

        return $data;
    }

    /**
     * Return subscription object by GUID
     *
     * @param string $gid subscription guid
     *
     * @return array
     */
    public function get_subscription_by_gid($gid)
    {
        $data = array();
        $result = $this->ci->db->select(implode(", ", $this->attrs))
                ->from(SUBSCRIPTIONS_TABLE)
                ->where("gid", $gid)
                ->get()->result_array();
        if (!empty($result)) {
            $data = $result[0];
        }
        $data['scheduler'] = $this->get_scheduler_format($data['scheduler']);

        return $data;
    }

    /**
     * Format sheduler string
     *
     * @param string $sheduler_str sheduler string
     *
     * @return array
     */
    public function get_scheduler_format($sheduler_str)
    {
        $return = unserialize($sheduler_str);

        return $return;
    }

    /**
     * Return subscription object by identifier
     *
     * @param integer $id subscription identifier
     *
     * @return array
     */
    public function get_subscription_by_id($id)
    {
        $data = array();
        $result = $this->ci->db->select(implode(", ", $this->attrs))->from(SUBSCRIPTIONS_TABLE)->where("id", $id)->get()->result_array();
        if (!empty($result)) {
            $data = $result[0];
            $data['scheduler'] = $this->get_scheduler_format($data['scheduler']);
        }

        return $data;
    }

    /**
     * Validate subscription object for storing to data source
     *
     * @param integer $id    subscription identifier
     * @param array   $data  subscription data
     * @param array   $langs subscription data related to languages
     *
     * @return array
     */
    public function validate_subscription($id, $data, $langs = null)
    {
        $return = array("errors" => array(), "data" => array(), 'langs' => array());
        if (isset($data['scheduler_type'])) {
            $sheduler_array = array();
            $sheduler_array['type'] = intval($data['scheduler_type']);
            $sheduler_array['date_for_cron'] = 0;
            if (intval($data['scheduler_type']) == 2 || intval($data['scheduler_type']) == 3) {
                $sheduler_array['date'] = strval($data['scheduler_date']);
                $sheduler_array['hours'] = intval($data['scheduler_hours']);
                $sheduler_array['minutes'] = intval($data['scheduler_minutes']);
                $sheduler_array['date_for_cron'] = strtotime($sheduler_array['date'] . ' ' . $sheduler_array['hours'] . ':' . $sheduler_array['minutes']);
            }
            if (intval($data['scheduler_type']) == 3) {
                $sheduler_array['period'] = strval($data['scheduler_period']);
            }
            $return["data"]["scheduler"] = serialize($sheduler_array);
        }
        if (isset($data["id_template"])) {
            $return["data"]["id_template"] = intval($data["id_template"]);
        }

        if (isset($data["subscribe_type"])) {
            $return["data"]["subscribe_type"] = strip_tags($data["subscribe_type"]);
        }

        if (isset($data["id_content_type"])) {
            $return["data"]["id_content_type"] = intval($data["id_content_type"]);
        }

        if (isset($data["subscriber_type"])) {
            $return["data"]["subscriber_type"] = strval($data["subscriber_type"]);
            if (!empty($return["data"]["subscriber_type"])) {
                $this->ci->load->model('Users_model');
                $user_types = $this->ci->Users_model->getUserTypes();
                if (!in_array($return["data"]["subscriber_type"], $user_types)) {
                    
                }
            }
        }

        if (!empty($langs)) {
            $default_lang_id = $this->ci->pg_language->current_lang_id;
            if (!isset($langs[$default_lang_id])) {
                $return['errors'][] = l('error_empty_subscription_name', "subscriptions");
            } else {
                $return["langs"][$default_lang_id] = trim(strip_tags($langs[$default_lang_id]));
                if (empty($return["langs"][$default_lang_id])) {
                    $return['errors'][] = l('error_empty_subscription_name', "subscriptions");
                } else {
                    foreach ($this->ci->pg_language->languages as $lid => $lang_data) {
                        if ($lid == $default_lang_id) {
                            continue;
                        }
                        if (!isset($langs[$lid])) {
                            $return["langs"][$lid] = $return["langs"][$default_lang_id];
                        } else {
                            $return["langs"][$lid] = trim(strip_tags($langs[$lid]));
                            if (empty($return["langs"][$lid])) {
                                $return["langs"][$lid] = $return["langs"][$default_lang_id];
                            }
                        }
                    }
                }
            }
        }

        return $return;
    }

    /**
     * Save subscription object to data source
     *
     * @param integer $id    subscription identifier
     * @param array   $data  subscription data
     * @param array   $langs data related to languages
     *
     * @return integer
     */
    public function save_subscription($id, $data, $langs = null)
    {
        if (is_null($id)) {
            $this->ci->db->insert(SUBSCRIPTIONS_TABLE, $data);
            $id = $this->ci->db->insert_id();
        } else {
            $this->ci->db->where('id', $id);
            $this->ci->db->update(SUBSCRIPTIONS_TABLE, $data);
        }

        if (!empty($id) && !empty($langs)) {
            $languages = $this->ci->pg_language->languages;
            $lang_ids = array_keys($languages);
            $this->ci->pg_language->pages->set_string_langs('subscriptions', "subscription_" . $id, $langs, $lang_ids);
        }

        return $id;
    }

    /**
     * Remove subscription object by identifier
     *
     * @param integer $id subscription identifier
     *
     * @return void
     */
    public function delete_subscription($id)
    {
        $this->ci->db->where("id", $id);
        $this->ci->db->delete(SUBSCRIPTIONS_TABLE);
        return;
    }

    /**
     * Remove subscription object by guid
     *
     * @param string $gid subscription guid
     *
     * @return void
     */
    public function delete_subscription_by_gid($gid)
    {
        $this->ci->db->where("gid", $gid);
        $this->ci->db->delete(SUBSCRIPTIONS_TABLE);
        return;
    }

    /**
     * Send subscription
     *
     * @param integer $id    subscription identifier
     * @param integer $page  mailing list page
     * @param integer $limit send subscription limit
     *
     * @return array
     */
    public function send_subscription($id, $page = null, $limit = 1000)
    {
        $this->ci->load->model('subscriptions/models/Subscriptions_users_model');
        $this->ci->load->model('subscriptions/models/Subscriptions_types_model');
        $this->ci->load->model('notifications/models/Templates_model');
        $this->ci->load->model('notifications/models/Sender_model');
        $count = $this->ci->Subscriptions_users_model->get_subscription_users_count($id);
        $data = array('sended' => 0, 'have_to_send' => 0);

        if ($count) {
            $users = $this->ci->Subscriptions_users_model->get_users_by_id_subscription($id, $page, $limit);
            $subscription_object = $this->get_subscription_by_id($id);
            $template = $this->ci->Templates_model->get_template_by_id($subscription_object['id_template']);

            //generate template
            $installed_langs = $this->ci->pg_language->get_langs('is_default DESC');
            $content = array();
            foreach ($installed_langs as $lang_id => $lang_info) {
                $vars = $this->ci->Subscriptions_types_model->get_subscriptions_type_content(
                        $subscription_object['id_content_type'], $lang_id
                );
                $content[$lang_id] = $this->ci->Templates_model->compile_template($template['gid'], $vars, $lang_id);
            }

            $data['sended'] = 0;

            foreach ($users as $key => $user) {
                if (empty($content[$user['lang_id']]['content'])) {
                    continue;
                }
                $this->ci->Sender_model->push($user['email'], $content[$user['lang_id']]['subject'],
                                              $content[$user['lang_id']]['content']);
                ++$data['sended'];
            }

            if ($count > $page * $limit) {
                $data['have_to_send'] = 1;
            } else {
                $data['have_to_send'] = 0;
            }
        }

        return $data;
    }

    /**
     * Send subscriptions by cron
     *
     * @return void
     */
    public function cron_send_subscriptions()
    {
        $subscriptions_list = $this->get_subscriptions_list();
        foreach ($subscriptions_list as $key => $value) {
            if ($value['scheduler']['type'] != 1 && $value['scheduler']['date_for_cron'] != 0 && $value['scheduler']['date_for_cron'] < time()) {
                $this->send_subscription($value['id']);
                if ($value['scheduler']['type'] == 2) {
                    $value['scheduler']['date_for_cron'] = 0;
                } elseif ($value['scheduler']['type'] == 3) {
                    $d = $value['scheduler']['date_for_cron'];
                    if ($value['scheduler']['period'] == 'day') {
                        $value['scheduler']['date_for_cron'] = mktime(
                                date("H", $d), date("i", $d), date("s", $d), date("n", $d), date("j", $d) + 1, date("Y", $d)
                        );
                    } elseif ($value['scheduler']['period'] == 'week') {
                        $value['scheduler']['date_for_cron'] = mktime(
                                date("H", $d), date("i", $d), date("s", $d), date("n", $d), date("j", $d) + 7, date("Y", $d)
                        );
                    } elseif ($value['scheduler']['period'] == 'month') {
                        $value['scheduler']['date_for_cron'] = mktime(
                                date("H", $d), date("i", $d), date("s", $d), date("n", $d) + 1, date("j", $d), date("Y", $d)
                        );
                    }
                }
                $data = array('scheduler' => serialize($value['scheduler']));
                $this->save_subscription($value['id'], $data);
            }
        }
    }

    /**
     * Import subscriptions data related to languages
     *
     * @param string $model         model name
     * @param array  $subscriptions subscriptions objects
     * @param array  $langs_file    data related to languages from file
     * @param array  $langs_ids     languages identifiers
     *
     * @return boolean
     */
    public function update_langs($model, $subscriptions, $langs_file, $langs_ids)
    {
        foreach ($subscriptions as $subscription) {
            $subscr = $this->get_subscription_by_gid($subscription['gid']);
            $this->ci->pg_language->pages->set_string_langs(
                    'subscriptions', 'subscription_' . $subscr['id'], $langs_file[$subscription['gid']], $langs_ids
            );
            $this->ci->pg_language->pages->set_string_langs(
                    $model, 'subscriptions_type_' . $subscription['gid'], $langs_file[$subscription['gid']], $langs_ids
            );
        }

        return true;
    }

    /**
     * Export subscriptions data related to languages
     *
     * @param array $subscriptions subscriptions objects
     * @param array $langs_ids     languages identifiers
     *
     * @return array
     */
    public function export_langs($subscriptions, $langs_ids = null)
    {
        foreach ($subscriptions as $subscription) {
            $subscr = $this->get_subscription_by_gid($subscription['gid']);
            $gids[$subscription['gid']] = 'subscription_' . $subscr['id'];
        }
        $langs = $this->ci->pg_language->export_langs('subscriptions', $gids, $langs_ids);

        return array_combine(array_keys($gids), $langs);
    }

}
