<?php

namespace Pg\Modules\Uploads\Models;

/**
 * Uploads module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
if (!defined('UPLOADS_TABLE')) {
    define('UPLOADS_TABLE', DB_PREFIX . 'uploads');
}
if (!defined('UPLOADS_THUMB_TABLE')) {
    define('UPLOADS_THUMB_TABLE', DB_PREFIX . 'uploads_thumb');
}
if (!defined('UPLOADS_WATERMARK_TABLE')) {
    define('UPLOADS_WATERMARK_TABLE', DB_PREFIX . 'uploads_watermark');
}

/**
 * Upload config model
 *
 * @package 	PG_RealEstate
 * @subpackage 	Uploads
 *
 * @category	models
 *
 * @copyright 	Copyright (c) 2000-2016 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Uploads_config_model extends \Model
{

    /**
     * Link to CodeIgniter object
     *
     * @var object
     */
    protected $ci;

    /**
     * All upload configuration object properties in data source
     *
     * @var array
     */
    public $fields_all = array(
        'id',
        'gid',
        'name',
        'min_height',
        'min_width',
        'max_height',
        'max_width',
        'max_size',
        'name_format',
        'file_formats',
        'default_img',
        'indicator_img',
        'date_add',
    );

    /**
     * Watermark object properties in data source
     *
     * @var array
     */
    public $watermark_fields_all = array(
        'id',
        'gid',
        'name',
        'wm_type',
        'img',
        'font_text',
        'font_color',
        'shadow_color',
        'shadow_distance',
        'font_face',
        'font_size',
        'position_hor',
        'position_ver',
        'alpha',
        'date_add',
    );

    /**
     * Thumbnail object properties in data source
     *
     * @var array
     */
    public $thumb_fields_all = array(
        'id',
        'config_id',
        'prefix',
        'width',
        'height',
        'effect',
        'watermark_id',
        'crop_param',
        'crop_color',
        'animation',
        'delay',
        'loops',
        'disposal',
        'transparent_color',
        'rotation_angle',
        'date_add',
    );

    /**
     * Watermark data by default
     *
     * @var array
     */
    public $default_watermark = array(
        'wm_type' => 'text',
        'font_color' => 'ffffff',
        'shadow_color' => '777777',
        'shadow_distance' => '3',
        'font_face' => 'arial',
        'font_size' => '16',
        'position_hor' => 'left',
        'position_ver' => 'top',
        'alpha' => 100,
        'font_text' => 'Test string',
    );

    /**
     * Limit text watermark options
     *
     * @var array
     */
    public $wm_text_limits = array(
        'min_font_size' => 9,
        'max_font_size' => 90,
        'min_shadow_distance' => 0,
        'max_shadow_distance' => 20,
    );

    /**
     * Available file formats
     *
     * @var array
     */
    public $file_formats = array("jpg", "jpeg", 'gif', 'png');

    /**
     * Path to watermark file
     *
     * @var string
     */
    public $watermark_path = "";

    /**
     * Url to watermark file
     *
     * @var string
     */
    public $watermark_url = "";

    /**
     * Path to watermark by default
     *
     * @var string
     */
    public $default_path = "";

    /**
     * Url to watermark by default
     *
     * @var string
     */
    public $default_url = "";

    /**
     * Image watermark file name for testing
     *
     * @var string
     */
    public $watermark_test_image = "watermark_test.jpg";

    /**
     * Folder to font files for text watermark
     *
     * @var string
     */
    public $fonts_folder = '';

    /**
     * Constructor
     *
     * @return Uploads
     */
    public function __construct()
    {
        parent::__construct();
        $this->ci = &get_instance();

        $this->watermark_path = SITE_PHYSICAL_PATH . UPLOAD_DIR . "watermark/";
        $this->watermark_url = SITE_VIRTUAL_PATH . UPLOAD_DIR . "watermark/";
        $this->default_path = SITE_PHYSICAL_PATH . UPLOAD_DIR . "default/";
        $this->default_url = SITE_VIRTUAL_PATH . UPLOAD_DIR . "default/";
        $this->fonts_folder = BASEPATH . "fonts/";
    }

    /**
     * Return configuration object by identifier
     *
     * @param integer $config_id configuration identifier
     *
     * @return array
     */
    public function get_config_by_id($config_id)
    {
        $data = array();
        $result = $this->ci->db->select(implode(", ", $this->fields_all))->from(UPLOADS_TABLE)->where("id", $config_id)->get()->result_array();
        if (!empty($result)) {
            $data = $result[0];
        }

        return $data;
    }

    /**
     * Return configuration object by guid
     *
     * @param string $config_gid configuration guid
     *
     * @return array
     */
    public function get_config_by_gid($config_gid)
    {
        $data = array();
        $result = $this->ci->db->select(implode(", ", $this->fields_all))->from(UPLOADS_TABLE)->where("gid", $config_gid)->get()->result_array();
        if (!empty($result)) {
            $data = $result[0];
        }

        return $data;
    }

    /**
     * Return configurations objects as array
     *
     * @return void
     */
    public function get_config_list()
    {
        $data = array();
        $this->ci->db->select(implode(", ", $this->fields_all))->from(UPLOADS_TABLE)->order_by('gid ASC');
        $results = $this->ci->db->get()->result_array();

        if (!empty($results) && is_array($results)) {
            foreach ($results as $r) {
                $data[] = $this->format_config($r);
            }
        }

        return $data;
    }

    /**
     * Return number of configurations objects
     *
     * @return void
     */
    public function get_config_count()
    {
        $this->ci->db->select(implode(", ", $this->fields_all))->from(UPLOADS_TABLE);
        $result = $this->ci->db->get()->result();
        if (!empty($result)) {
            return intval($result[0]->cnt);
        } else {
            return 0;
        }
    }

    /**
     * Save configuration object to data source
     *
     * @param integer $config_id configuration identifier
     * @param array   $data      configuration data
     *
     * @return integer
     */
    public function save_config($config_id, $data)
    {
        if (is_null($config_id)) {
            $data["date_add"] = date("Y-m-d H:i:s");
            $this->ci->db->insert(UPLOADS_TABLE, $data);
            $config_id = $this->ci->db->insert_id();
        } else {
            $this->ci->db->where('id', $config_id);
            $this->ci->db->update(UPLOADS_TABLE, $data);
        }

        if (!empty($data["default_img"])) {
            $this->ci->load->model('Uploads_model');
            $thumbs_data = $this->get_config_thumbs($config_id);
            if (!empty($thumbs_data)) {
                $this->ci->Uploads_model->create_default($data["default_img"], $this->default_path, $thumbs_data);
            }
        }

        if (!empty($data["indicator_img"])) {
            $this->ci->load->model('Uploads_model');
            $thumbs_data = $this->get_config_thumbs($config_id);
            if (!empty($thumbs_data)) {
                $this->ci->Uploads_model->create_indicator($data["indicator_img"], $this->default_path, $thumbs_data);
            }
        }

        return $config_id;
    }

    /**
     * Remove configuration object by idnetifier
     *
     * @param integer $config_id configuration identifier
     *
     * @return void
     */
    public function delete_config($config_id)
    {
        $data = $this->get_config_by_id($config_id);
        if (empty($data)) {
            return false;
        }

        $data = $this->format_config($data);
        if (!empty($data["default_img_path"]) && file_exists($data["default_img_path"])) {
            unlink($data["default_img_path"]);
        }
        if (!empty($data["indicator_img_path"]) && file_exists($data["indicator_img_path"])) {
            unlink($data["indicator_img_path"]);
        }
        $this->ci->db->where('id', $config_id);
        $this->ci->db->delete(UPLOADS_TABLE);

        if (!empty($data["default_img"]) || !empty($data["indicator_img"])) {
            $thumbs = $this->get_config_thumbs($config_id);
            if (!empty($thumbs)) {
                foreach ($thumbs as $thumb) {
                    if (!empty($data["default_img"])) {
                        if (file_exists($this->default_path . $thumb["prefix"] . '-' . $data["default_img"])) {
                            unlink($this->default_path . $thumb["prefix"] . '-' . $data["default_img"]);
                        }
                        if (file_exists($this->default_path . $thumb["prefix"] . '-animation-' . $data["default_img"])) {
                            unlink($this->default_path . $thumb["prefix"] . '-animation-' . $data["default_img"]);
                        }
                    }
                    if (!empty($data["indicator_img"])) {
                        if (file_exists($this->default_path . $thumb["prefix"] . '-' . $data["indicator_img"])) {
                            unlink($this->default_path . $thumb["prefix"] . '-' . $data["indicator_img"]);
                        }
                    }
                    $this->delete_thumb($thumb["id"]);
                }
            }
        }

        return;
    }

    /**
     * Validate configuration object for saving to data source
     *
     * @param integer $config_id configuration idnetifier
     * @param array   $data      configuration data
     *
     * @return array
     */
    public function validate_config($config_id = null, $data = array())
    {
        $return = array("errors" => array(), "data" => array());

        if (isset($data["name"])) {
            $return["data"]["name"] = strip_tags($data["name"]);
            if (empty($return["data"]["name"])) {
                $return["errors"][] = l('error_title_invalid', 'uploads');
            }
        }

        if (isset($data["gid"])) {
            $return["data"]["gid"] = strip_tags($data["gid"]);
            $return["data"]["gid"] = preg_replace("/[\n\t\s]{1,}/", "-", trim($return["data"]["gid"]));
            if (empty($return["data"]["gid"])) {
                $return["errors"][] = l('error_gid_invalid', 'uploads');
            }
        } elseif (!$config_id) {
            $return["errors"][] = l('error_gid_invalid', 'uploads');
        }

        if (isset($data["min_width"])) {
            $return["data"]["min_width"] = intval($data["min_width"]);
        }

        if (isset($data["min_height"])) {
            $return["data"]["min_height"] = intval($data["min_height"]);
        }

        if (isset($data["max_width"])) {
            $return["data"]["max_width"] = intval($data["max_width"]);
        }

        if (isset($data["max_height"])) {
            $return["data"]["max_height"] = intval($data["max_height"]);
        }

        if (isset($data["max_size"])) {
            $return["data"]["max_size"] = intval($data["max_size"]);
        }

        if (isset($data["name_format"])) {
            $return["data"]["name_format"] = $data["name_format"];
        }

        if (isset($data["file_formats"])) {
            if (!is_array($data["file_formats"]) || empty($data["file_formats"])) {
                $return["errors"][] = l('error_empty_file_formats', 'uploads');
            } else {
                $return["data"]["file_formats"] = serialize($data["file_formats"]);
            }
        }

        if (isset($data["default_img"]) && is_array($data["default_img"]) && is_uploaded_file($data["default_img"]["tmp_name"])) {
            $this->ci->load->helper('upload');
            $config = array('allowed_types' => 'gif|jpg|jpeg|png');
            $img_return = upload_file('default_img', $this->default_path, $config);
            if (!empty($img_return["error"])) {
                foreach ($img_return["error"] as $imgError) {
                    $return["errors"][] = $imgError;
                }
                $img_name = "";
            } else {
                $old_name = $img_return["data"]["file_name"];
                $new_name = "default_" . $return["data"]["gid"] . $img_return["data"]["file_ext"];
                $rename = rename_file($img_return["data"]["file_path"] . $old_name,
                                      $img_return["data"]["file_path"] . $new_name);
                $img_name = ($rename) ? $new_name : $old_name;
            }
            $return["data"]["default_img"] = $img_name;
        }

        if (isset($data["indicator_img"]) && is_array($data["indicator_img"]) && is_uploaded_file($data["indicator_img"]["tmp_name"])) {
            $this->ci->load->helper('upload');
            $config = array('allowed_types' => 'gif');
            $img_return = upload_file('indicator_img', $this->default_path, $config);
            if (!empty($img_return["error"])) {
                foreach ($img_return["error"] as $imgError) {
                    $return["errors"][] = $imgError;
                }
                $img_name = "";
            } else {
                $old_name = $img_return["data"]["file_name"];
                $new_name = "indicator_" . $return["data"]["gid"] . $img_return["data"]["file_ext"];
                $rename = rename_file($img_return["data"]["file_path"] . $old_name,
                                      $img_return["data"]["file_path"] . $new_name);
                $img_name = ($rename) ? $new_name : $old_name;
            }
            $return["data"]["indicator_img"] = $img_name;
        }

        return $return;
    }

    /**
     * Format configuration object
     *
     * @param array $data configuration data
     *
     * @return array
     */
    public function format_config($data)
    {
        if (!empty($data["default_img"])) {
            $data["default_url"] = $this->default_url;
            $data["default_path"] = $this->default_path;
            $data["default_img_path"] = $this->default_path . $data["default_img"];
            $data["default_img_url"] = $this->default_url . $data["default_img"];
        }
        if (!empty($data["indicator_img"])) {
            $data["indicator_url"] = $this->default_url;
            $data["indicator_path"] = $this->default_path;
            $data["indicator_img_path"] = $this->default_path . $data["indicator_img"];
            $data["indicator_img_url"] = $this->default_url . $data["indicator_img"];
        }
        if (!empty($data["file_formats"])) {
            $data["file_formats"] = unserialize($data["file_formats"]);
            if (!empty($data["file_formats"])) {
                foreach ($data["file_formats"] as $format) {
                    $data["enable_formats"][$format] = 1;
                }
            }
            $data["file_formats_str"] = implode("|", $data["file_formats"]);
        } else {
            $data["file_formats_str"] = "";
        }

        return $data;
    }

    /**
     * Return configuration thumbnail object data by identifier
     *
     * @param integer $thumb_id thumbnail identifier
     *
     * @return array
     */
    public function get_thumb_by_id($thumb_id)
    {
        $data = array();
        $result = $this->ci->db->select(implode(", ", $this->thumb_fields_all))->from(UPLOADS_THUMB_TABLE)->where("id",
                                                                                                                  $thumb_id)->get()->result_array();
        if (!empty($result)) {
            $data = $result[0];
        }

        return $data;
    }

    /**
     * Return configuration thumbnails objects as array
     *
     * @param integer $config_id configuration identifier
     *
     * @return array
     */
    public function get_config_thumbs($config_id)
    {
        $data = array();
        $this->ci->db->select(implode(", ", $this->thumb_fields_all))->from(UPLOADS_THUMB_TABLE)->where("config_id",
                                                                                                        $config_id)->order_by('id ASC');
        $results = $this->ci->db->get()->result_array();

        if (!empty($results) && is_array($results)) {
            foreach ($results as $r) {
                $data[] = $this->format_thumb($r);
            }
        }

        return $data;
    }

    /**
     * Save configuration thumbnail object to data source
     *
     * @param integer $thumb_id thumbnail identifier
     * @param array   $data     thumbnail data
     *
     * @return integer
     */
    public function save_thumb($thumb_id, $data)
    {
        if (is_null($thumb_id)) {
            $data["date_add"] = date("Y-m-d H:i:s");
            $this->ci->db->insert(UPLOADS_THUMB_TABLE, $data);
            $thumb_id = $this->ci->db->insert_id();
        } else {
            $this->ci->db->where('id', $thumb_id);
            $this->ci->db->update(UPLOADS_THUMB_TABLE, $data);
        }

        $this->ci->load->model('Uploads_model');
        $thumb_data = $this->format_thumb($this->get_thumb_by_id($thumb_id));
        $config_data = $this->get_config_by_id($thumb_data['config_id']);

        if ($config_data["default_img"]) {
            $thumb_file = $this->default_path . $thumb_data["prefix"] . '-' . $config_data["default_img"];
            if (file_exists($thumb_file)) {
                unlink($thumb_file);
            }
            copy($this->default_path . $config_data["default_img"], $thumb_file);
            $this->ci->Uploads_model->action($thumb_file, $thumb_data);
        }

        if ($config_data["indicator_img"]) {
            $fname = basename($config_data["indicator_img"], '.gif');

            $this->ci->load->library('PG_GIFEncoder');
            $this->ci->load->library('PG_GIFDecoder');

            $parsed = $this->ci->pg_gifdecoder->parse($this->default_path . $config_data["indicator_img"]);

            $cframes = count($parsed['frames']);
            for ($i = 0; $i < $cframes; ++$i) {
                fwrite(fopen($this->default_path . $fname . '-frame' . $i . '.gif', "wb"), $parsed['frames'][$i]);
                unset($parsed['frames'][$i]);
            }

            $thumb_file = $this->default_path . $thumb_data["prefix"] . '-' . $config_data["indicator_img"];
            if (file_exists($thumb_file)) {
                unlink($thumb_file);
            }

            $sources = array();

            for ($i = 0; $i < $cframes; ++$i) {
                $source = $this->default_path . $thumb_data["prefix"] . '-' . $fname . '-frame' . $i . '.gif';
                copy($this->default_path . $fname . '-frame' . $i . '.gif', $source);
                $this->ci->Uploads_model->crop($source, $thumb_data["width"], $thumb_data["height"], false);
                $sources[] = $source;
            }

            $params["GIF_src"] = $sources;
            $params["GIF_dly"] = $parsed['dly'];
            $params["GIF_lop"] = $parsed['lop'];
            $params["GIF_dis"] = $parsed['dis'];
            $params["GIF_red"] = $parsed['red'];
            $params["GIF_grn"] = $parsed['grn'];
            $params["GIF_blu"] = $parsed['bln'];
            $params["GIF_mod"] = "url";

            $content = $this->ci->pg_gifencoder->create($params);
            fwrite(fopen($thumb_file, "wb"), $content);

            foreach ($sources as $source) {
                unlink($source);
            }

            for ($i = 0; $i < $cframes; ++$i) {
                unlink($this->default_path . $fname . '-frame' . $i . '.gif');
            }

            unset($parsed);
        }

        return $thumb_id;
    }

    /**
     * Remove thumbnail object by idnetifier
     *
     * @param integer $thumb_id thumbnail idnetifier
     *
     * @return void
     */
    public function delete_thumb($thumb_id)
    {
        $thumb_data = $this->get_thumb_by_id($thumb_id);
        if (empty($thumb_data)) {
            return false;
        }

        $config_data = $this->get_config_by_id($thumb_data["config_id"]);
        $config_data = $this->format_config($config_data);

        if (!empty($config_data["default_img"]) && file_exists($this->default_path . $thumb_data["prefix"] . '-' . $config_data["default_img"])) {
            unlink($this->default_path . $thumb_data["prefix"] . '-' . $config_data["default_img"]);
        }

        if (!empty($config_data["indicator_img"]) && file_exists($this->default_path . $thumb_data["prefix"] . '-' . $config_data["indicator_img"])) {
            unlink($this->default_path . $thumb_data["prefix"] . '-' . $config_data["indicator_img"]);
        }

        $this->ci->db->where('id', $thumb_id);
        $this->ci->db->delete(UPLOADS_THUMB_TABLE);

        return;
    }

    /**
     * Validate thumbnail object for saving to data source
     *
     * @param integer $thumb_id thumbnail identifier
     * @param array   $data     thumbnail data
     *
     * @return array
     */
    public function validate_thumb($thumb_id = null, $data = array())
    {
        $return = array("errors" => array(), "data" => array());

        if (isset($data["prefix"])) {
            $return["data"]["prefix"] = strip_tags($data["prefix"]);
            if (empty($return["data"]["prefix"])) {
                $return["errors"][] = l('error_prefix_invalid', 'uploads');
            }
        }

        if (isset($data["width"])) {
            $return["data"]["width"] = intval($data["width"]);
            if (empty($return["data"]["width"])) {
                $return["errors"][] = l('error_width_invalid', 'uploads');
            }
        }

        if (isset($data["height"])) {
            $return["data"]["height"] = intval($data["height"]);
            if (empty($return["data"]["height"])) {
                $return["errors"][] = l('error_height_invalid', 'uploads');
            }
        }

        if (isset($data["watermark_id"])) {
            $return["data"]["watermark_id"] = intval($data["watermark_id"]);
        }

        if (isset($data["config_id"])) {
            $return["data"]["config_id"] = intval($data["config_id"]);
        }

        if (isset($data["effect"])) {
            $return["data"]["effect"] = $data["effect"];
        }

        if (isset($data["crop_param"])) {
            $return["data"]["crop_param"] = $data["crop_param"];
        }

        if (isset($data["crop_color"])) {
            $return["data"]["crop_color"] = $data["crop_color"];
            if (empty($return["data"]["height"])) {
                $return["data"]["crop_color"] = "ffffff";
            }
        }

        if (isset($data["animation"])) {
            $return["data"]["animation"] = $data["animation"] ? 1 : 0;
        }

        if (isset($data["delay"])) {
            $return["data"]["delay"] = intval($data["delay"]);
        }

        if (isset($data["loops"])) {
            $return["data"]["loops"] = intval($data["loops"]);
        }

        if (isset($data["disposal"])) {
            $return["data"]["disposal"] = intval($data["disposal"]);
        }

        if (isset($data["transparent_color"])) {
            $return["data"]["transparent_color"] = substr($data["transparent_color"], 0, 6);
        }

        return $return;
    }

    /**
     * Format thumbnail object
     *
     * @param array $data thumbnail data
     *
     * @return array
     */
    public function format_thumb($data)
    {
        if (empty($data["crop_color"])) {
            $data["crop_color"] = "ffffff";
        }
        if (empty($data["crop_param"])) {
            $data["crop_param"] = 'resize';
        }
        if (empty($data["transparent_color"])) {
            $data["transparent_color"] = 'ffffff';
        }

        return $data;
    }

    /**
     * Return watermark object by identifier
     *
     * @param integer $wm_id watermark identifier
     *
     * @return array
     */
    public function get_watermark_by_id($wm_id)
    {
        $data = array();
        $result = $this->ci->db->select(implode(", ", $this->watermark_fields_all))->from(UPLOADS_WATERMARK_TABLE)->where("id",
                                                                                                                          $wm_id)->get()->result_array();
        if (!empty($result)) {
            $data = $result[0];
        }

        return $data;
    }

    /**
     * Return watermark object by guid
     *
     * @param string $wm_gid watermark guid
     *
     * @return array
     */
    public function get_watermark_by_gid($wm_gid)
    {
        $data = array();
        $result = $this->ci->db->select(implode(", ", $this->watermark_fields_all))->from(UPLOADS_WATERMARK_TABLE)->where("gid",
                                                                                                                          $wm_gid)->get()->result_array();
        if (!empty($result)) {
            $data = $result[0];
        }

        return $data;
    }

    /**
     * Format watermark object
     *
     * @param array $data watermark data
     *
     * @return array
     */
    public function format_watermark($data)
    {
        if (empty($data["wm_type"])) {
            $data["wm_type"] = 'text';
        }
        if ($data["wm_type"] == 'img') {
            if (!empty($data["img"])) {
                $data["img_path"] = $this->watermark_path . $data["img"];
                $data["img_url"] = $this->watermark_url . $data["img"];
            } else {
                $data["wm_type"] == 'text';
            }
        }

        if ($data["wm_type"] == 'text') {
            if (empty($data["font_color"])) {
                $data["font_color"] = $this->default_watermark['font_color'];
            }

            if (empty($data["shadow_color"])) {
                $data["shadow_color"] = $this->default_watermark['shadow_color'];
            }

            if (empty($data["shadow_distance"])) {
                $data["shadow_distance"] = $this->default_watermark['shadow_distance'];
            }

            if (empty($data["font_face"])) {
                $data["font_face"] = $this->default_watermark['font_face'];
            }

            if (empty($data["font_size"])) {
                $data["font_size"] = $this->default_watermark['font_size'];
            }

            if (empty($data["font_text"])) {
                $data["font_text"] = $this->default_watermark['font_text'];
            }
        }

        if (empty($data["alpha"])) {
            $data["alpha"] = $this->default_watermark['alpha'];
        }

        return $data;
    }

    /**
     * Return watermarks objects as array
     *
     * @return array
     */
    public function get_watermark_list()
    {
        $data = array();
        $this->ci->db->select(implode(", ", $this->watermark_fields_all))->from(UPLOADS_WATERMARK_TABLE)->order_by('gid ASC');
        $results = $this->ci->db->get()->result_array();

        if (!empty($results) && is_array($results)) {
            foreach ($results as $r) {
                $data[] = $this->format_watermark($r);
            }
        }

        return $data;
    }

    /**
     * Return number of watermarks objects
     *
     * @return integer
     */
    public function get_watermark_count()
    {
        $this->ci->db->select(implode(", ", $this->watermark_fields_all))->from(UPLOADS_WATERMARK_TABLE);
        $result = $this->ci->db->get()->result();
        if (!empty($result)) {
            return intval($result[0]->cnt);
        } else {
            return 0;
        }
    }

    /**
     * Save watermarks object to data source
     *
     * @param integer $wm_id watermark identifier
     * @param array   $data  watermark data
     *
     * @return integer
     */
    public function save_watermark($wm_id, $data)
    {
        if (is_null($wm_id)) {
            $data["date_add"] = date("Y-m-d H:i:s");
            $this->ci->db->insert(UPLOADS_WATERMARK_TABLE, $data);
            $wm_id = $this->ci->db->insert_id();
        } else {
            $this->ci->db->where('id', $wm_id);
            $this->ci->db->update(UPLOADS_WATERMARK_TABLE, $data);
        }

        return $wm_id;
    }

    /**
     * Remove watermark object by identifier
     *
     * @param integer $wm_id watermark identifier
     *
     * @return void
     */
    public function delete_watermark($wm_id)
    {
        $data = $this->get_watermark_by_id($wm_id);
        if (empty($data)) {
            return false;
        }

        $data = $this->format_watermark($data);
        if (!empty($data["img_path"])) {
            unlink($data["img_path"]);
        }
        $this->ci->db->where('id', $wm_id);
        $this->ci->db->delete(UPLOADS_WATERMARK_TABLE);

        $this->ci->db->where('watermark_id', $wm_id);
        $this->ci->db->update(UPLOADS_THUMB_TABLE, array('watermark_id' => 0));

        return;
    }

    /**
     * Validate watermark object for saving to data source
     *
     * @param integer $wm_id watermark identifier
     * @param array   $data  watermark data
     *
     * @return array
     */
    public function validate_watermark($wm_id = null, $data = array())
    {
        $return = array("errors" => array(), "data" => array());

        if (isset($data["name"])) {
            $return["data"]["name"] = strip_tags($data["name"]);
            if (empty($return["data"]["name"])) {
                $return["errors"][] = l('error_title_invalid', 'uploads');
            }
        }

        if (isset($data["gid"])) {
            $return["data"]["gid"] = strip_tags($data["gid"]);
            $return["data"]["gid"] = preg_replace("/[\n\t\s]{1,}/", "-", trim($return["data"]["gid"]));
            if (empty($return["data"]["gid"])) {
                $return["errors"][] = l('error_gid_invalid', 'uploads');
            }
        }

        if (isset($data["position_ver"])) {
            $return["data"]["position_ver"] = $data["position_ver"];
        }

        if (isset($data["position_hor"])) {
            $return["data"]["position_hor"] = $data["position_hor"];
        }

        if (isset($data["alpha"])) {
            $return["data"]["alpha"] = intval($data["alpha"]);
        }

        if (isset($data["wm_type"])) {
            $return["data"]["wm_type"] = $data["wm_type"];
        }

        if (isset($data["img"]) && is_array($data["img"]) && is_uploaded_file($data["img"]["tmp_name"])) {
            $this->ci->load->helper('upload');
            $config = array('allowed_types' => 'gif|jpg|jpeg|png');
            $img_return = upload_file('img', $this->watermark_path, $config);
            if (!empty($img_return["error"])) {
                foreach ($img_return["error"] as $imgError) {
                    $return["errors"][] = $imgError;
                }
                $img_name = "";
            } else {
                $old_name = $img_return["data"]["file_name"];
                $new_name = "wm_" . $return["data"]["gid"] . $img_return["data"]["file_ext"];
                $rename = rename_file($img_return["data"]["file_path"] . $old_name,
                                      $img_return["data"]["file_path"] . $new_name);
                $img_name = ($rename) ? $new_name : $old_name;
            }
            $return["data"]["img"] = $img_name;
        }

        if (isset($data["font_size"])) {
            $return["data"]["font_size"] = intval($data["font_size"]);
        }

        if (isset($data["shadow_distance"])) {
            $return["data"]["shadow_distance"] = intval($data["shadow_distance"]);
        }

        if (isset($data["font_color"])) {
            $return["data"]["font_color"] = $data["font_color"];
        }

        if (isset($data["shadow_color"])) {
            $return["data"]["shadow_color"] = $data["shadow_color"];
        }

        if (isset($data["font_face"])) {
            $return["data"]["font_face"] = $data["font_face"];
        }

        if (isset($data["font_text"])) {
            $return["data"]["font_text"] = strip_tags($data["font_text"]);
        }

        return $return;
    }

}
