<?php

/**
 * Users module
 *
 * @package     PG_RealEstate
 *
 * @copyright   Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author      Pilot Group Ltd <http://www.pilotgroup.net/>
 */
/**
 * Users management
 *
 * @package     PG_RealEstate
 * @subpackage  Users
 *
 * @category    helpers
 *
 * @copyright   Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author      Pilot Group Ltd <http://www.pilotgroup.net/>
 */

if (!function_exists("login_form")) {

    /**
     * Return login form block
     *
     * @return string
     */
    function login_form()
    {
        $ci = &get_instance();
        if ($ci->session->userdata("auth_type") == "user") {
            $ci->load->model("Users_model");
            $user_data = $ci->Users_model->get_user_by_id($ci->session->userdata("user_id"));
            $user_data = $ci->Users_model->format_user($user_data);
            $ci->view->assign("user_data", $user_data);
        }

        return $ci->view->fetch("helper_login_form", null, "users");
    }

}

if (!function_exists("users_lang_select")) {

    /**
     * Return lanuages selector
     *
     * @return string
     */
    function users_lang_select()
    {
        $ci = &get_instance();

        $languages = $ci->pg_language->languages;
        foreach ($languages as $lid => $lang_data) {
            if (!$lang_data["status"]) {
                unset($languages[$lid]);
            }
        }
        if (count($languages) <= 1) {
            return '';
        }
        $ci->view->assign("languages", $languages);

        $current_lang = $ci->pg_language->current_lang_id;
        $ci->view->assign("current_lang", $current_lang);

        return $ci->view->fetch("helper_lang_select", null, "users");
    }

}

if (!function_exists("auth_links")) {

    /**
     * Return sign in block
     *
     * @return string
     */
    function auth_links()
    {
        $ci = &get_instance();
        $ci->load->model("Users_model");

        if (!is_login_enabled()) {
            return "";
        }

        if ($ci->session->userdata("auth_type") == "user") {
            $user_id = $ci->session->userdata("user_id");
            $ci->view->assign('user_id', $user_id);

            $user_type = $ci->session->userdata("user_type");
            $ci->view->assign('user_type', $user_type);

            $user_name = $ci->session->userdata("name");
            $ci->view->assign('user_name', $user_name);

            $user_logo = $ci->session->userdata("logo");
            $ci->view->assign('user_logo', $user_logo);

            $format_users = array('get_company');
            $ci->Users_model->set_format_settings(array_fill_keys($format_users, true));
            $user = $ci->Users_model->get_user_by_id($user_id);
            $ci->Users_model->set_format_settings(array_fill_keys($format_users, false));
            if (isset($user['company']['city']) && !empty($user['company']['city'])) {
                $ci->view->assign('user_city', $user['company']['city']);
            }

            $ci->view->assign('auth_user', 1);
        } else {
            $ci->view->assign('auth_user', 0);
        }

        $user_types = array();
        foreach ($ci->Users_model->getUserTypes() as $user_type) {
            if ($ci->pg_module->get_module_config("users", $user_type . "_register_enabled")) {
                $user_types[] = $user_type;
            }
        }
        $ci->view->assign("user_types", $user_types);

        return $ci->view->fetch("helper_auth_links", null, "users");
    }

}

if (!function_exists("last_registered")) {

    /**
     * Return last registrated users block
     *
     * @param integer $count number of users
     *
     * @return string
     */
    function last_registered($count = 9)
    {
        $ci = &get_instance();
        $ci->load->model("Users_model");
        $attrs["where"]["user_logo !="] = "";
        $users = $ci->Users_model->get_active_users_list(1, $count, array("date_created" => "DESC"), $attrs);
        $ci->view->assign("users", $users);

        return $ci->view->fetch("helper_last_registered", null, "users");
    }

}

if (!function_exists("featured_users_block")) {

    /**
     * Return featured users block
     *
     * @param array $params block parameters
     *
     * @return string
     */
    function featured_users_block($params)
    {
        $ci = &get_instance();
        $ci->load->model("Users_model");

        $user_types = $ci->Users_model->getUserTypes();

        $header = $params["header"];

        $criteria["where"]["user_type"] = (isset($params["user_type"]) && in_array($params["user_type"], $user_types)) ? $params["user_type"] : current($user_types);
        $criteria["where"]["is_featured"] = 1;

        $items_on_page = intval($params["count"]);
        if (!($items_on_page)) {
            $items_on_page = $ci->pg_module->get_module_config("users", "items_per_page");
        }

        $carousel_items = intval($params["visible"]);
        if (!$carousel_items) {
            $carousel_items = 5;
        }

        $order = "date_modified";
        $order_direction = "DESC";
        $order_array[$order] = $order_direction;

        $items_count = $ci->Users_model->get_active_users_count($criteria);

        if (!$items_count) {
            return '';
        }

        $users = $ci->Users_model->get_active_users_list(1, $items_on_page, $order_array, $criteria);
        $ci->view->assign("users", $users);

        $page_data = array(
            "header" => $header,
            "rand" => rand(100000, 999999),
            "count" => count($users),
            "visible" => $carousel_items,
        );
        $ci->view->assign("users_page_data", $page_data);

        return $ci->view->fetch("users_carousel_block", null, "users");
    }

}

if (!function_exists("user_select")) {

    /**
     * Return users selector block
     *
     * @param array   $selected   selected users
     * @param integer $max_select max selected users
     * @param string  $var_name   variable name
     * @param integer $user_type  user type
     * @param string  $template   block template
     * @param array   $params     block parameters
     *
     * @return string
     */
    function user_select($selected = array(), $max_select = 0, $var_name = "id_user", $user_type = '', $template = "default", $params = array())
    {
        $ci = &get_instance();
        $ci->load->model("Users_model");

        if ($max_select == 1 && !empty($selected) && !is_array($selected)) {
            $selected = array($selected);
        }

        if (!empty($selected)) {
            if (!is_array($selected)) {
                $selected = array($selected);
            }
            $data["selected"] = $ci->Users_model->get_users_list(null, null, null, array(), $selected, true);
            $data["selected_str"] = implode(",", $selected);
        } else {
            $data["selected_str"] = "";
        }

        if (!empty($user_type)) {
            $params['user_type'] = $user_type;
        }

        $data["max_select"] = $max_select;
        $data["var_name"] = $var_name;
        $data["template"] = $template;
        $data["params"] = $params;
        $data["rand"] = rand(100000, 999999);

        $ci->view->assign("select_data", $data);

        return $ci->view->fetch("helper_user_select_" . $template, null, "users");
    }

}

if (!function_exists("view_user_block")) {

    /**
     * Return user contacts block
     *
     * @param array $params block parameters
     *
     * @return string
     */
    function view_user_block($params)
    {
        $ci = &get_instance();
        $ci->load->model("Users_model");
        if (!isset($params['user'])) {
            $ci->Users_model->set_format_settings(array('get_contact' => true, 'get_field_editor' => true));
            $params['user'] = $ci->Users_model->get_user_by_id($params['user_id']);
            $ci->Users_model->set_format_settings(array('get_contact' => false, 'get_field_editor' => false));
        }

        switch ($params['user']['user_type']) {
            case 'agent': {
                    if ($params['user']['agent_company'] != 0) {
                        if ($params['user']['agent_status'] != 0) {
                            $ci->load->model('Users_model');
                            $params['company_user'] = $ci->Users_model->get_user_by_id($params['user']['agent_company']);
                            if (!empty($params['company_user'])) {
                                $ci->view->assign('company_user', $params['company_user']);
                            }
                        }
                    }
                }
        }

        $ci->load->library('mobile_detect');
        $is_mobile = $ci->mobile_detect->isMobile();
        $ci->view->assign('is_mobile', $is_mobile);

        $ci->load->helper('countries');

        $location_data = array(
            array(
                'country' => $params['user']['id_country'],
                'region' => $params['user']['id_region'],
                'city' => $params['user']['id_city'],
                'district' => $params['user']['id_district'],
                'address' => $params['user']['address'],
                'zip_code' => $params['user']['postal_code'],
            ),
        );
        $user_locations = address_output_format($location_data, 'city', true);
        $params['user']['location'] = $user_locations[0];

        $ci->view->assign("user_data", $params['user']);
        if (isset($params['template'])) {
            $ci->view->assign("template", $params['template']);
        }

        return $ci->view->fetch("helper_view_user_block", null, "users");
    }

}

if (!function_exists("admin_home_users_block")) {

    /**
     * Return statistics block for admin homepage
     *
     * @return string
     */
    function admin_home_users_block()
    {
        $ci = &get_instance();

        $auth_type = $ci->session->userdata("auth_type");
        if ($auth_type != "admin") {
            return "";
        }

        $user_type = $ci->session->userdata("user_type");

        $show = true;

        $stat_users = array(
            "index_method" => true,
            "moderation_method" => true,
        );

        if ($user_type == "moderator") {
            $show = false;
            $ci->load->model("Moderators_model");
            $methods_users = $ci->Moderators_model->get_module_methods("users");
            $methods_moderation = $ci->Moderators_model->get_module_methods("moderation");
            if ((is_array($methods_users) && !in_array("index", $methods_users)) || (is_array($methods_moderation) && !in_array("index", $methods_moderation))) {
                $show = true;
            } else {
                $permission_data = $ci->session->userdata("permission_data");
                if (isset($permission_data["users"]["index"]) && $permission_data["users"]["index"] == 1) {
                    $show = true;
                    $stat_users["index_method"] = (bool) $permission_data["users"]["index"];
                }
            }
        }

        if (!$show) {
            return "";
        }

        $ci->load->model("Users_model");
        $user_types = $ci->Users_model->getUserTypes();
        foreach ($user_types as $user_type) {
            $stat_users[$user_type]["all"] = $ci->Users_model->get_users_count_by_filters(array('type' => $user_type));
            $stat_users[$user_type]["active"] = $ci->Users_model->get_users_count_by_filters(array('type' => $user_type, 'active' => 1));
            $stat_users[$user_type]["blocked"] = $ci->Users_model->get_users_count_by_filters(array('type' => $user_type, 'not_active' => 1));
            $stat_users[$user_type]["unconfirm"] = $ci->Users_model->get_users_count_by_filters(array('type' => $user_type, 'not_confirm' => 1));
        }

        if ($ci->pg_module->is_module_active("moderation")) {
            $ci->load->model("Moderation_model");
            $stat_users["icons"] = $ci->Moderation_model->get_moderation_list_count("user_logo");
        }

        $ci->view->assign("user_types", $user_types);
        $ci->view->assign("stat_users", $stat_users);

        return $ci->view->fetch("helper_admin_home_block", "admin", "users");
    }

}

if (!function_exists("users_search_block")) {

    /**
     * Return form of users quick search
     *
     * @param array $params block parameters
     *
     * @return string
     */
    function users_search_block(array $params=array())
    {
        $ci = &get_instance();
        $ci->load->model('Users_model');

        $user_types = $ci->Users_model->getUserTypes();
        foreach ($user_types as $key => $user_type) {
            $search_enabled = $ci->pg_module->get_module_config('users', $user_type . '_search_enabled');
            if (!$search_enabled) {
                unset($user_types[$key]);
            }
        }
        if (empty($user_types)) {
            return '';
        }

        $ci->view->assign('user_types', $user_types);
        $filters = isset($_SESSION['users_list']['data']) ? $_SESSION['users_list']['data'] : array();
        $ci->view->assign("data", $filters);

        if (!empty($filters['type'])) {
            $user_type = $filters['type'];
        } else {
            $user_type = current($user_types);
        }

        if (isset($params['user_type'])) {
            $user_type = $params['user_type'];
        }

        //load quick form
        $quick_search_form_data = $ci->Users_model->get_search_extend_form('user', 'quick_search_form_gid', false);
        if (count($quick_search_form_data) > 0) {
            if (isset($filters['field_editor_data']['quick_search_form'])) {
                $ci->view->assign('data', $filters['field_editor_data']['quick_search_form']);
            }
            $ci->view->assign('form_data', $quick_search_form_data);
            $ci->view->assign('form_rand', rand(100000, 999999));
            $quick_search_main_form = $ci->view->fetch('helper_search_form_block', 'user', 'users');
            $ci->view->assign('quick_search_main_form', $quick_search_main_form);
        }

        //load extended form
        $quick_search_form_data = $ci->Users_model->get_search_extend_form($user_type, 'quick_search_form_gid', false);
        if (count($quick_search_form_data) > 0) {
            if (isset($filters['field_editor_data']['quick_search_form'])) {
                $ci->view->assign('data', $filters['field_editor_data']['quick_search_form']);
            }
            $ci->view->assign('form_data', $quick_search_form_data);
            $ci->view->assign('form_rand', rand(100000, 999999));
            $quick_search_extend_form = $ci->view->fetch('helper_search_form_block', 'user', 'users');
            $ci->view->assign('quick_search_extend_form', $quick_search_extend_form);
        }

        return $ci->view->fetch('helper_search_block', 'user', 'users');
    }

}

if (!function_exists("make_agent")) {

    /**
     * Return approve/decline agent block
     *
     * @param integer $user_id user identifier
     *
     * @return string
     */
    function make_agent($user_id)
    {
        $ci = &get_instance();
        if ($ci->session->userdata("auth_type") != "user") {
            return;
        }

        $ci->load->model("Users_model");
        $user_data = $ci->Users_model->get_user_by_id($ci->session->userdata("user_id"));
        $user_data = $ci->Users_model->format_user($user_data);
        $ci->view->assign("user_data", $user_data);

        return $ci->view->fetch("helper_make_agent", null, "users");
    }

}

if (!function_exists('users_currency_select')) {

    /**
     * Returns currency selector block
     *
     * @return string
     */
    function users_currency_select()
    {
        $ci = &get_instance();
        if (!$ci->pg_module->is_module_active("currencies")) {
            return '';
        }

        $ci->load->model('Currencies_model');
        $currencies = $ci->Currencies_model->get_currency_list();

        foreach ($currencies as $cid => $currency_data) {
            if (!$currency_data["status"]) {
                unset($currencies[$cid]);
            }
        }
        if (count($currencies) <= 1) {
            return '';
        }


        $ci->view->assign("currencies", $currencies);

        return $ci->view->fetch("helper_currency_select", null, "users");
    }

}

if (!function_exists('user_info')) {

    /**
     * Return user info block
     *
     * @param array $params block parameters
     *
     * @return string
     */
    function user_info($params)
    {
        if (!isset($params['user']) || !$params['user']) {
            return '';
        }

        $ci = &get_instance();

        if (!is_array($params['user'])) {
            $ci->load->model('Users_model');
            $params['user'] = $ci->Users_model->get_user_by_id($params['user']);
        }

        if (empty($params['user'])) {
            return '';
        }

        $ci->view->assign('user', $params['user']);

        if (isset($params['all_listings'])) {
            $ci->view->assign('show_all_listings', $params['all_listings']);
        }

        if ($ci->session->userdata("auth_type") != 'user') {
            $ci->view->assign('show_contact_form', 1);
        }

        switch ($params['user']['user_type']) {
            case 'company': {
                    $map_markers = array(
                        array(
                            'gid' => $params['user']['id'],
                            "country" => $params['user']["country"],
                            "region" => $params['user']["region"],
                            "city" => $params['user']["city"],
                            "address" => $params['user']["address"],
                            "postal_code" => $params['user']["postal_code"],
                            "lat" => (float) $params['user']["lat"],
                            "lon" => (float) $params['user']["lon"],
                            "html" => ($params['user']['location'] ? $params['user']['location'] : $params['user']["output_name"] . ', ' . l($params['users']['user_type'], 'users')),
                            "dragging" => false,
                        ),
                    );

                    $current_language = $ci->pg_language->get_lang_by_id($ci->pg_language->current_lang_id);
                    $view_settings = array(
                        "class" => "",
                        "geocode_listener" => "get_user_geocode_data",
                        "lang" => $current_language["code"],
                    );
                    $ci->view->assign("map_settings", $view_settings);
                    $ci->view->assign("map_markers", $map_markers);
                    break;
                }
        }

        $is_owner = 0;
        if ($ci->session->userdata("auth_type") == "user") {
            $auth_id = $ci->session->userdata("user_id");
            if ($auth_id == $params['user']['id']) {
                $is_owner = 1;
            }
        }
        $ci->view->assign('is_user_owner', $is_owner);

        if (!isset($params['message'])) {
            $params['message'] = l('text_user_message', 'users');
        }
        $ci->view->assign('contact_message', $params['message']);

        return $ci->view->fetch("helper_user_info", null, "users");
    }

}

if (!function_exists('show_profile_info')) {

    /**
     * Return profile completeness block
     *
     * @return string
     */
    function show_profile_info()
    {
        $ci = &get_instance();
        $auth_type = $ci->session->userdata('auth_type');
        if (!$auth_type || 'guest' === $auth_type) {
            return '';
        }
        $ci->uri->_fetch_uri_string();
        $uri = trim(substr($ci->uri->uri_string, 1), "/");
        if (empty($uri) || count(explode("/", $uri)) < 2) {
            $class = $ci->router->fetch_class(true);
            $method = $ci->router->fetch_method();
            $uri = $class . '/' . $method;
        }

        $user = $ci->Users_model->get_user_by_id($ci->session->userdata("user_id"));
        $ci->view->assign('user', $user);
        $ci->view->assign('complete', $ci->Users_model->get_profile_complete($user));

        return $ci->view->fetch('helper_profile_info', null, "users");
    }

}

if (!function_exists('is_login_enabled')) {

    /**
     * Check user login is enabled
     *
     * @return boolean
     */
    function is_login_enabled()
    {
        $ci = &get_instance();
        $ci->load->model('Users_model');
        $login_enabled = false;
        $user_types = $ci->Users_model->getUserTypes();
        foreach ($user_types as $user_type) {
            $user_login_enabled = $ci->pg_module->get_module_config("users", $user_type . "_login_enabled");
            $login_enabled = $login_enabled || $user_login_enabled;
        }

        return $login_enabled;
    }

}

if (!function_exists('get_user_types')) {

    /**
     * Return user types
     *
     * @return boolean
     */
    function get_user_types()
    {
        $ci = &get_instance();
        $ci->load->model('Users_model');
        return $ci->Users_model->getUserTypesNames();
    }

}

if (!function_exists("users_visitors_block")) {

    /**
     * Return profile visitors block
     *
     * @return string
     */
    function users_visitors_block()
    {
        $ci = &get_instance();

        $user_id = intval($ci->session->userdata("user_id"));

        $ci->load->model("users/models/Users_visitor_model");

        $visitors_count_week = $ci->Users_visitor_model->get_visitors_count(array('user' => $user_id, 'week' => 1));
        $ci->view->assign("visitors_count_week", $visitors_count_week);

        $visitors_count_month = $ci->Users_visitor_model->get_visitors_count(array('user' => $user_id, 'month' => 1));
        $ci->view->assign("visitors_count_month", $visitors_count_month);

        return $ci->view->fetch("helper_profile_visitors", null, "users");
    }

}

if (!function_exists("users_visits_block")) {

    /**
     * Return profile visits block
     *
     * @return string
     */
    function users_visits_block()
    {
        $ci = &get_instance();

        $user_id = intval($ci->session->userdata("user_id"));

        $ci->load->model("users/models/Users_visitor_model");

        $visits_count_week = $ci->Users_visitor_model->get_visitors_count(array('visitor' => $user_id, 'week' => 1));
        $ci->view->assign("visits_count_week", $visits_count_week);

        $visits_count_month = $ci->Users_visitor_model->get_visitors_count(array('visitor' => $user_id, 'month' => 1));
        $ci->view->assign("visits_count_month", $visits_count_month);

        return $ci->view->fetch("helper_profile_visits", null, "users");
    }

}

if (!function_exists("users_pagination_block")) {

    /**
     * Return user pagination block
     *
     * @param array $user data
     *
     * @return string
     */
    function users_pagination_block($user)
    {
        $ci = &get_instance();
        $ci->load->model('Users_model');

        $prev_user = $user;
        $next_user = $user;
        $current_page = 1;
        $total_pages = 1;

        $current_settings = isset($_SESSION["users_pagination"]) ? $_SESSION["users_pagination"] : array();
        if (!isset($current_settings["order"])) {
            $current_settings["order"] = "id";
        }
        if (!isset($current_settings["order_direction"])) {
            $current_settings["order_direction"] = "DESC";
        }
        if (!isset($current_settings["page"])) {
            $current_settings["page"] = 1;
        }
        if (!isset($current_settings["data"])) {
            $current_settings["data"] = array();
        }
        $items_on_page = $ci->pg_module->get_module_config("start", "index_items_per_page") + 2;
        if ($current_settings['data']['type'] != $user['user_type']) {
            $current_settings['data'] = array('type' => $user['user_type']);
        }
        $page = ceil(($current_settings["page"] * $items_on_page - 3) / $items_on_page);

        $params = $current_settings['data'];
        $params['active'] = 1;

        $users_count = $ci->Users_model->get_users_count_by_filters($params);
        if ($users_count > 1) {
            $order_array = array();

            $order = $current_settings['order'];
            $order_direction = $current_settings['order_direction'];
            $user_type = $current_settings['data']['type'];

            switch ($order) {
                case 'default':
                    $order_array = array('id' => $order_direction);
                    break;
                case 'name':
                    if ($user_type == 'company') {
                        $order_array = array('company_name' => $order_direction);
                    } else {
                        $order_array = array("CONCAT(fname, ' ', sname)" => $order_direction);
                    }
                    break;
                default:
                    $order_array = array($order => $order_direction);
                    break;
            }

            $users = $ci->Users_model->get_users_pagination($params, $page, $items_on_page, $order_array);
            $pages_count = ceil($users_count / $items_on_page);
            $count = count($users);
            foreach ($users as $i => $data) {
                if ($data['id'] == $user['id']) {
                    $page = $current_settings["page"];
                    $current_page = ($current_settings["page"] - 1) * $items_on_page + $i + ($current_settings["page"] == 1 ? 1 : -1);
                    $total_pages = $users_count;

                    if ($i > 1) {
                        $prev_user = $users[$i - 1];
                    } elseif ($current_settings["page"] > 1) {
                        $prev_user = $users[$i - 1];
                        --$page;
                    } elseif ($i > 0) {
                        $prev_user = $users[$i - 1];
                    }

                    if ($i < $count - ($pages_count > 1 ? 2 : 1)) {
                        $next_user = $users[$i + 1];
                    } elseif ($current_settings["page"] < $pages_count) {
                        $next_user = $users[$i + 1];
                        ++$page;
                    } elseif ($current_settings["page"] > 1 && $current_settings["page"] == $pages_count && $i < $count - 1) {
                        $next_user = $users[$i + 1];
                    }

                    if ($page != $current_settings["page"]) {
                        $current_settings["page"] = $page;
                        $_SESSION["users_pagination"] = $current_settings;
                    }
                    break;
                }
            }
        }

        $ci->view->assign('prev_user', $prev_user);
        $ci->view->assign('next_user', $next_user);
        $ci->view->assign('current_page', $current_page);
        $ci->view->assign('total_pages', $total_pages);

        return $ci->view->fetch("helper_users_pagination", null, "users");
    }

}

if (!function_exists('users_last_registered')) {

    /**
     * Return last registered users block
     *
     * @param string  $with_logo with logo
     * @param integer $count     users count
     * @param string  $view      view mode
     *
     * @return void
     */
    function users_last_registered($with_logo, $count = 8, $view = 'gallery')
    {
        $ci = &get_instance();
        $ci->load->model('Users_model');

        $count = intval($count);
        if (!$count) {
            $count = 8;
        }

        if ($params["with_logo"] == "yes") {
            $attrs["where"]["user_logo !="] = "";
        } else {
            $attrs = array();
        }

        $users = $ci->Users_model->get_active_users_list(1, $count, array("date_created" => "DESC"), $attrs);
        $ci->view->assign("users", $users);

        return $ci->view->fetch("helper_last_registered", null, "users");
    }

}

if (!function_exists('select_user_type')) {

    /**
     * Return select user type block
     *
     * @param array $params block parameters
     *
     * @return void
     */
    function select_user_type($params)
    {
        $ci = &get_instance();
        if (empty($params['var'])) {
            $params['var'] = 'user_type';
        }
        $ci->load->model('Users_model');
        $user_types = $ci->Users_model->getUserTypes();
        $ci->view->assign("user_types", $user_types);
        $ci->view->assign('var', $params['var']);
        $ci->view->assign('value', $params['value']);

        return $ci->view->fetch("helper_select_user_type", "admin", "users");
    }

}
