<?php

namespace Pg\Modules\Users\Models;

/**
 * Users module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
if (!defined("BASEPATH")) {
    exit("No direct script access allowed");
}

if (!defined('USERS_EVENTS_TABLE')) {
    define("USERS_EVENTS_TABLE", DB_PREFIX . "users_events");
}

/**
 * Users events model
 *
 * @package 	PG_RealEstate
 * @subpackage 	Users
 *
 * @category	models
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 * */
class Users_events_model extends \Model
{

    /**
     * Link to CodeIgniter object
     *
     * @var object
     */
    protected $CI;

    /**
     * Link to database object
     *
     * @var object
     */
    protected $DB;

    /**
     * Module properties in data source
     *
     * @var array
     */
    protected $fields = array(
        'id',
        'module',
        'model',
        'save_callback',
        'delete_callback',
    );

    /**
     * Modules cache
     *
     * @var array
     */
    protected $updates = array();

    /**
     * Class constructor
     *
     * @return Users_events_model
     */
    public function __construct()
    {
        parent::__construct();
        $this->CI = &get_instance();
        $this->DB = &$this->CI->db;

        $this->updates = $this->_get_updates_list();
    }

    /**
     * Return events modules as array
     *
     * @return array
     */
    protected function _get_updates_list()
    {
        $this->DB->select(implode(', ', $this->fields));
        $this->DB->from(USERS_EVENTS_TABLE);
        $results = $this->DB->get()->result_array();
        if (!empty($results) && is_array($results)) {
            return $results;
        }

        return array();
    }

    /**
     * Add module
     *
     * @param array $module_data module data
     *
     * @return void
     */
    public function add_module($module_data)
    {
        $this->DB->insert(USERS_EVENTS_TABLE, $module_data);
    }

    /**
     * Remove module
     *
     * @param string $module_gid module guid
     *
     * @return void
     */
    public function delete_module($module_gid)
    {
        $this->DB->where("module", $module_gid);
        $this->DB->delete(USERS_EVENTS_TABLE);
    }

    /**
     * Call functions on save user
     *
     * @param array $user_old_data user old data
     * @param array $user_new_data user new data
     *
     * @return void
     */
    public function onsave_user($user_old_data, $user_new_data)
    {
        foreach ($this->updates as $update) {
            if (strtolower($update['module']) . '_model' != strtolower($update['model'])) {
                $model_path = $update['module'] . '/models/' . $update['model'];
            } else {
                $model_path = $update['model'];
            }
            $this->CI->load->model($model_path);
            $this->CI->{$update['model']}->{$update['save_callback']}($user_old_data, $user_new_data);
        }
    }

    /**
     * Call functions on remove user
     *
     * @param array $user_data user data
     *
     * @return void
     */
    public function ondelete_user($user_data)
    {
        foreach ($this->updates as $update) {
            if (strtolower($update['module']) . '_model' != strtolower($update['model'])) {
                $model_path = $update['module'] . '/models/' . $update['model'];
            } else {
                $model_path = $update['model'];
            }
            $this->CI->load->model($model_path);
            $this->CI->{$update['model']}->{$update['delete_callback']}($user_data);
        }
    }

}
