<?php

namespace Pg\Modules\Users_payments\Controllers;

/**
 * Users payments module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2016 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */

/**
 * Users payments api controller
 *
 * @package 	PG_RealEstate
 * @subpackage 	Users payments
 *
 * @category	controllers
 *
 * @copyright 	Copyright (c) 2000-2016 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Api_Users_payments extends \Controller
{

    /**
     * Constructor
     *
     * @return Api_Users_payments
     */
    public function __construct()
    {
        parent::__construct();
    }

    /**
     * Save payment for payment system
     *
     * @return void
     */
    public function save_payment()
    {
        $user_id = $this->session->userdata('user_id');
        $amount = $this->input->post('amount', true);
        $system_gid = $this->input->post('system_gid', true);
        $payment_id = 0;

        if (empty($amount)) {
            $this->set_api_content('errors', l('error_empty_amount', 'users_payments'));
        } elseif (empty($system_gid)) {
            $this->set_api_content('errors', l('error_empty_system_gid', 'users_payments'));
        } else {
            $base_currency_gid = $this->pg_module->get_module_config('start', 'base_currency');

            $this->load->model('Currencies_model');
            $default_currency = $this->Currencies_model->get_currency_default(true);

            $this->load->helper('start');
            $amount = get_currency_value(array('value' => $amount, 'cur_gid' => $default_currency['gid'], 'to_gid' => $base_currency_gid));

            $additional['name'] = l('header_add_funds', 'users_payments');

            $this->load->model('payments/models/Payment_systems_model');
            if ($this->Payment_systems_model->action_html($system_gid)) {
                
            } elseif ($this->Payment_systems_model->action_js($system_gid)) {
                $save_data = array(
                    "payment_type_gid" => 'account',
                    "id_user" => $user_id,
                    "amount" => $amount,
                    "currency_gid" => $base_currency_gid,
                    "system_gid" => $system_gid,
                    "payment_data" => $additional,
                );
                $this->load->model('Payments_model');
                $validate_data = $this->Payments_model->validate_payment(null, $save_data);
                if (!empty($validate_data["errors"])) {
                    $this->set_api_content('errors', $validate_data["errors"]);
                } else {
                    $payment_id = $this->Payments_model->save_payment(null, $validate_data['data']);
                }
            } else {
                $for_validate = array(
                    "payment_type_gid" => 'account',
                    "id_user" => $user_id,
                    "amount" => $amount,
                    "currency_gid" => $base_currency_gid,
                    "system_gid" => $system_gid,
                    "payment_data" => $additional,
                );
                $this->load->model('Payments_model');
                $validate_data = $this->Payments_model->validate_payment(null, $for_validate);
                if (!empty($validate_data["errors"])) {
                    $this->set_api_content("errors", $validate_data["errors"]);
                } else {
                    $payment_id = $this->Payments_model->save_payment(null, $validate_data["data"]);
                }
            }
        }
        $this->set_api_content('data',
                               array('amount' => $amount, 'system_gid' => $system_gid, 'payment_id' => $payment_id));
    }

    /**
     * Get account value
     *
     * @return void
     */
    public function account()
    {
        $user_id = $this->session->userdata("user_id");

        $this->load->model("Users_payments_model");
        $account = $this->Users_payments_model->get_user_account($user_id);
        $data['account'] = $account;
        
        $this->load->helper('start');
       
        $data['account_output'] = currency_format_output(array('value' => $account));

        $this->set_api_content('data', $data);
    }

    /**
     * Get payment systems
     *
     * @return void
     */
    public function systems()
    {
        $this->load->model("payments/models/Payment_systems_model");
        $billing_systems = $this->Payment_systems_model->get_active_system_list();
        $data['billing_systems'] = $billing_systems;

        $this->load->model("Currencies_model");
        $base_currency = $this->Currencies_model->get_currency_default(true);

        $this->load->helper('start');

        foreach ($data['billing_systems'] as $key => $system) {
            if (isset($system['settings_data'])) {
                unset($data['billing_systems'][$key]['settings_data']);
            }
            $data['billing_systems'][$key]['form'] = $this->Payment_systems_model->action_html($system['gid']);
            $data['billing_systems'][$key]['js'] = $this->Payment_systems_model->action_js($system['gid']);
            if (empty($system['tarifs_data'])) {
                continue;
            }
            foreach ($system['tarifs_data'] as $operator_gid => $operator_data) {
                foreach ($operator_data as $k => $v) {
                    $name = currency_format_output(array('value' => $v, 'cur_gid' => $base_currency['gid']));
                    $data['billing_systems'][$key]['tarifs_data'][$operator_gid][$k] = array(
                        'value' => $v,
                        'name' => strip_tags($name),
                    );
                }
            }
        }

        $this->set_api_content('data', $data);
    }

}
