<?php

namespace Pg\Modules\Users_payments\Models;

/**
 * Users payments module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */

/**
 * Users payments install model
 *
 * @package 	PG_RealEstate
 * @subpackage 	Users payments
 *
 * @category	models
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Users_payments_install_model extends \Model
{

    /**
     * Link to CodeIgniter object
     *
     * @var object
     */
    protected $ci;

    /**
     * Payments configuration
     *
     * @var array
     */
    protected $payment_types = array(
        array(
            'gid' => 'account',
            'callback_module' => 'users_payments',
            'callback_model' => 'Users_payments_model',
            'callback_method' => 'update_user_account',
        ),
    );

    /**
     * Notifications configuration
     *
     * @var array
     */
    protected $notifications = array(
        'notifications' => array(
            array('gid' => 'users_update_account', 'send_type' => 'simple'),
        ),
        'templates' => array(
            array('gid' => 'users_update_account', 'name' => 'Add funds on account', 'vars' => array('account', 'email', 'fname', 'sname', 'nickname'), 'content_type' => 'text'),
        ),
    );

    /**
     * Constructor
     *
     * @return Users_payments_install_model
     */
    public function __construct()
    {
        parent::__construct();
        $this->ci = &get_instance();
        $this->ci->load->model('Install_model');
    }

    /**
     * Install users payments data to payments module
     *
     * @return void
     */
    public function install_payments()
    {
        // add account payment type
        $this->ci->load->model("Payments_model");
        foreach ($this->payment_types as $payment_type) {
            $data = array(
                'gid' => $payment_type['gid'],
                'callback_module' => $payment_type['callback_module'],
                'callback_model' => $payment_type['callback_model'],
                'callback_method' => $payment_type['callback_method'],
            );
            $this->ci->Payments_model->save_payment_type(null, $data);
        }
    }

    /**
     * Import users payments languages to payments module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return boolean
     */
    public function install_payments_lang_update($langs_ids = null)
    {
        if (empty($langs_ids)) {
            return false;
        }
        $langs_file = $this->ci->Install_model->language_file_read('users_payments', 'payments', $langs_ids);
        if (!$langs_file) {
            log_message('info', 'Empty payments langs data');

            return false;
        }
        $this->ci->load->model('Payments_model');
        $this->ci->Payments_model->update_langs($this->payment_types, $langs_file, $langs_ids);
    }

    /**
     * Export users services languages from payments module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return array
     */
    public function install_payments_lang_export($langs_ids = null)
    {
        $this->ci->load->model('Payments_model');
        $langs = $this->ci->Payments_model->export_langs($this->payment_types, $langs_ids);

        return array('payments' => $langs);
    }

    /**
     * Uninstall users services data from payments module
     *
     * @return void
     */
    public function deinstall_payments()
    {
        $this->ci->load->model('Payments_model');
        foreach ($this->payment_types as $payment_type) {
            $this->ci->Payments_model->delete_payment_type_by_gid($payment_type['gid']);
        }
    }

    /**
     * Install users services data to notifications module
     *
     * @return void
     */
    public function install_notifications()
    {
        // add notification
        $this->ci->load->model('Notifications_model');
        $this->ci->load->model('notifications/models/Templates_model');

        foreach ($this->notifications['templates'] as $tpl) {
            $template_data = array(
                'gid' => $tpl['gid'],
                'name' => $tpl['name'],
                'vars' => serialize($tpl['vars']),
                'content_type' => $tpl['content_type'],
                'date_add' => date('Y-m-d H:i:s'),
                'date_update' => date('Y-m-d H:i:s'),
            );
            $tpl_ids[$tpl['gid']] = $this->ci->Templates_model->save_template(null, $template_data);
        }

        foreach ($this->notifications['notifications'] as $notification) {
            $notification_data = array(
                'gid' => $notification['gid'],
                'send_type' => $notification['send_type'],
                'id_template_default' => $tpl_ids[$notification['gid']],
                'date_add' => date("Y-m-d H:i:s"),
                'date_update' => date("Y-m-d H:i:s"),
            );
            $this->ci->Notifications_model->save_notification(null, $notification_data);
        }
    }

    /**
     * Import users services languages to notifications module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return boolean
     */
    public function install_notifications_lang_update($langs_ids = null)
    {
        if (empty($langs_ids)) {
            return false;
        }
        $this->ci->load->model('Notifications_model');

        $langs_file = $this->ci->Install_model->language_file_read('users_payments', 'notifications', $langs_ids);
        if (!$langs_file) {
            log_message('info', 'Empty notifications langs data');

            return false;
        }

        $this->ci->Notifications_model->update_langs($this->notifications, $langs_file, $langs_ids);

        return true;
    }

    /**
     * Export users services languages from notifications module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return array
     */
    public function install_notifications_lang_export($langs_ids = null)
    {
        $this->ci->load->model('Notifications_model');
        $langs = $this->ci->Notifications_model->export_langs($this->notifications, $langs_ids);

        return array('notifications' => $langs);
    }

    /**
     * Install data of users services module
     *
     * @return void
     */
    public function _arbitrary_installing()
    {
        
    }

    /**
     * Uninstall data of users services module
     *
     * @return void
     */
    public function _arbitrary_deinstalling()
    {
        
    }

}
