<?php

namespace Pg\Modules\Users_settings\Controllers;

/**
 * Users settings module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
use Pg\Libraries\View;

/**
 * Users settings admin side controller
 *
 * @package 	PG_RealEstate
 * @subpackage 	Users_settings
 *
 * @category	controllers
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Admin_Users_settings extends \Controller
{

    /**
     * Constructor
     *
     * @return Admin_Users
     */
    public function __construct()
    {
        parent::__construct();
        $this->load->model("Users_model");
        $this->load->model("Menu_model");
        $this->Menu_model->set_menu_active_item("admin_menu", "users_menu_item");
    }

    /**
     * Manage users types settings
     *
     * @return void
     */
    public function index()
    {
        $user_types = $this->Users_model->getUserTypes();

        if ($this->input->post("btn_auth_save")) {
            $post_data = $this->input->post('data', true);
            $validate_data = $this->Users_model->validate_settings($post_data);

            if (!empty($validate_data["errors"])) {
                $this->system_messages->addMessage(View::MSG_ERROR, $validate_data["errors"]);
            } else {
                foreach ($validate_data["data"] as $setting => $value) {
                    $this->pg_module->set_module_config("users", $setting, $value);
                }

                // menu updates
                $this->load->model("Menu_model");

                if (in_array("agent", $user_types)) {
                    $menu = $this->Menu_model->get_menu_by_gid("company_account_menu");
                    $menu_item = $this->Menu_model->get_menu_item_by_gid("my_agents_item", $menu["id"]);
                    $this->Menu_model->save_menu_item($menu_item["id"],
                                                      array("status" => $validate_data["data"]["agent_login_enabled"] ? 1 : 0));
                }

                // menu updates professionals
                $menu = $this->Menu_model->get_menu_by_gid("guest_main_menu");

                if (in_array("agent", $user_types)) {
                    $menu_item = $this->Menu_model->get_menu_item_by_gid("main-menu-pro-agent-item", $menu["id"]);
                    $this->Menu_model->save_menu_item($menu_item["id"],
                                                      array("status" => $validate_data["data"]["agent_search_enabled"] ? 1 : 0));
                }

                if (in_array("company", $user_types)) {
                    $menu_item = $this->Menu_model->get_menu_item_by_gid("main-menu-pro-company-item", $menu["id"]);
                    $this->Menu_model->save_menu_item($menu_item["id"],
                                                      array("status" => $validate_data["data"]["company_search_enabled"] ? 1 : 0));
                }

                if (in_array("private", $user_types)) {
                    $menu_item = $this->Menu_model->get_menu_item_by_gid("main-menu-pro-private-item", $menu["id"]);
                    $this->Menu_model->save_menu_item($menu_item["id"],
                                                      array("status" => $validate_data["data"]["private_search_enabled"] ? 1 : 0));
                }

                if (in_array("private", $user_types)) {
                    $menu = $this->Menu_model->get_menu_by_gid("private_main_menu");
                    $menu_item = $this->Menu_model->get_menu_item_by_gid("private-main-menu-pro-agent-item", $menu["id"]);
                    $this->Menu_model->save_menu_item($menu_item["id"],
                                                      array("status" => $validate_data["data"]["agent_search_enabled"] ? 1 : 0));
                    $menu_item = $this->Menu_model->get_menu_item_by_gid("private-main-menu-pro-company-item",
                                                                         $menu["id"]);
                    $this->Menu_model->save_menu_item($menu_item["id"],
                                                      array("status" => $validate_data["data"]["company_search_enabled"] ? 1 : 0));
                    $menu_item = $this->Menu_model->get_menu_item_by_gid("private-main-menu-pro-private-item",
                                                                         $menu["id"]);
                    $this->Menu_model->save_menu_item($menu_item["id"],
                                                      array("status" => $validate_data["data"]["private_search_enabled"] ? 1 : 0));
                }

                if (in_array("company", $user_types)) {
                    $menu = $this->Menu_model->get_menu_by_gid("company_main_menu");
                    $menu_item = $this->Menu_model->get_menu_item_by_gid("company-main-menu-pro-agent-item", $menu["id"]);
                    $this->Menu_model->save_menu_item($menu_item["id"],
                                                      array("status" => $validate_data["data"]["agent_search_enabled"] ? 1 : 0));
                    $menu_item = $this->Menu_model->get_menu_item_by_gid("company-main-menu-pro-company-item",
                                                                         $menu["id"]);
                    $this->Menu_model->save_menu_item($menu_item["id"],
                                                      array("status" => $validate_data["data"]["company_search_enabled"] ? 1 : 0));
                    $menu_item = $this->Menu_model->get_menu_item_by_gid("company-main-menu-pro-private-item",
                                                                         $menu["id"]);
                    $this->Menu_model->save_menu_item($menu_item["id"],
                                                      array("status" => $validate_data["data"]["private_search_enabled"] ? 1 : 0));
                }

                if (in_array("agent", $user_types)) {
                    $menu = $this->Menu_model->get_menu_by_gid("agent_main_menu");
                    $menu_item = $this->Menu_model->get_menu_item_by_gid("agent-main-menu-pro-agent-item", $menu["id"]);
                    $this->Menu_model->save_menu_item($menu_item["id"],
                                                      array("status" => $validate_data["data"]["agent_search_enabled"] ? 1 : 0));
                    $menu_item = $this->Menu_model->get_menu_item_by_gid("agent-main-menu-pro-company-item", $menu["id"]);
                    $this->Menu_model->save_menu_item($menu_item["id"],
                                                      array("status" => $validate_data["data"]["company_search_enabled"] ? 1 : 0));
                    $menu_item = $this->Menu_model->get_menu_item_by_gid("agent-main-menu-pro-private-item", $menu["id"]);
                    $this->Menu_model->save_menu_item($menu_item["id"],
                                                      array("status" => $validate_data["data"]["private_search_enabled"] ? 1 : 0));
                }

                if (!$validate_data["data"]["private_search_enabled"] && !$validate_data["data"]["company_search_enabled"] && !$validate_data["data"]["agent_search_enabled"]) {
                    $menu = $this->Menu_model->get_menu_by_gid("guest_main_menu");
                    $menu_item = $this->Menu_model->get_menu_item_by_gid("main-menu-pro-item", $menu["id"]);
                    $this->Menu_model->save_menu_item($menu_item["id"], array("status" => 0));

                    if (in_array("private", $user_types)) {
                        $menu = $this->Menu_model->get_menu_by_gid("private_main_menu");
                        $menu_item = $this->Menu_model->get_menu_item_by_gid("private-main-menu-pro-item", $menu["id"]);
                        $this->Menu_model->save_menu_item($menu_item["id"], array("status" => 0));
                    }

                    if (in_array("company", $user_types)) {
                        $menu = $this->Menu_model->get_menu_by_gid("company_main_menu");
                        $menu_item = $this->Menu_model->get_menu_item_by_gid("company-main-menu-pro-item", $menu["id"]);
                        $this->Menu_model->save_menu_item($menu_item["id"], array("status" => 0));
                    }

                    if (in_array("agent", $user_types)) {
                        $menu = $this->Menu_model->get_menu_by_gid("agent_main_menu");
                        $menu_item = $this->Menu_model->get_menu_item_by_gid("agent-main-menu-pro-item", $menu["id"]);
                        $this->Menu_model->save_menu_item($menu_item["id"], array("status" => 0));
                    }
                } else {
                    $menu = $this->Menu_model->get_menu_by_gid("guest_main_menu");
                    $menu_item = $this->Menu_model->get_menu_item_by_gid("main-menu-pro-item", $menu["id"]);
                    $this->Menu_model->save_menu_item($menu_item["id"], array("status" => 1));

                    if (in_array("private", $user_types)) {
                        $menu = $this->Menu_model->get_menu_by_gid("private_main_menu");
                        $menu_item = $this->Menu_model->get_menu_item_by_gid("private-main-menu-pro-item", $menu["id"]);
                        $this->Menu_model->save_menu_item($menu_item["id"], array("status" => 1));
                    }

                    if (in_array("company", $user_types)) {
                        $menu = $this->Menu_model->get_menu_by_gid("company_main_menu");
                        $menu_item = $this->Menu_model->get_menu_item_by_gid("company-main-menu-pro-item", $menu["id"]);
                        $this->Menu_model->save_menu_item($menu_item["id"], array("status" => 1));
                    }

                    if (in_array("agent", $user_types)) {
                        $menu = $this->Menu_model->get_menu_by_gid("agent_main_menu");
                        $menu_item = $this->Menu_model->get_menu_item_by_gid("agent-main-menu-pro-item", $menu["id"]);
                        $this->Menu_model->save_menu_item($menu_item["id"], array("status" => 1));
                    }
                }

                $this->system_messages->addMessage(View::MSG_SUCCESS, l("success_settings_saved", "users"));
            }
        }

        $this->Menu_model->set_menu_active_item("admin_menu", "users_menu_item");
        $data = array();
        foreach ($user_types as $type) {
            $data[$type]["login_enabled"] = $this->pg_module->get_module_config("users", $type . "_login_enabled") ? 1 : 0;
            $data[$type]["register_enabled"] = $this->pg_module->get_module_config("users", $type . "_register_enabled") ? 1 : 0;
            $data[$type]["register_mail_confirm"] = $this->pg_module->get_module_config("users",
                                                                                        $type . "_register_mail_confirm") ? 1 : 0;
            $data[$type]["search_enabled"] = $this->pg_module->get_module_config("users", $type . "_search_enabled") ? 1 : 0;
        }

        $this->view->assign("data", $data);
        $this->view->setHeader(l("admin_header_users_list", "users_settings"));

        $this->view->render("settings");
    }

}
