<?php

namespace Pg\Modules\Users_settings\Models;

/**
 * Users settings module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */

/**
 * Users settings install model
 *
 * @package 	PG_RealEstate
 * @subpackage 	Users_settings
 *
 * @category	models
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Users_settings_install_model extends \Model
{

    /**
     * Link to CodeIgniter object
     *
     * @var object
     */
    protected $ci;

    /**
     * Menu configuration
     *
     * @var array
     */
    protected $menu = array(
        "admin_users_types_menu" => array(
            "action" => "none",
            "name" => "Admin mode - Users - tabs",
            "items" => array(
                "users_types_settings_item" => array("action" => "create", "link" => "admin/users_settings/index", "status" => 1, "sorter" => 4),
            ),
        ),
    );

    /**
     * Moderators configuration
     *
     * @var array
     */
    protected $moderators = array(
        array("module" => "users_settings", "method" => "index", "is_default" => 1),
    );

    /**
     * Constructor
     *
     * @return Users_settings_install_model
     */
    public function __construct()
    {
        parent::__construct();
        $this->ci = &get_instance();
    }

    /**
     * Install users_settings data to menu module
     *
     * @return void
     */
    public function install_menu()
    {
        $this->ci->load->helper("menu");

        foreach ($this->menu as $gid => $menu_data) {
            $this->menu[$gid]["id"] = linked_install_set_menu($gid, $menu_data["action"], $menu_data["name"]);
            linked_install_process_menu_items($this->menu, "create", $gid, 0, $this->menu[$gid]["items"]);
        }
    }

    /**
     * Import users_settings languages to menu module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return boolean
     */
    public function install_menu_lang_update($langs_ids = null)
    {
        if (empty($langs_ids)) {
            return false;
        }
        $langs_file = $this->ci->Install_model->language_file_read("users_settings", "menu", $langs_ids);

        if (!$langs_file) {
            log_message("info", "Empty menu langs data (users_settings)");

            return false;
        }

        $this->ci->load->helper("menu");

        foreach ($this->menu as $gid => $menu_data) {
            linked_install_process_menu_items($this->menu, "update", $gid, 0, $this->menu[$gid]["items"], $gid,
                                              $langs_file);
        }

        return true;
    }

    /**
     * Export users_settings languages from menu module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return array
     */
    public function install_menu_lang_export($langs_ids)
    {
        if (empty($langs_ids)) {
            return false;
        }
        $this->ci->load->helper("menu");

        $return = array();
        foreach ($this->menu as $gid => $menu_data) {
            $temp = linked_install_process_menu_items($this->menu, "export", $gid, 0, $this->menu[$gid]["items"], $gid,
                                                      $langs_ids);
            $return = array_merge($return, $temp);
        }

        return array("menu" => $return);
    }

    /**
     * Uninstall users_settings data from menu module
     *
     * @return void
     */
    public function deinstall_menu()
    {
        $this->ci->load->helper("menu");

        foreach ($this->menu as $gid => $menu_data) {
            if ($menu_data["action"] == "create") {
                linked_install_set_menu($gid, "delete");
            } else {
                linked_install_delete_menu_items($gid, $this->menu[$gid]["items"]);
            }
        }
    }

    /**
     * Install users_settings data to moderators module
     *
     * @return void
     */
    public function install_moderators()
    {
        // install moderators permissions
        $this->ci->load->model("Moderators_model");

        foreach ((array) $this->moderators as $method_data) {
            $validate_data = array("errors" => array(), "data" => $method_data);
            if (!empty($validate_data["errors"])) {
                continue;
            }
            $this->ci->Moderators_model->save_method(null, $validate_data["data"]);
        }
    }

    /**
     * Import users_settings languages to moderators module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return boolean
     */
    public function install_moderators_lang_update($langs_ids = null)
    {
        $langs_file = $this->ci->Install_model->language_file_read("users_settings", "moderators", $langs_ids);
        if (!$langs_file) {
            log_message("info", "Empty moderators langs data");

            return false;
        }

        // install moderators permissions
        $this->ci->load->model("Moderators_model");
        $params["where"]["module"] = "users_settings";
        $methods = $this->ci->Moderators_model->get_methods_lang_export($params);

        foreach ($methods as $method) {
            if (!empty($langs_file[$method["method"]])) {
                $this->ci->Moderators_model->save_method($method["id"], array(), $langs_file[$method["method"]]);
            }
        }

        return true;
    }

    /**
     * Export users_settings languages from moderators module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return array
     */
    public function install_moderators_lang_export($langs_ids)
    {
        $this->ci->load->model("Moderators_model");
        $params["where"]["module"] = "users_settings";
        $methods = $this->ci->Moderators_model->get_methods_lang_export($params, $langs_ids);
        foreach ($methods as $method) {
            $return[$method["method"]] = $method["langs"];
        }

        return array("moderators" => $return);
    }

    /**
     * Uninstall users_settings data from moderators module
     *
     * @return void
     */
    public function deinstall_moderators()
    {
        ///// delete moderation methods in moderators
        $this->ci->load->model("Moderators_model");
        $params = array();
        $params["where"]["module"] = "users_settings";
        $this->ci->Moderators_model->delete_methods($params);
    }

    /**
     * Install data of users_settings module
     *
     * @return void
     */
    public function _arbitrary_installing()
    {
        $this->ci->pg_module->set_module_config("users", "agent_register_enabled", 1);
        $this->ci->pg_module->set_module_config("users", "agent_register_mail_confirm", 1);
        $this->ci->pg_module->set_module_config("users", "company_register_enabled", 1);
        $this->ci->pg_module->set_module_config("users", "company_register_mail_confirm", 1);
        $this->ci->pg_module->set_module_config("users", "private_register_enabled", 1);
        $this->ci->pg_module->set_module_config("users", "private_register_mail_confirm", 1);
    }

    /**
     * Import languages of users_settings module
     *
     * @return boolean
     */
    public function _arbitrary_lang_install()
    {
        
    }

    /**
     * Export languages of users_settings module
     *
     * @param array $langs_ids languages identifiers
     *
     * @return array
     */
    public function _arbitrary_lang_export($langs_ids = null)
    {
        
    }

    /**
     * Unistall data of users_settings module
     *
     * @return void
     */
    public function _arbitrary_deinstalling()
    {
        $this->ci->pg_module->set_module_config("users", "agent_register_enabled", 0);
        $this->ci->pg_module->set_module_config("users", "agent_register_mail_confirm", 0);
        $this->ci->pg_module->set_module_config("users", "company_register_enabled", 0);
        $this->ci->pg_module->set_module_config("users", "company_register_mail_confirm", 0);
        $this->ci->pg_module->set_module_config("users", "private_register_enabled", 0);
        $this->ci->pg_module->set_module_config("users", "private_register_mail_confirm", 0);
    }

}
